//#ifdef MMAPI_AUDIO
package device;

import java.io.InputStream;

import javax.microedition.media.Manager;
import javax.microedition.media.Player;
import javax.microedition.media.PlayerListener;
import javax.microedition.media.control.VolumeControl;

//import game.PacBowlMIDlet;
import genplay.util.MediaImplementation;
import genplay.util.Media;
import genplay.vp.core.VPHandler;



/**
 * Generic MIDP2.0 Multimedia Engine
 */
public class MIDP20MediaEngine implements MediaImplementation,PlayerListener{

    public static final String IMP_NAME = "midp20";
    public static final int NONE = -1;
    public static final int MIN_TIME_BETWEEN_PLAYS = 2000;
    
	private Player player;
	String[] soundFiles;
	
	Object playLock = new Object();
	Object loadLock = new Object();
	
	int loaded,queueLoad;
	boolean isPlaying;
	boolean firstRun = true;
	boolean forceLoad = false;
	
	long lastPlayTime;
	
	public MIDP20MediaEngine(String[] names) {

		soundFiles = new String[names.length];
		for (int c = 0; c < names.length; c++) {
		    soundFiles[c] = names[c];
		}
		loaded = NONE;
		queueLoad = NONE;
		isPlaying = false;
	}

	private void loadSound(int s) {
	    
	    synchronized  (playLock) {
	        
		    if ((loaded != s && !isPlaying) || forceLoad) {
		        
		        forceLoad = false;
		        
		        if (loaded != NONE) {
		            stopSound(false);
	                if (player != null) {
	    			    player.close();
	    			    player = null;
	    			    loaded = NONE;
	    			}
	    			System.gc();
		        }
		        //System.out.print("Attempting to Load: " + soundFiles[s] + ": "); 
		        InputStream is = getClass().getResourceAsStream(soundFiles[s]);
		        
		        try {
		           
		            
		            //#ifdef mp3
			        if (soundFiles[s].endsWith("mp3")) {
			            //System.out.print("MP3, ");
		                player = Manager.createPlayer(is, "audio/mp3");
			        } else 
			        //#elifdef wav
		            if (soundFiles[s].endsWith("wav")) {
			            //System.out.print("MP3, ");
		                player = Manager.createPlayer(is, "audio/wav");
			        } else   
			        //#endif
			        if (soundFiles[s].endsWith("mid")) {
			            //System.out.print("MIDI, ");
		                player = Manager.createPlayer(is, "audio/midi");
			        } else {
			            ////System.out.print("UNKNOWN TYPE: ");
			        }
			        
			        boolean success = false;			        
				            
			        player.realize();
					player.prefetch();
					player.addPlayerListener(this);
	            
	            
					loaded = s;
					
					//System.out.println("SUCCESS");
					//PacBowlMIDlet.output.append(soundFiles[s] + "(LOAD: SUCCESS)");
					
				} catch (Exception real) {
				    //PacBowlMIDlet.output.append(soundFiles[s] + "(LOAD: "+real+")");
				    //System.out.println("FAILED: " + real);
				} finally {
					try {
						is.close();
					} catch (Exception e) {
						// doesn't matter anyways
					}
				}
	        
				System.gc();
				
		    } else if (isPlaying) {
		        queueLoad = s;
		    }
	    }
	}
	
	public void playSound(int s) {
		//System.out.println("PLAYING:::"+Strings.MEDIA_FILES[s]);
	    if (System.currentTimeMillis() >= lastPlayTime + MIN_TIME_BETWEEN_PLAYS) {    
		    synchronized (playLock) {
		        try {      
		            if (loaded != s) {
		                forceLoad = true;
		                loadSound(s);
		            } else {
		                if (isPlaying) stopSound(false);
		                //System.out.println("PreCached: SUCCESS");
		            }
		            player.start();
		            isPlaying = true;
		            
				} catch (Exception e) {
				    //PacBowlMIDlet.output.append("(PLAY: " + e +")");
					////System.out.println(IMP_NAME+"SndPlay: " + e);
				}
		    }
	    }
	}
	
	public void stopSound(boolean force) {
		try {
		    if (player != null) {
		        isPlaying = false;
		        player.stop();
		    }
		} catch (Exception e) {
		    //PacBowlMIDlet.output.append("(STOP: " + e +")");
			////System.out.println(IMP_NAME+"SndStop: " + e);
		}
		
	}

	public void vibrate(int ms) {
	    VPHandler.display.vibrate(ms);
	}

	public void playerUpdate(Player p, String event, Object data) {
		if (event == PlayerListener.END_OF_MEDIA) {
			stopSound(false);
			if (queueLoad != NONE) {
			    forceLoad = true;
	            loadSound(queueLoad);
	            queueLoad = NONE;
	        }
		}
	}
}
//#endif
