//#ifdef NOKIA_MID_AUDIO
package device;

import java.io.InputStream;

import javax.microedition.media.Manager;
import javax.microedition.media.Player;
import javax.microedition.media.PlayerListener;
import javax.microedition.media.control.VolumeControl;
import com.nokia.mid.ui.DeviceControl;

import genplay.util.MediaImplementation;

/**
 * Media Engine for Nokia Series 60 Version 1.0 handsets
 */
public class Nokia60MediaEngine implements MediaImplementation,PlayerListener{
    
    public static final String IMP_NAME = "midp20";
    public static final int NONE = -1;
    public static final int MIN_TIME_BETWEEN_PLAYS = 2000;
    
    private Player[] players;
    
    int loaded,queueLoad;
    boolean isPlaying;
    boolean firstRun = true;
    boolean forceLoad = false;
    
    long lastPlayTime;
    
    public Nokia60MediaEngine(String[] names) {
		
    	players = new Player[names.length];
		for (int c = 0; c < names.length; c++) {
			try {
				
				InputStream is = getClass().getResourceAsStream(names[c]);
			        
				try {
				    //#ifdef WAV
				    if (names[c].endsWith("wav")) {
						players[c] = Manager.createPlayer(is, "audio/x-wav");
					} else   
					//#endif
				    if (names[c].endsWith("mid")) {
				        players[c] = Manager.createPlayer(is, "audio/midi");
				    } 
				       
				    players[c].realize();
				    players[c].prefetch();
				    players[c].addPlayerListener(this);
				} catch (IllegalStateException e) {
					//ignore
				} finally {
					is.close();
				}
				
			} catch (Exception real) {
			    //PacBowlMIDlet.output.append(soundFiles[s] + "(LOAD: "+real+")");
			}
		}
		loaded = NONE;
		queueLoad = NONE;
		isPlaying = false;
    }
    
    public void playSound(int s) {
		try {
			if (isPlaying) {
				stopSound(true);
			}
			if (players[s] != null) {
				players[s].setMediaTime(0);
			    players[s].start();
			    loaded = s;
				isPlaying = true;  
			}
        } catch (Exception e) {
	    //System.out.println(IMP_NAME+"SndPlay: " + e);
        }	
    }
   
    public void stopSound(boolean force) {
	
		try {
			if (players[loaded] != null) {
				if (players[loaded].getState() == Player.STARTED) {
				    isPlaying = false;
				   	players[loaded].stop();
				}
		    }
		} catch (Exception e) {
		    //System.out.println(IMP_NAME+"SndStop: " + e);
		}
    }
 
    public void vibrate(int ms) {
    	vibrate(ms);
    }
    
    public void playerUpdate(Player p, String event, Object data) {
		synchronized (this) {
		    try {
				if (event == PlayerListener.END_OF_MEDIA) {
				    stopSound(false);
				}
		    } catch (Exception e) {
			//System.out.println(IMP_NAME+"SndUpdt: " + e);
		    }
		}
    }

}
//#endif