package genplay.vp.core;
/* Copyright GenPlay Games, Inc. 2005. 
 * All Rights Reserved 
 * 
 * ----- * PROPRIETARY AND CONFIDENTIAL * -----
 * Created on 4/12/05
 * 
 * Notes:
 * 
 * 7/9/05 - Removed the old notes because they have _nothing_ to do with this
 * Class anymore... 
 */

//#ifdef VENDOR_NOKIA
import com.nokia.mid.ui.*;
	//#define SOFTKEYEMULATION
//#endif
import genplay.vp.game.Game;
import genplay.vp.logi.Strings;
import genplay.vp.logi.TournInfo;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Timer;
import java.lang.Thread;
import java.util.TimerTask;
import java.util.Vector;
import javax.microedition.lcdui.*;
import javax.microedition.rms.RecordStore;

/**
 * @author E Scott Cragg, scragg@genplay.com
 * 
 * GPCanvas is what one might call a 'rennisassance canvas' (or a
 * smart programmer would call a mess).  Currently it is a jack of all
 * trades relating to output, it serves the following purposes:
 * 
 * - Forms a Superclass for all of the Game's Canvases - Including all shared 
 * References/data/constructors
 * 
 * - Manages the application display modes
 * 
 * - Manages all resources for all of the canvases, including its own thread 
 * implementation for a background image loader
 * 
 * - Supplements the 'Graphics' Class with specialty drawing methods for both 
 * Image Fonts and the graphics handler
 * 
 * - Cooks, cleans and does the dishes.
 * 
 * Currently this Class supports:
 * 
 * - Emulated softkeys Vs. System softkey support
 * - Nokia fullcanvas vs regular canvas
 * - Image handling (with optional management on game-state-change)
 * - Image Font Support
 * - Various useful functions/utilities
 * 
 * Supported Precompiler defs (Currently only Antenna)
 * 
 * VENDOR_NOKIA 
 * 		Build for Nokia FullCanvas, autosets SOFTKEYEMULATION 
 * 
 * SOFTKEYEMULATION
 * 		Softkeys are drawn by the application, not the system default
 * 		Requires a MIDP 2.0 or Nokia FullCanvas compliant phone.
 * 
 * Eventually I would like to grow this canvas into a 'gameCanvas' master
 * controlling multiple slave 'screens' that merely have 'paint' and 
 * 'keypressed' event handlers... But that is a job for another day.
 */
public class GPCanvas extends 
//#ifndef VENDOR_NOKIA
Canvas
	//#ifndef SOFTKEYEMULATION
implements CommandListener 
	//#endif
{
//#else
    //#include ${build.file.defs}/etc/fullcanvas.h
    //#endinclude
//#endif
	
	public static final int NONE = -1;
	
	//#ifdef PLACEHOLDER
	/* 
	 * The following values are screen-specific references available to all canvases.
     * 
     * Actual values for each screen size can be found in defs/etc/${build.size.display}.h
     * 
	 */
    
	public static final short SPLASH_TITLE_OFFSET_Y = 0; // Offset from top of main game title
	public static final short SPLASH_TITLE_OFFSET_X = 0;
	public static final short SOFTKEY_FROM_SIDE = 0; // Horizontal Distance from side of Softkey Text
	public static final int SOFTKEY_HEIGHT = 0;		// height allocated to softkeys
    
    // Vertical distances from drawPoint
	public static final short NUM_TOP = 0; // Top of slot number bar
	public static final short NUM_BOTTOM = 0; // Bottom of slot number bar
	public static final short REF_SLOT_Y = 0; // VCENTER card slots
	public static final short REF_ACTION_Y = 0; // VCENTER of hold/badhold/badraw slots
	public static final short REF_BETDRAW_Y = 0; // TOP of Bet Digit Text
	public static final short REF_BETNUM_Y = 0; // VCENTER of BET/COINS/DRAW line
	
	// Horizontal references from sCenter (getWidth/2)
	public static final short[] REF_SLOT_X = {0,0,0,0,0}; // HCENTER of card slots
	public static final short REF_BET_X = 0; // HCENTER of Bet Button
	public static final short REF_DRAW_X = 0; // HCENTER of Draw Button
	public static final short REF_BETNUM_X = 0; // LEFT of Bet Digit Text
	
	// References from HCENTER | VCENTER of each individual card slot
	public static final short REF_CARDNUM_Y = 0; // TOP of Card Numbers
	public static final short REF_CARDNUM_NOWILD_X = 0; // LEFT of Card Number if standard Card
	public static final short REF_CARDNUM_WILD_X = 0; // LEFT of Card Number if Wild Card
	public static final short REF_CARDSUIT_Y = 0; // TOP of card suit if Standard Card
	public static final short REF_CARDSUIT_X = 0; // LEFT of card suit if Standard Card
	public static final short REF_WILD_Y = 0; // BOTTOM of wild image
	public static final short REF_WILD_X = 0;
	public static final short REF_SMALL_SUIT_X = 0; // TOP of card suit if Wild Card
	public static final short REF_SMALL_SUIT_Y = 0; // LEFT of card suit if Wild Card
	
	// Menu Rendering Constants
	public static final short MENU_LIGHT_OFFSET = 0;
	public static final short MENU_TEXT_OFFSET = 0; // Distance of first text line from backTop Image 
	public static final short MENU_TEXT_WIDTH = 0; // Width MenuScreen of select box
	public static final short MENU_LIGHT_WIDTH = 0;	
	// Info Rendering Constants
	public static final short INFO_OFFSET = 0; // Distance of first text line from backTop Image
	public static final short INFO_TEXT_Y_OFFSET = 0;
	
	// Game Rendering Constants
	public static final int GAME_TOPSTRING_OFFSET = 0;
    public static final int GAME_INFOSTRING_OFFSET = 0;
    public static final short GAME_TITLE_HEIGHT = 0;
    public static final short GAME_TITLE_OFFSET_Y = 0;
    public static final short GAME_TITLE_TEXT_OFFSET = 0;

	//#else
	//#include ${build.file.defs.screens}/${build.display}.h
	//#endinclude
	//#endif
	
	/*
     * --------------------- General Declarations -------------------------- 
     */
    // Canvas Modes
    public static final int MODE_CMD_MODIFY = -1;
    public static final int MODE_STARTUP = 0;
    public static final int MODE_LOGO_ANIMATE = 0;
    public static final int MODE_LOGO_PERSISTANT = 1;
    public static final int MODE_SPLASH = 2;
    public static final int MODE_TEXT = 3;
    public static final int MODE_STATIC_MENU = 4;
    public static final int MODE_DYNAMIC_MENU = 5;
    public static final int MODE_DIALOG = 6;
    public static final int MODE_MOD_PAYOFF = 7;
    public static final int MODE_SHOW_PAYOFF = 8;
    public static final int MODE_STRATEGY = 9;
    public static final int MODE_STATISTICS = 10;
    public static final int MODE_GLOSSARY = 11;
    public static final int MODE_GAME = 12;
    
    // Animation Timings
    public static final int GENPLAY_ANIMATE_INTERVAL = 3000;
    public static final int GENPLAY_GLOW_DENOMINATOR = 8;
    public static final int GENPLAY_PERSISTANT_INTERVAL = 1500;
    public static final int SPLASH_PERSISTANT_INTERVAL = 3000;
	public static final int TICK = 100;
	public static final int MENU_LIGHT_INTERVAL = 500;
	
	private static GameScreen gameHandler;

    // Canvas State reference for application
    public static int newState;
    public static int curState;
    public static int canvasMode = MODE_STARTUP;
    public static int animateMode = MODE_STARTUP;
    public static int nextMode = NONE;
    
	public static int MAX_FILESIZE = 5120;
	
	// Command handling
    public static final int CMD_FORE = 0;
    public static final int CMD_BACK = 1;
    public static final int CMD_QUICK = 2;
    public static int[] commands = VPHandler.NOCOMMANDS;
    protected static int[] newCommands = VPHandler.NOCOMMANDS;
    public static int keyFore,keyBack,keyBack2;	// Keycodes to use for actions
    
    // Repaint flags
    public static boolean repaintAnimation;
    public static boolean repaintAll;
    //#ifdef SLOW
    public static boolean needPaintConfirmation = false;
    //#endif

    // Local Display Dimension References
    public static int sHeight;			// Height of Screen
    public static int mHeight;			// Height of main (game) region
    public static int softkeyHeight;	// Height of Softkey region
    public static int sWidth;			// Width of screen
    public static int sCenter;			// Screen centerline
    public static boolean sizeSet = false;
	
    // Image resource manager
    public static final int IMG_DUMP = 0;
	public static final int IMG_LOAD = 2;
	private static long startMem;
	//#ifdef DEBUG
    public static Runtime memCheck = Runtime.getRuntime();
    //#endif
    
    // Be sure to set/unset this on every 'paint'
    public static Graphics cachedGraphics;
    
    // Mirrored locally for Speed Reasons
    public static final int 	TOP = Graphics.TOP;
    public static final int 	BASELINE = Graphics.BASELINE;
    public static final int 	BOTTOM = Graphics.BOTTOM;
    public static final int 	DOTTED = Graphics.DOTTED;
    public static final int 	HCENTER = Graphics.HCENTER;
    public static final int 	LEFT = Graphics.LEFT;
    public static final int 	RIGHT = Graphics.RIGHT;
    public static final int 	SOLID = Graphics.SOLID;
    public static final int 	VCENTER = Graphics.VCENTER;
	
    // App color constants
    public static final int TAN = 0xFFF0D6;
	public static final int BLACK = 0x000000;
    public static final int WHITE = 0xffffff;
    public static final int RED = 0xff0000;
    public static final int GREEN = 0x00AA00; 
    public static final int YELLOW = 0xffff33;
    public static final int ORANGE = 0xf5cf59;  
	public static final int C_BLACK = 0;
	public static final int C_WHITE = 1;
	public static final int C_RED = 2;
	public static final int C_YELLOW = 3;
	public static final int C_GREEN = 4;
	public static final int C_ORANGE = 5;
	public static final int[] fontColors = {BLACK,WHITE,RED,YELLOW,GREEN,ORANGE};

    // Stored Menu Indexes
    public static final byte WINDOW_INDEX = 0;
    public static final byte SELECTED_INDEX = 1;
    public static short[][] refIndexes = new short[VPHandler.NUM_STATES][2];
    
	/*
     * --------------------- Image Handling -------------------------------- 
     * 
     * All images in this application are stored here.
     * 
     * This section is static for speed/memory reasons.
     * 
     * Images are stored and referenced as a bitwise coded integer "ID".  This integer 
     * contains All information necessary for localConfigure, drawString and drawImage to
     * correctly handle rendering of that image resource.
     * 
     * The Bitwise Encoding is as follows-
     * 
     * 0x000000ff - 1 Byte, Image source file Index (into IMAGE_NAMES);
     * 0x0000ff00 - 1 Byte, Number of frames or Special Instructions. If None of these are set, 
     * 				the byte stores the number of frames on the strip
     * 		FRAME_REGULAR - 0x00 - Use the Resource image Verbatim, no special treatment
     * 						A simple IMAGE_NAMES index denotes a regular frame
     * 		FRAME_COMPLEX - 0xff - This image must be specially drawn and contructed from its resource.
     * 						 The Code for this special construction must be placed in handleImages(); 
     * 		FRAME_PALETTE - 0xfe - The normal 'base' image is used with a simple palette shift.
     * 		FRAME_FONT 	- 0xfd - Special image not meant to be drawn... used by drawString instead
     * 0x00ff0000 - 1 Byte, Reference to the source Image ID (only set if not FRAME_REGULAR)
     * 0xff000000 - 1 Byte, The number of this particular frame on the strip (only set if FRAME is not a special)
     * 
     * Each image is indexed by a public static final reference
     */
    
    public static final int FONT = 39; // (FONT is an index to the first fontImage, not its own image)
    public static final String[] IMAGE_NAMES = {
    		"/backgnd0.png",//0
			"/backgnd1.png",//1
			"/backgnd2.png",//2
			"/backgnd3.png",//3
			"/backgnd4.png",//4
			"/backgnd5.png",//5
			"/splash.png",//6
			null, // USED TO BE THE TITLE
			"/splashsubtitle.png",//8
			"/titlelogo.png",//9
			"/copyright.png",//10
			"/menubackgnd.png",//11
			"/menubox.png",//12
			"/menuborder.png",//13
			"/menubottom.png",//14
			"/menulight.png",//15
			"/arrows.png",//16
			"/dot1.png",//17
			"/select0.png",//18
			"/select1.png",//19
			"/suits.png",//20
			"/suits2.png",//21
			"/wild.png",//22
			"/betdigits.png",//23
			"/black.png",//24
			"/slottags.png",//25
			"/cards.png",//26
			"/coins.png",//27
			"/drawbet.png",//28
			"/win_bigwin.png",//29
			"/win_excel.png",//30
			"/win_exquis.png",//31
			"/win_great.png",//32
			"/win_greathand.png",//33
			"/win_jackpot.png",//34
			"/win_lucky.png",//35
			"/win_readyvegas.png",//36
			"/win_topnotch.png",//37
			"/win_winner.png",//38
			"/font11bold.png",//39
			"/font11.png",//40
			"/genplay.png"};//41
    
    // Image Dimensions (Cached for speed, Referenced based on NAME-Index, not ID-Index)
    public static int[] imgWidth = new int[IMAGE_NAMES.length];
    public static int[] imgHeight = new int[IMAGE_NAMES.length];
    
	// Image ID Masks
	public static final int MASK_IMAGE = 	0x000000ff;	
	public static final int MASK_FRAME = 	0x0000ff00;
	public static final int MASK_RES_ID =   0x00ff0000;
	public static final int MASK_FRAMENUM = 0xff000000;
	
	// Image ID Shifts
	public static final int SHIFT_FRAME = 8;
	public static final int SHIFT_RES_ID = 16;
	public static final int SHIFT_FRAMENUM = 24;
	
	// Frame Special Tags
	public static final int FRAME_REGULAR = 0;
	public static final int FRAME_COMPLEX = 0x0ff;
	public static final int FRAME_PALETTE = 0x0fe;
	public static final int FRAME_FONT = 	0x0fd;
	
//	 V- Dynamic Resources -V
	//1-6 are the game backgrounds
	public static final int SPLASHBACK = 6;
	//public static final int TITLE = 7; -- I am _NOT_ renumbering these again... *sigh*
	public static final int SUBTITLE = 8;
	public static final int LOGO = 9;
	public static final int COPYRIGHT = 10;
	public static final int ANIMATELOGO = 11;
	public static final int MBACKGRN = 12;
	public static final int MBACKTOP = 13;
	public static final int M_BORDER = 14;
	public static final int M_BOTTOM = 15;
	public static final int WIN_BIGWIN = 16;
	public static final int WIN_EXCELLENT = 17;
	public static final int WIN_EXQUISITE = 18;
	public static final int WIN_GREAT = 19;
	public static final int WIN_GREATHAND = 20;
	public static final int WIN_JACKPOT = 21;
	public static final int WIN_LUCKYHAND = 22;
	public static final int WIN_READYVEGAS = 23;
	public static final int WIN_TOPNOTCH = 24;
	public static final int WIN_WINNER = 25;
	public static final int NUM_DYNAMIC_IMAGES = 26;	
	// V- Fixed Resources -V
	public static final int M_LIGHT = 26;
	public static final int ARROWS = 30;
	public static final int DOWN = 31;
	public static final int UP = 32;
	public static final int DOT_BOTTOM = 33;
	public static final int DOT_TOP = 34;
	public static final int DOT_RIGHT = 35;
	public static final int DOT_LEFT = 36;
	public static final int SELECTSM = 37;
	public static final int SELECTLG = 38;
	public static final int SUITS = 39;
	public static final int SUITSSM = 40;
	public static final int WILD = 41;
	public static final int BETDIGITS = 42;
	public static final int CARDNUM = 43;
	public static final int BLACKCARDS = 44;
	public static final int REDCARDS = 45;
	public static final int SLOTTAGS = 46;
	public static final int TAG_BADDRAW = 47;
	public static final int TAG_BADHOLD = 48;
	public static final int TAG_HOLD = 49;
	public static final int TAG_DRAW = 50;
	public static final int CARDS = 51;
	public static final int CARD = 52;
	public static final int JOKER = 53;
	public static final int COINS = 54;
	public static final int BUTTONS = 55;
	public static final int BUTTON_DRAW = 56;
	public static final int BUTTON_BET = 57;
	public static final int NUM_STATIC_IMAGES = 58;
	public static final int[] IMAGE_ID_LIST = {
			/*
			 *  						---------------- Dynamic Resources -----------------
			 */
			// V - Game Screen Backgrounds - V
			0,
			1,
			2,
			3,
			4,
			5,	
			// V - Splash - V
			6, 			// Splash background
			NONE,		// USED TO BO THE TITLE!
			8,			// Subtitle
			9,			// Genplay Logo
			10,			// Copyright
			41,			// Animated logo
			// V - Main Menu - V
			11,			// Background
			12,			// Title Box
			13,			// Menu Frame
			14,			// Menu Bottom
			// V - Win Banners -V
			29,			// Big Win!
			30,			// Excellent!
			31,			// Exsquisite!
			32,			// Great!
			33,			// Great Hand!
			34,			// JACKPOT!
			35,			// Lukcy Hand!
			36,			// Ready For Vegas!
			37,			// Top Notch!
			38,			// Winner
			/*
			 * 						---------------- Fixed Resources -----------------
			 */								
			15,
			//(0<<SHIFT_FRAMENUM)|(M_LIGHT<<SHIFT_RES_ID)|4<<SHIFT_FRAME|15,	// Menu Light Bar 1
			(1<<SHIFT_FRAMENUM)|(M_LIGHT<<SHIFT_RES_ID)|4<<SHIFT_FRAME|15, 	// Menu Light Bar 2
			(2<<SHIFT_FRAMENUM)|(M_LIGHT<<SHIFT_RES_ID)|4<<SHIFT_FRAME|15, 	// Menu Light Bar 2
			(3<<SHIFT_FRAMENUM)|(M_LIGHT<<SHIFT_RES_ID)|4<<SHIFT_FRAME|15, 	// Menu Light Bar 2
			16,										// Arrow Resource
			(0<<SHIFT_FRAMENUM)|(ARROWS<<SHIFT_RES_ID)|(2<<SHIFT_FRAME)|16,				// Down Arrow
			(1<<SHIFT_FRAMENUM)|(ARROWS<<SHIFT_RES_ID)|(2<<SHIFT_FRAME)|16,				// Up Arrow
			// 21								V - GamePlay - V
			17, 																		// Bottom Dot Animation
			(0<<SHIFT_FRAMENUM)|(DOT_BOTTOM<<SHIFT_RES_ID)|FRAME_COMPLEX<<SHIFT_FRAME|17,	// Top Dot Animation
			(2<<SHIFT_FRAMENUM)|(DOT_BOTTOM<<SHIFT_RES_ID)|FRAME_COMPLEX<<SHIFT_FRAME|17, 	// Right Dot Animation
			(3<<SHIFT_FRAMENUM)|(DOT_BOTTOM<<SHIFT_RES_ID)|FRAME_COMPLEX<<SHIFT_FRAME|17, 	// Left Dot Animation
			18,			// Small Select Box
			19,			// Large Select Box
			20,			// Large Suits (main)
			21,			// Small Suits (wild)
			22,			// Wild Flag
			23,			// Bet Digits
			24,										// Card Number Resource
			(C_BLACK<<SHIFT_FRAMENUM)|(CARDNUM<<SHIFT_RES_ID)|FRAME_PALETTE<<SHIFT_FRAME|24,// Black Card Numbers
			(C_RED<<SHIFT_FRAMENUM)|(CARDNUM<<SHIFT_RES_ID)|FRAME_PALETTE<<SHIFT_FRAME|24,// Red Card Numbers
			25,										// Slot Tag Resource
			(1<<SHIFT_FRAMENUM)|(SLOTTAGS<<SHIFT_RES_ID)|(4<<SHIFT_FRAME)|25,			// Bad Draw Tag
			(0<<SHIFT_FRAMENUM)|(SLOTTAGS<<SHIFT_RES_ID)|(4<<SHIFT_FRAME)|25,			// Bad Hold Tag
			(2<<SHIFT_FRAMENUM)|(SLOTTAGS<<SHIFT_RES_ID)|(4<<SHIFT_FRAME)|25,			// Hold Tag
			(3<<SHIFT_FRAMENUM)|(SLOTTAGS<<SHIFT_RES_ID)|(4<<SHIFT_FRAME)|25,
			26,										// Cards Resource
			(0<<SHIFT_FRAMENUM)|(CARDS<<SHIFT_RES_ID)|(2<<SHIFT_FRAME)|26,				// Blank Card
			(1<<SHIFT_FRAMENUM)|(CARDS<<SHIFT_RES_ID)|(2<<SHIFT_FRAME)|26,				// Joker
			27,			// Coins box
			28,										// Bet/Draw Resource
			(0<<SHIFT_FRAMENUM)|(BUTTONS<<SHIFT_RES_ID)|(2<<SHIFT_FRAME)|28,				// Draw Button
			(1<<SHIFT_FRAMENUM)|(BUTTONS<<SHIFT_RES_ID)|(2<<SHIFT_FRAME)|28,				// Bet Buttom
	};
	
    // Image Storage structure
    public static Image[] images = new Image[IMAGE_ID_LIST.length];
    
    /*
     * Image Sets are integer arrays of size NUM_DYNAMIC
     * 
     * Requested Actions: (used in the mode Sets)
     * REQ_DUMP		- Image will never be needed again. clear immediately
	 * REQ_NOT_NEEDED - Image is queued for Removal, but keep on if room
	 * REQ_NEEDED 	- Image is required immediately
	 */
    public static final int REQ_DUMP = -1;
    public static final int REQ_NOT_NEEDED = 0;
    public static final int REQ_NEEDED = 1;
    
    
    public static final int[] SPLASH_IMAGE_SET = {
    		REQ_NEEDED,		// BACKIMAGES	// needed for infoCanvas and gameCanvas initializers
			REQ_NOT_NEEDED,	// 1
			REQ_NOT_NEEDED,	// 2
			REQ_NOT_NEEDED,	// 3
			REQ_NOT_NEEDED,	// 4
			REQ_NOT_NEEDED,	// 5
			REQ_NEEDED,		// SPLASHBACK
			REQ_NEEDED,		// TITLE
			REQ_NEEDED,		// SUBTITLE
			REQ_NEEDED,		// LOGO
			REQ_NEEDED,		// COPYRIGHT
			REQ_NEEDED,		// ANIMATELOGO
			REQ_NOT_NEEDED,	// MBACKGRN
			REQ_NEEDED,		// MBACKTOP // needed for infoCanvas and gameCanvas initializers
			REQ_NOT_NEEDED,	// M_BORDER
			REQ_NOT_NEEDED,	// M_BOTTOM
			REQ_NOT_NEEDED,	// WIN_BIGWIN
			REQ_NOT_NEEDED,	// WIN_EXCELLENT
			REQ_NOT_NEEDED,	// WIN_EXQUISITE
			REQ_NOT_NEEDED,	// WIN_GREAT
			REQ_NOT_NEEDED,	// WIN_GREATHAND
			REQ_NOT_NEEDED,	// WIN_JACKPOT
			REQ_NOT_NEEDED,	// WIN_LUCKYHAND
			REQ_NOT_NEEDED,	// WIN_READYVEGAS
			REQ_NOT_NEEDED,	// WIN_TOPNOTCH
			REQ_NOT_NEEDED,	// WIN_WINNER
    };
	
	public static final int[] MENU_IMAGE_SET = {
			REQ_DUMP,	// BACKIMAGES	
			REQ_DUMP,	// 1
			REQ_DUMP,	// 2
			REQ_DUMP,	// 3
			REQ_DUMP,	// 4
			REQ_DUMP,	// 5
			REQ_DUMP,			// SPLASHBACK
			REQ_DUMP,			// TITLE
			REQ_DUMP,			// SUBTITLE
			REQ_DUMP,			// LOGO
			REQ_DUMP,			// COPYRIGHT
			REQ_DUMP,			// ANIMATELOGO
			REQ_NEEDED,		// MBACKGRN
			REQ_NEEDED,		// MBACKTOP
			REQ_NEEDED,		// M_BORDER
			REQ_NEEDED,		// M_BOTTOM
			REQ_DUMP,	// WIN_BIGWIN
			REQ_DUMP,	// WIN_EXCELLENT
			REQ_DUMP,	// WIN_EXQUISITE
			REQ_DUMP,	// WIN_GREAT
			REQ_DUMP,	// WIN_GREATHAND
			REQ_DUMP,	// WIN_JACKPOT
			REQ_DUMP,	// WIN_LUCKYHAND
			REQ_DUMP,	// WIN_READYVEGAS
			REQ_DUMP,	// WIN_TOPNOTCH
			REQ_DUMP,	// WIN_WINNER
	};
	public static boolean needConfirmation = false;
	
	// Mode Image Sets - NOTE: THESE MUST BE SORTED BY imageID INDEX (low->high)
	public static final int[] INFO_IMAGE_SET = {
			REQ_NOT_NEEDED,	// BACKIMAGES	
			REQ_NOT_NEEDED,	// 1
			REQ_NOT_NEEDED,	// 2
			REQ_NOT_NEEDED,	// 3
			REQ_NOT_NEEDED,	// 4
			REQ_NOT_NEEDED,	// 5
			REQ_DUMP,			// SPLASHBACK
			REQ_DUMP,			// TITLE
			REQ_DUMP,			// SUBTITLE
			REQ_DUMP,			// LOGO
			REQ_DUMP,			// COPYRIGHT
			REQ_DUMP,			// ANIMATELOGO
			REQ_NEEDED,		// MBACKGRN
			REQ_NEEDED,		// MBACKTOP
			REQ_NOT_NEEDED,	// M_BORDER
			REQ_NOT_NEEDED,	// M_BOTTOM
			REQ_NOT_NEEDED,	// WIN_BIGWIN
			REQ_NOT_NEEDED,	// WIN_EXCELLENT
			REQ_NOT_NEEDED,	// WIN_EXQUISITE
			REQ_NOT_NEEDED,	// WIN_GREAT
			REQ_NOT_NEEDED,	// WIN_GREATHAND
			REQ_NOT_NEEDED,	// WIN_JACKPOT
			REQ_NOT_NEEDED,	// WIN_LUCKYHAND
			REQ_NOT_NEEDED,	// WIN_READYVEGAS
			REQ_NOT_NEEDED,	// WIN_TOPNOTCH
			REQ_NOT_NEEDED,	// WIN_WINNER
	};
	public static final int[] GAME_IMAGE_SET = {
			REQ_DUMP,	// BACKIMAGES - one of the following will be set to IMAGE_NEEDED by the game initializer	
			REQ_DUMP,
			REQ_DUMP,	
			REQ_DUMP,	
			REQ_DUMP,	
			REQ_DUMP,	
			REQ_DUMP,			// SPLASHBACK
			REQ_DUMP,			// TITLE
			REQ_DUMP,			// SUBTITLE
			REQ_DUMP,			// LOGO
			REQ_DUMP,			// COPYRIGHT
			REQ_DUMP,			// ANIMATELOGO
			REQ_DUMP,	// MBACKGRN
			REQ_DUMP,	// MBACKTOP
			REQ_DUMP,	// M_BORDER
			REQ_DUMP,	// M_BOTTOM
			REQ_NOT_NEEDED,		// WIN_BIGWIN
			REQ_NOT_NEEDED,		// WIN_EXCELLENT
			REQ_NOT_NEEDED,		// WIN_EXQUISITE
			REQ_NOT_NEEDED,		// WIN_GREAT
			REQ_NOT_NEEDED,		// WIN_GREATHAND
			REQ_NOT_NEEDED,		// WIN_JACKPOT
			REQ_NOT_NEEDED,		// WIN_LUCKYHAND
			REQ_NOT_NEEDED,		// WIN_READYVEGAS
			REQ_NOT_NEEDED,		// WIN_TOPNOTCH
			REQ_NEEDED,		// WIN_WINNER
	};

    /*
     * --------------------- Font Handling --------------------------------- 
     * 
     * Because of the need to switch font palettes and such, Fonts are
     * Currently handled separately from other images.
     * 
     */
    public static final int		FONT_SYSTEM = -1;
    public static final int 	FONT_IMAGE = 0;
    public static int fontType = FONT_SYSTEM;
    
	public static final int F_BOLD = 0;
	public static final int F_SMALL = 1;
	public static final String[] fontNames = {"font11bold","font11"};
	public static byte[][] fontByteBuffer;
    public static int[][] fontCharIndexes;
    public static int[][] fontCharWidths;
    
    public static int fontLength;
    public static int startsWith,endsWith;
    
    public static final int BOLD_WHITE = 0;
    public static final int BOLD_RED = 1;
	public static final int BOLD_YELLOW = 2;
	public static final int BOLD_ORANGE = 3;
	public static final int BOLD_BLACK = 4;
	public static final int BOLD_GREEN = 5;
	public static final int SMALL_WHITE = 6;
	public static final int SMALL_RED = 7;
	public static final int SMALL_GREEN = 8;
	public static final int SMALL_BLACK = 9;
	public static final int SMALL_ORANGE = 10;
    public static final int[] FONT_IDS = {
    		(C_WHITE<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|39,
    		(C_RED<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|39,
    		(C_YELLOW<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|39,
    		(C_ORANGE<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|39,
    		(C_BLACK<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|39,
    		(C_GREEN<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|39,
    		(C_WHITE<<SHIFT_FRAMENUM)|(BOLD_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|40,
			(C_RED<<SHIFT_FRAMENUM)|(SMALL_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|40,
			(C_GREEN<<SHIFT_FRAMENUM)|(SMALL_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|40,
			(C_BLACK<<SHIFT_FRAMENUM)|(SMALL_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|40,
			(C_ORANGE<<SHIFT_FRAMENUM)|(SMALL_WHITE<<SHIFT_RES_ID)|(FRAME_FONT<<SHIFT_FRAME)|40,		
    };

    public static Image[] fontImageCache = new Image[FONT_IDS.length];
    
	private static int softkeyFont = BOLD_WHITE;
    
    /*
     * -------------- Canvas Handling ------------------------------------
     * 
     * This is the meat of shared canvas utilities, they include constructors
     * Softkey/Command handling (both SOFTKEYEMULATION and not), and other 
     * misc shared necessities.
     */
    
    public GPCanvas() { }
    
    public GPCanvas(int softKeyHeight, boolean needSize) {
    	gameHandler = new GameScreen();
    	if (softKeyHeight != 0) {
    		//#ifdef SOFTKEYEMULATION
            softkeyHeight = softKeyHeight;
            //#else
            softkeyHeight = 0;
            //#endif
            mHeight = sHeight - softkeyHeight;
            repaintAll = true;
    	}
    	//#ifdef SOFTKEYEMULATION
    		//#ifdef VENDOR_MOTOROLA
    	keyFore = 22;
		keyBack = 21;
    		//#elifdef DEBUG
		keyFore = KEY_NUM6;
    	keyBack = KEY_NUM7;
    		//#else
		keyFore = 6;
		keyBack = 7;
		keyBack2 = 8;
    		//#endif
			//#ifndef VENDOR_NOKIA
			//#ifdef MIDP2.0
		setFullScreenMode(true);
			//#endif
			//#endif
		//#else
		setCommandListener(this);
		//#endif
		if (needSize) sizeChanged(getWidth(),getHeight());
    }
    
	protected void showNotify() {
		
		if (canvasMode == MODE_GAME) {
			
			if (!GameScreen.timerRunning) {
				GameScreen.timerRunning = true;
				if (Game.gState != Game.STATE_PREGAME) {
					GameScreen.timeStartPlay = System.currentTimeMillis();
					if (VPHandler.selectedGame == Game.TOURNAMENT) {
						GameScreen.timeRemaining = TournInfo.nTime - Game.nSeconds+1;
						GameScreen.timeTournyEnd = GameScreen.timeStartPlay + (GameScreen.timeRemaining)*1000;
					}
				} else {
					GameScreen.timeStartPlay = NONE;
				}
			   
			    if (Game.gState != Game.STATE_PREGAME) {
		    		GameScreen.maskCard = new boolean[Game.NUM_CARDS];
		    	}
			    
				if (VPHandler.selectedGame == Game.TOURNAMENT) {
					GameScreen.gameHelper = Game.HELPER_OFF;
				} else {
					Game.gamePlayed[Game.activeGame] = true;
					GameScreen.gameHelper = Game.hSetting;
				}
			}
			
			if (Game.gState <= Game.STATE_BET) {
				GameScreen.cursorPos = GameScreen.CURSOR_BET;
				GameScreen.topString = Strings.BLANK;
				GameScreen.infoStringFont = GameScreen.infoStringNormal;
				for (int c = 0; c < 3; c++) {
				    GameScreen.infoStrings[c] = Strings.STR_GAMESTART[c];
				}
		    }
			
		} 
		
		repaintAll = true;
		repaint();
		
    }
  
	protected void hideNotify() {   
		
		if (canvasMode == MODE_GAME) {
			
			if (GameScreen.timerRunning) {
				GameScreen.timerRunning = false;
				
				Game.saveGame();
			
				if (GameScreen.timeStartPlay != NONE) {
					GameScreen.timeStopPlay = System.currentTimeMillis();
					Game.nSeconds += (GameScreen.timeStopPlay-GameScreen.timeStartPlay)/1000;
				}
				
			    GameScreen.topStringAnimation = GameScreen.OFF;    
			    
			    // If we're in any animation, end it.
			    switch (Game.gState) {
			        case Game.STATE_FINISH:
			        case Game.STATE_BANNER:
			        case Game.STATE_DRAW:
			            GameScreen.cardAnimation = GameScreen.OFF;
			        	GameScreen.maskCard = new boolean[GameScreen.NUM_CARDS];
			            Game.gState = Game.STATE_BET;
			            break;
			        case Game.STATE_DEAL:
			        	GameScreen.cardAnimation = GameScreen.OFF;
			        	GameScreen.maskCard = new boolean[GameScreen.NUM_CARDS];
			            Game.gState = Game.STATE_PICK_CARDS;
			            break;
			    }
			}
		}
		
	}

    protected void sizeChanged(int x, int y) {
    	sHeight = y;
        sWidth = x;
        sCenter = sWidth>>1;
        mHeight = sHeight - softkeyHeight;
        repaintAll = true;
    }
   
//#ifndef SOFTKEYEMULATION
    /*
     * System Softkey registers-
     * 
     * If SOFTKEYEMULATION is not available, the Back and Fore Commands
     * are placed on the Displayable per standard MIDP 1.0 standards.
     * 
     * If the command-action fires, the events are triggered exactly the
     * same way they are triggered via the keyPressed handler.
     */
    public static Command foreCommand;
    public static Command backCommand;
	
    public void commandAction(Command c, Displayable d) {
    	try {
	    	if (!VPHandler.isBusy) {
		    	if (c == foreCommand) VPHandler.event(commands[CMD_FORE], true);
		    	if (c == backCommand) VPHandler.event(commands[CMD_BACK], true);
	    	}
    	} catch (Exception e) {
    		
    	}
    }
//#endif

    protected void keyPressed(int keyCode) {

    	if (!VPHandler.isBusy) {
//#ifdef SOFTKEYEMULATION
    		if (commands == newCommands) {
				if (Math.abs(keyCode) == keyFore && commands[CMD_FORE]!=NONE) {
					VPHandler.event(commands[CMD_FORE], true);
					return;
				}
				if ((Math.abs(keyCode) == keyBack || Math.abs(keyCode) == keyBack2) && commands[CMD_BACK]!=NONE){
					VPHandler.event(commands[CMD_BACK], true);
					return;
				}
    		}
//#endif
		    switch (canvasMode) {
		    case MODE_STARTUP:
			case MODE_TEXT:
			case MODE_STATIC_MENU:
			case MODE_DYNAMIC_MENU:
		    case MODE_DIALOG:
		    	MenuScreen.keyAction(keyCode);
		    	break;
		    case MODE_MOD_PAYOFF:
		    case MODE_SHOW_PAYOFF:
		    case MODE_STRATEGY:
		    case MODE_STATISTICS:
		    case MODE_GLOSSARY:
		    	InfoScreen.keyAction(keyCode);
		    	break;
		    case MODE_GAME:
		    	GameScreen.keyAction(keyCode);
		    	break;
		    }
    	}
    }
    
	protected void paint(Graphics g) {
		
		cachedGraphics = g;
		
		boolean fullRepaint = repaintAll;
		repaintAll = false;
		
		if (fullRepaint) {
			g.setColor(0x000000);
			g.fillRect(0,0,sWidth,sHeight);
		}
			
		if (!VPHandler.isBusy || fullRepaint) { // Repaint as usual
			
			animateMode = canvasMode;

			try {
				
				switch (canvasMode) {
				
					case MODE_STARTUP:
					case MODE_LOGO_PERSISTANT:
					case MODE_SPLASH:
				    case MODE_TEXT:
				    case MODE_STATIC_MENU:
				    case MODE_DYNAMIC_MENU:
				    case MODE_DIALOG:
				    	MenuScreen.draw(g,fullRepaint);
				    	synchronized (VPHandler.screen) { VPHandler.screen.notifyAll(); }; // let animate thread know to start up again
				    	break;
				    case MODE_MOD_PAYOFF:
				    case MODE_SHOW_PAYOFF:
				    case MODE_STRATEGY:
				    case MODE_STATISTICS:
				    case MODE_GLOSSARY:
				    	InfoScreen.draw(g,fullRepaint);
				    	break;
				    case MODE_GAME:
				    	GameScreen.draw(g,fullRepaint);
				    break;
				}
				
			} catch (Exception e) { 
				
				repaintAll = true;
				
			}
			
		} 	
		
		// animate the canvases	
		switch (animateMode) {
		
			case MODE_STARTUP:
			case MODE_LOGO_PERSISTANT:
			case MODE_SPLASH:
		    case MODE_TEXT:
		    case MODE_STATIC_MENU:
		    case MODE_DYNAMIC_MENU:
		    case MODE_DIALOG:
		    	MenuScreen.animate(g);
		    	break;
		    case MODE_GAME:
		    	VPHandler.display.callSerially(gameHandler);
		    	GameScreen.animate(g);
		    break;
		    
		}
		
		if (commands != newCommands || fullRepaint) { // Paint New Softkeys
			if (VPHandler.isBusy) {
				commands = VPHandler.NOCOMMANDS;
			} else {
				commands = newCommands;
//#ifdef DEBUG
				System.out.print("paint: HANDLE SOFTKEYS: ");
				if (commands[CMD_BACK] != NONE) System.out.print("<-" + Strings.COMMANDS[commands[CMD_BACK]>>>8] +" ");
				if (commands[CMD_FORE] != NONE) System.out.print(Strings.COMMANDS[commands[CMD_FORE]>>>8] +"->");
				System.out.println();
//#endif
			}
			
//#ifdef SOFTKEYEMULATION
			g.setColor(BLACK);
			g.setClip(0,mHeight,sWidth,softkeyHeight);
			g.fillRect(0,mHeight,sWidth,softkeyHeight);
    		if (commands[CMD_BACK] != NONE) {
    //#ifdef SWAPSOFT
    			drawString(Strings.COMMANDS[commands[CMD_BACK]>>>8],softkeyFont,sWidth-SOFTKEY_FROM_SIDE,sHeight-1,BOTTOM | RIGHT);
    //#else
    			drawString(Strings.COMMANDS[commands[CMD_BACK]>>>8],softkeyFont,SOFTKEY_FROM_SIDE,sHeight-1,BOTTOM | LEFT);
    //#endif
    		}
    		if (commands[CMD_FORE] != NONE) {
    //#ifdef SWAPSOFT
    			drawString(Strings.COMMANDS[commands[CMD_FORE]>>>8],softkeyFont,SOFTKEY_FROM_SIDE,sHeight-1,BOTTOM | LEFT);
    //#else
    			drawString(Strings.COMMANDS[commands[CMD_FORE]>>>8],softkeyFont,sWidth-SOFTKEY_FROM_SIDE,sHeight-1,BOTTOM | RIGHT);
    //#endif
    		} 
//#else
			try {
				if (commands[CMD_FORE] == NONE || foreCommand.getLabel() != Strings.COMMANDS[commands[CMD_FORE]>>>8]) {
					removeCommand(foreCommand);
					foreCommand = null;
				}
			} catch (Exception e) {
	//#ifdef DEBUG
				System.out.println("SOFTKEY: None Present: FORE");
	//#endif
			} finally {
				if (foreCommand == null && commands[CMD_FORE] != NONE) {
	    			foreCommand = new Command(Strings.COMMANDS[commands[CMD_FORE]>>>8],Command.OK,1);
		    		addCommand(foreCommand);
				}
			}
		
			try {
				if (commands[CMD_BACK] == NONE || backCommand.getLabel() != Strings.COMMANDS[commands[CMD_BACK]>>>8]) {
					System.out.println("REMOVE BACK!"); //XXX
					removeCommand(backCommand);
	    			backCommand = null;
				}
			} catch (Exception e) {
	//#ifdef DEBUG
				System.out.println("SOFTKEY: None Present: BACK");
	//#endif
			} finally {
				if (backCommand == null && commands[CMD_BACK] != NONE) {
					backCommand = new Command(Strings.COMMANDS[commands[CMD_BACK]>>>8],Command.BACK,1);
		    		addCommand(backCommand);
				}
			}
//#endif
		} else {
//#ifdef SOFTKEYEMULATION
			if (commands == null) {
				g.setColor(BLACK);
				g.fillRect(0,mHeight,sWidth,softkeyHeight);
			} else if (commands == VPHandler.NOCOMMANDS) {
				g.setColor(BLACK);
				g.fillRect(0,mHeight,sWidth,softkeyHeight);
			}
//#endif
		}
  
		
		cachedGraphics = null;
		
	}
    
    public static void setMode(int newMode,int[] setCommands,String name,String[] data, int textFileIndex) {
    	
    	if (newMode!=MODE_CMD_MODIFY) {
    		canvasMode = newMode;
	    	// Save our current indexes (window and selected)
    		if (newMode != MODE_STARTUP) {
				refIndexes[curState][WINDOW_INDEX] = (short)MenuScreen.windowIndex;
				refIndexes[curState][SELECTED_INDEX] = (short)MenuScreen.selectedIndex;	
    		}
    		MenuScreen.showDialog = false;
    		switch (newMode) {
				case MODE_STARTUP:
			    case MODE_TEXT:
			    case MODE_STATIC_MENU:
			    case MODE_DYNAMIC_MENU:
			    case MODE_DIALOG:
			    	MenuScreen.mode(newMode,name,data,textFileIndex);
			    	MenuScreen.repaintAllItems = true;	
			    	break;
			    case MODE_MOD_PAYOFF:
			    case MODE_SHOW_PAYOFF:
			    case MODE_STRATEGY:
			    case MODE_STATISTICS:
			    case MODE_GLOSSARY:
			    	MenuScreen.showDialog = false;
			    	InfoScreen.mode(newMode);
			    	break;
			    case MODE_GAME:
			    	MenuScreen.showDialog = false;
			    	GameScreen.gameMode();
			    	break;
    		}
    		
    	}
    	
		newCommands = setCommands;
    	
	}
    
    /*
     * -------------- Static Canvas Handling Methods ------------------------
     * 
     * Initialize does two seperate tasks:
     * 
     * - reads the font data and loads all possible fonts/colors.
     * 
     * - maintains information on all of the images (imgWidth/height) in the application.
	 * 	 This either loads that data from record, or if no record exists, loads all images
	 * 	 and creates that Record.
	 * 
	 * If MEM_LOW is defined, the image array is left null, otherwise, an instance of 
	 * each image is created
	 * 
	 * Note: I am not enforcing an 'initalized' flag, mostly for speed reasons. If you try
	 * and use this class without initializing it, you are dumb and it will break.
	 */
    
	static void getImage(int imageIDIndex) {
		try {
			long startMem = 0;
			//#ifdef Debug
			startMem = memCheck.totalMemory() - memCheck.freeMemory();
			//#endif
			images[imageIDIndex] = Image.createImage(IMAGE_NAMES[IMAGE_ID_LIST[imageIDIndex]&MASK_IMAGE]);;
			if (imgWidth[IMAGE_ID_LIST[imageIDIndex]&MASK_IMAGE] == 0) {
				imgWidth[IMAGE_ID_LIST[imageIDIndex]&MASK_IMAGE] = images[imageIDIndex].getWidth();
				imgHeight[IMAGE_ID_LIST[imageIDIndex]&MASK_IMAGE] = images[imageIDIndex].getHeight();
			}
			//#ifdef DEBUG
    		System.out.println("     NAME:"+IMAGE_NAMES[IMAGE_ID_LIST[imageIDIndex]&MASK_IMAGE]+ " SIZE:" + (memCheck.totalMemory() - memCheck.freeMemory() - startMem));
    		//#endif
		} catch (Exception e) {
			//#ifdef DEBUG
			System.out.println("getImage: " + e);
			//#endif
		}
	}
	
    public static void initialize() {
    	
    	//#ifdef DEBUG
    	System.gc();
    	startMem = memCheck.totalMemory() - memCheck.freeMemory();
    	System.out.println("GPCanvas: Initialize");
    	//#endif
    	
    	byte[] resourceByteBuffer = null;
    	int resourceByteBufferImageID = NONE;
    	//#ifdef DEBUG
    	System.out.println("  initialize: loading Static Images");
    	//#endif
  
    	for (int imageIDIndex = NUM_DYNAMIC_IMAGES; imageIDIndex < NUM_STATIC_IMAGES;imageIDIndex++) {
    		int imageID = IMAGE_ID_LIST[imageIDIndex];
			int imageFileNum = imageID&MASK_IMAGE;
			int imageFrame = (imageID&MASK_FRAME)>>>SHIFT_FRAME;
			int imageResID = (imageID&MASK_RES_ID)>>>SHIFT_RES_ID;
			//#ifdef DEBUG
			System.out.print("   "+imageIDIndex + "- FILE:" + imageFileNum + " FRAME:"+ imageFrame + " RES:"+ imageResID);
			//#endif
			// Load image
			switch (imageFrame) {
			default:
				//#ifdef DEBUG
				System.out.println(" SETTING SPRITE");
				//#endif
				case FRAME_REGULAR: 
					// Standard case, nothing special
					if (images[imageIDIndex] == null) {
						//#ifdef DEBUG
						System.out.println(" LOADING REGULAR. ");
						//#endif					
						getImage(imageIDIndex);
					}
					//#ifdef DEBUG
					else System.out.println();
					//#endif
					break;
					
				case FRAME_PALETTE:
					// Image has swapped Palette
					if (resourceByteBufferImageID != imageFileNum) {
						//#ifdef DEBUG
						System.out.print(" LOADING PALETTE BYTES:");
						//#endif
						resourceByteBufferImageID = imageFileNum;
						resourceByteBuffer = loadBytes(IMAGE_NAMES[imageFileNum]);
					}
					int replaceColor = (imageID&MASK_FRAMENUM)>>>SHIFT_FRAMENUM;
					replaceColor(resourceByteBuffer,1,fontColors[replaceColor]);	
					//#ifdef DEBUG
					System.out.println(" NEW PALETTE IMAGE. ");
					//#endif
					images[imageIDIndex]=Image.createImage(resourceByteBuffer,0,resourceByteBuffer.length);
					break;
					
				case FRAME_COMPLEX:
					//#ifdef DEBUG
					System.out.println(" NEW COMPLEX IMAGE. ");
					//#endif
					// Image is a complex construction
					int dotWidth = imgHeight[IMAGE_ID_LIST[imageResID]&MASK_IMAGE];
					int aDist = dotWidth*2;
					int groupLength = aDist*7;
					Image tmp=null;
					Graphics i;
					switch (imageIDIndex) {
			    		case DOT_TOP:
			    			tmp = Image.createImage(groupLength,dotWidth);
			    			i = tmp.getGraphics();
			    			i.setColor(BLACK);
			    			i.fillRect(0,0,groupLength,dotWidth);
			    			for (int d=1;d<7;d++) {
			    				i.setClip(d*aDist,0,aDist,dotWidth);
			    				i.drawImage(images[imageResID],d*aDist+dotWidth-((6-d)*aDist),0,TOP | LEFT);
			    			}
			    			break;
			    		case DOT_RIGHT:
			    			tmp = Image.createImage(dotWidth,groupLength);
			    			i = tmp.getGraphics();
			    			i.setColor(BLACK);
			    			i.fillRect(0,0,dotWidth,groupLength);
			    			for (int d=0;d<7;d++) {
			    				if (d != 0) i.drawImage(images[imageResID],-(6-d)*aDist,d*aDist,TOP | LEFT);
			    				i.drawImage(images[imageResID],-dotWidth,(d*aDist) + dotWidth,TOP | LEFT);
			    			}
			    			break;
			    		case DOT_LEFT:
			    			tmp = Image.createImage(dotWidth,groupLength);
			    			i = tmp.getGraphics();
			    			i.setColor(BLACK);
			    			i.fillRect(0,0,dotWidth,groupLength);
			    			for (int d=0;d<7;d++) {
			    				if (d != 6) i.drawImage(images[imageResID],-d*aDist,d*aDist,TOP | LEFT);
			    				else i.drawImage(images[imageResID],-dotWidth,d*aDist,TOP | LEFT);
			    				i.drawImage(images[imageResID],-dotWidth,(d*aDist) + dotWidth,TOP | LEFT);
			    			}
			    			break;
			    	}
					images[imageIDIndex] = tmp;
					break;
				
			}		
			System.gc();
    	}
    	
		// Initialize Fonts
    	fontCharIndexes = new int[fontNames.length][];
    	fontCharWidths = new int[fontNames.length][];
    	fontByteBuffer = new byte[fontNames.length][];
    	//#ifdef DEBUG
    	try {   		
    	//#endif
	    	for (int fontType = 0; fontType < fontNames.length; fontType++) { // For each font image
                DataInputStream is = null;
                try {
                	// load font data file
                	is = new DataInputStream(new Object().getClass().getResourceAsStream("/"+fontNames[fontType] +".bin"));
                	int input = (int)is.readByte();
                	input &= 255;
                	input = input | ((int)is.readByte() << 8);
                	startsWith = input;
                	input = (int)is.readByte();
                	input &= 255;
                	input = input | ((int)is.readByte() << 8);
                	endsWith = input;
                	fontLength = endsWith - startsWith;
                	fontCharWidths[fontType] = new int[fontLength];
                    fontCharIndexes[fontType] = new int[fontLength+1];
                    for (int charIndex = 0; charIndex < fontCharIndexes[fontType].length; charIndex++) {
                    	if (charIndex < fontLength) {
	            			input = (int)is.readByte();
	            			input &= 255;
	            			input = input | ((int)is.readByte() << 8);
	                		fontCharIndexes[fontType][charIndex] = input;
	                		if (charIndex > 0) fontCharWidths[fontType][charIndex-1] = fontCharIndexes[fontType][charIndex] - fontCharIndexes[fontType][charIndex-1];
                    	} else {
                    		fontCharIndexes[fontType][fontLength] = imgWidth[FONT+fontType];
                    		fontCharWidths[fontType][charIndex-1] = fontCharIndexes[fontType][charIndex] - fontCharIndexes[fontType][charIndex-1];
                    	}
                    }
                } catch (Exception e) {
                	System.out.println("FONTPARSE: "+ e);
                	e.printStackTrace();
                } finally {
                	try {
                    if (is != null) is.close();
                	} catch (Exception e) {}
                }
	    	
		    	byte[] fontBytes = loadBytes(IMAGE_NAMES[FONT+fontType]);
		    	fontByteBuffer[fontType] = new byte[fontBytes.length];
		    	System.arraycopy(fontBytes,0,fontByteBuffer[fontType],0,fontBytes.length);
		    	Image tmpFontImage = null;
		    	try {
		    		tmpFontImage = Image.createImage(IMAGE_NAMES[FONT+fontType]);
		    		imgWidth[FONT+fontType] = tmpFontImage.getWidth();
					imgHeight[FONT+fontType] = tmpFontImage.getHeight();
		    	} catch (Exception e) { }
	    	}
	    	
	    	System.gc();
	    //#ifdef DEBUG
    	} catch (Exception e) {
    		System.out.println("ERROR LOADING FONT " + e);
    		e.printStackTrace();
        }
    	long curMem = memCheck.totalMemory() - memCheck.freeMemory();
    	System.out.println("initialize: COMPLETE: Memory Used: "+(curMem-startMem)+ ". Total: " + curMem);
    	//#endif
    }
    
    /*
	 * Designed to be called in the 'showNotify' of each and every canvas in the app-
	 * This simply tells the resource manager which images are un-necessary for the given 
	 * context and can be removed from memory.
	 * 
	 * It take two parameters:
	 * needImages - an array with image indexes that will be needed by the canvas/displaymode
	 * 
	 * NOTE! -  needImages MUST BE ARRANGED (low->high) BASED ON imageId INDEX OR THIS WILL FAIL !
	 * 
	 * Notes:
	 * 
	 * - On further thought, I can significantly better this function's run-time if I assume a
	 * sorted (low->high based on imageID index) input list.
	 * 
	 */
	public static void imageHandler(int[] newSet) {
		//#ifdef DEBUG
		startMem = memCheck.totalMemory() - memCheck.freeMemory();
		System.out.println("imageHandler: ");
		try {			
		//#endif
			long timeStart = System.currentTimeMillis();
			// iterate through needimages
			for (int newSetIndex = newSet.length-1; newSetIndex>=0;newSetIndex--) {
				switch (newSet[newSetIndex]) {
					case REQ_DUMP:
						//#ifdef DEBUG
						if (images[newSetIndex]!=null)System.out.println("     " + newSetIndex + "- " + IMAGE_NAMES[IMAGE_ID_LIST[newSetIndex]&MASK_IMAGE] + " DUMPED");
						//#endif
						images[newSetIndex] = null;
						break;
					case REQ_NEEDED:
						if (images[newSetIndex]==null) getImage(newSetIndex);
				}
			}
			System.gc();
		//#ifdef DEBUG
			long curMem = memCheck.totalMemory() - memCheck.freeMemory();
			System.out.println("imageHandler SUCCEEDED: Memory Used:"+(curMem-startMem)+ " Total:" + curMem+ " Time:"+(System.currentTimeMillis()-timeStart) +"ms");
		} catch (RuntimeException e) {
			System.out.println("imageHandler: "+ e);
			e.printStackTrace();
		}
		//#endif
	}
	
    /*
     * -------------- Extensions to Graphics -----------------------------
     * 
     * These functions replace the existing Graphics object functionality.
     * 
     * In order to work, the Graphics object must be cached in cachedGraphics
     * At the beginning of every paint and then make _SURE_ to dereference it 
     * at the end.
     * 
     * All functions here replace their Graphics counterparts (I.e, they update
     * Graphics as needed).
     * 
     * Note- They are all Static for speed reasons (only).  They will operate
     * on the current cachedGraphics.
     */
    
    public static int getStringWidth(int fontType,String toMeasure) {
    	//#ifdef DEBUG
    	try {
    	//#endif
	        int totalWidth = 0;
	        int totalLength = toMeasure.length();
	        for (int stringIndex = totalLength-1; stringIndex >= 0; stringIndex--) {
	        	int drawIndex = (int)toMeasure.charAt(stringIndex) - startsWith;
	            if (0 > drawIndex || drawIndex >= fontCharWidths[fontType].length) {
	                totalWidth += 3; 
	            } else {
	            	totalWidth += fontCharWidths[fontType][drawIndex]+1;
	            }
	        }
	        return totalWidth;
	    //#ifdef DEBUG
    	} catch (Exception e) {
    		System.out.println("getStringWidth: " + e);
    	}
        return 0;
        //#endif
    }

    public static void drawString(String str,int curFont, int drawX, int drawY, int anchor) {
    	int[] oldClip = {cachedGraphics.getClipX(),cachedGraphics.getClipY(),cachedGraphics.getClipWidth(),cachedGraphics.getClipHeight()};
    	try {
    		if (VPHandler.isRunning) {
	    	 	int fontImageID = FONT_IDS[curFont];
	    	 	char[] data = str.toCharArray();
	    	 	int fontImage = fontImageID&MASK_IMAGE;
		    	int fontType = fontImage-FONT;
		    	//#ifdef DEBUG
		    	//System.out.println("Drawing:|"+str+"|: fontImage:" + fontType + " fontImageID:" + curFont);
		        //#endif
		    	int totalWidth = 0;
		        int totalLength = data.length;
		        if ((anchor & LEFT) == 0) {
		        	for (int stringIndex = totalLength-1; stringIndex >= 0; stringIndex--) {
			        	int drawIndex = (int)data[stringIndex] - startsWith;
			        	if (0 > drawIndex || drawIndex >= fontCharWidths[fontType].length) {
			                totalWidth += 3; 
			            } else {
			            	totalWidth += fontCharWidths[fontType][drawIndex]+1;
			            }
			        }
			        if ((anchor & RIGHT) != 0) {
			            drawX -= totalWidth;
			        } else {
			            drawX -= (totalWidth>>1);
			        }  
		        } 
		        if ((anchor & TOP) == 0) {
		        	 drawY -= imgHeight[fontImage];
		        }
		        if (fontImageCache[curFont] == null) {
		        	int replaceColor = (fontImageID&MASK_FRAMENUM)>>>SHIFT_FRAMENUM;
					replaceColor(fontByteBuffer[fontType],1,fontColors[replaceColor]);	
					//#ifdef DEBUG
					System.out.println("Created font Image: " + curFont);
					//#endif
					fontImageCache[curFont]=Image.createImage(fontByteBuffer[fontType],0,fontByteBuffer[fontType].length);
		        }
		        for (int stringIndex = 0; stringIndex < totalLength; stringIndex++) {
		        	int drawIndex = (int)data[stringIndex] - startsWith;
		        	if (0 > drawIndex || drawIndex >= fontCharWidths[fontType].length) {
		        		drawX += 3;
		        	} else {
		        		cachedGraphics.setClip(drawX,drawY,fontCharWidths[fontType][drawIndex],imgHeight[fontImage]);
		        		cachedGraphics.drawImage(fontImageCache[curFont],drawX-fontCharIndexes[fontType][drawIndex],drawY,TOP | LEFT); 
		        		drawX += fontCharWidths[fontType][drawIndex]+1;
		        	}
		        }    
    		}
	     } catch (Exception e) {
	     	// Ignore these exceptions (caused when the imageHandler has switched to a different canvas state before the screen switches)
	     	// XXX this is an awfully big try block... tighten for speed?
	     }      
	     cachedGraphics.setClip(oldClip[0],oldClip[1],oldClip[2],oldClip[3]);
    	
	}
    
    public static void	drawImage(int index, int x, int y, int anchor) {
    	try {
	    	int imageID = IMAGE_ID_LIST[index];  	
	    	int imageFrame = (imageID&MASK_FRAME)>>>SHIFT_FRAME;
	    	//System.out.println("Drawing Image: #" + index + " : " + IMAGE_NAMES[imageID&MASK_IMAGE] + " Frame: "+imageFrame);
	    	if (VPHandler.isRunning) {
		    	switch (imageFrame) {
		    		case FRAME_REGULAR:
		    		case FRAME_PALETTE:
		    		case FRAME_COMPLEX:
	    				if (images[index] == null) {
	    					//#ifdef DEBUG
	    					System.out.println("DRAWIMAGE: EMERGENCY LOAD");
	    					//#endif
	    					getImage(index); 	
	    				}
	    				cachedGraphics.drawImage(images[index],x,y,anchor);
		    		break;
		    		default:
			    		// Sprite case
			    		int imageNum = imageID&MASK_IMAGE;
			    		int imageFrameNum = (imageID&MASK_FRAMENUM)>>SHIFT_FRAMENUM;
			    		int imageWidth = imgWidth[imageNum]/imageFrame;
			    		int imageOffset = imageWidth*imageFrameNum;
			    		int drawX,drawY;
			    		if ((anchor & RIGHT) != 0) {
				            drawX = x - imageWidth;
				        } else if ((anchor & HCENTER) != 0) {
				            drawX = x - (imageWidth>>1);
				        } else {
				        	drawX = x;
				        }
				        if ((anchor & BOTTOM) != 0) {
				        	drawY = y-imgHeight[imageNum];
				        } else if ((anchor & VCENTER) != 0) {
				            drawY = y-(imgHeight[imageNum]>>1);
				        } else {
				            drawY = y;
				        } 
				        cachedGraphics.setClip(drawX,drawY,imageWidth,imgHeight[imageNum]);
				        drawX -= imageOffset;
				        cachedGraphics.drawImage(images[(imageID&MASK_RES_ID)>>SHIFT_RES_ID],drawX,drawY,TOP|LEFT);
			    }
	    	}
    	} catch (Exception e) {
    		// Ignore these exceptions (caused when the imageHandler has switched to a different canvas state before the screen switches)
    		// XXX Large tryblock... tighten for speed?
    	}
    }
    
    
	/*
	 * -------------- Palette Changing Utilities ----------------------------
	 * 
	 * The Following is stolen from Vince's PaletteChanger Class...
	 * 
	 * I've inlined all the crc stuff for speed.
	 * 
	 */
    public static byte[] replaceColor(byte imageBytes[], int id, int rgb) {
    	int nr = (rgb>>16) & 0xff;
    	int ng = (rgb>>8) & 0xff;
    	int nb = (rgb & 0xff);
		int i = 0;
		boolean stop = false;
		while(i < imageBytes.length&& !stop) {
		    if(imageBytes[i] == 0x50) {
		        if(i+3 < imageBytes.length) {
		            if(imageBytes[i+1] == 0x4c && imageBytes[i+2] == 0x54 && imageBytes[i+3] == 0x45) {
		                int length = 0; //get length from bytes preceding chunk header
		                length |= imageBytes[i-1];
		                length |= ((int)imageBytes[i-2]) << 8;

		                int q=0,w=0,e=0;
		                
	                    q=0;w=0;e=0;
	                    q |= (byte)imageBytes[i+4+id*3] & 0x000000ff;
	                    w |= (byte)imageBytes[i+4+id*3+1] & 0x000000ff;
	                    e |= (byte)imageBytes[i+4+id*3+2] & 0x000000ff;

	                    imageBytes[i+4+id*3] = (byte)(nr&0xff);
                        imageBytes[i+4+id*3+1] = (byte)(ng&0xff);
                        imageBytes[i+4+id*3+2] = (byte)(nb&0xff);

                        
		                byte crcbuf[] = new byte[4+length];
		                
		                System.arraycopy(imageBytes, i, crcbuf, 0, 4+length);
		                int len = 4 + length;
		                
		                long x = 0xffffffffL;
		                long[] crc_table = new long[256];
		                long c;
		                int n, k;
		                for ( n = 0; n < 256; n++ )
		                {
		                    c = (long) n;
		                    for ( k = 0; k < 8; k++ )
		                    {
		                        if ( (c & 1) > 0 )
		                            c = 0xedb88320L ^ (c >> 1);
		                        else
		                            c = c >> 1;
		                    }
		                    crc_table[n] = c;
		                }
		                for ( n = 0; n < len; n++ )
		                {
		                    x = crc_table[(int) ((x ^ crcbuf[n]) & 0xff)] ^ (x >> 8);
		                }
		                 
		                long crc = x ^ 0xffffffffL;
		                
	                    imageBytes[i+4+length] = (byte)((crc>>24)& 0xff);
	                    imageBytes[i+4+length+1] = (byte)((crc>>16) & 0xff);
	                    imageBytes[i+4+length+2] = (byte)((crc>>8) & 0xff);
	                    imageBytes[i+4+length+3]  = (byte)(crc & 0xff);
	                    
		                stop = true;
		            }
		            
		        }
		    }
		    i++;
		}
		return imageBytes;
    }

    public static byte[] loadBytes(String name)
	{
	    //System.out.println("LOADIBYTES: " + name);
		try
	    {
	        byte[] array = new byte[MAX_FILESIZE];
	        
	        InputStream is = new Object().getClass().getResourceAsStream(name);
	        
	        int toRead = 0;
	        toRead = is.read(array, 0, MAX_FILESIZE);
	        
	        byte[] toSave = new byte[toRead];
	        System.arraycopy(array, 0, toSave, 0, toRead);
	        
	        //System.out.println("LOADIBYTES: " + toSave.length);
	        return toSave;
	    }
	    catch ( IOException e )
	    {
	        //System.out.println("LOADIBYTES: " + e);
	        e.printStackTrace();
	    }
	    return null;
	    
	}
    
}
