package genplay.vp.core;

import java.util.Enumeration;
import java.util.Vector;
import genplay.vp.logi.*;

import java.io.IOException;
import java.lang.Runnable;
import genplay.vp.game.*;
import genplay.vp.core.*;

import javax.microedition.lcdui.*;

import genplay.util.*;
import genplay.util.Media;


public class GameScreen extends GPCanvas implements Runnable {

    public static final int CMD_PAUSE = 0;
    public static final int CMD_BACK = 1;
    public static final int CMD_STRAT = 2;
    public static final int CMD_PAYOFF = 3;
	
	public static final int NUM_CARDS = 5;
	
	// Animation Timings (all ms)
    public static final int TIME_TICK = 50;
    public static final int TIME_CURSOR_FLASH = 100;
	public static final int TIME_ANIMATION = 50;
	public static final int TIME_CARD_TURNOVER = 150;
	public static final int TIME_TOPSTRING_FLASH = 3000; // How long to display topString instead of info if collapsed
	
	// Garbage Collect every X cycles:
	public static final int GC_COUNT = 50;
	
    public static final int NONE = -1;
    public static final int OFF = -1;
    public static final int START = 0;
    public static final int ON = 1;
    
	public static final byte CURSOR_BET = -1;
	public static final byte CURSOR_DRAW = 5;
	
	public static final int[] flashColors = {0x0000ff,0x00ff00};
	
	public static final byte DOT_ON = 7;

    public static final byte NUM_LIGHTS = 4;
	public static final byte LIGHT_1 = 0;
	public static final byte LIGHT_2 = 1;
	public static final byte LIGHT_3 = 2;
	public static final byte LIGHT_4 = 3;
	public static final byte X = 0;
	public static final byte Y = 1;
	public static final byte STATE = 2;
	public static final byte STEPS = 3;
	public static final byte L_STATE_TOP = 0;
	public static final byte L_STATE_RIGHT = 1;
	public static final byte L_STATE_BOTTOM = 2;
	public static final byte L_STATE_LEFT = 3;
	public static final byte NUM_TOP_LIGHTS = 18;
	public static final byte NUM_SIDE_LIGHTS = 4;
	
	public static final byte COLLAPSE_NONE = 0;
	public static final byte COLLAPSE_TITLE = 1;
	public static final byte COLLAPSE_NUMBERS = 2;
	public static final byte COLLAPSE_TOPSTRING = 3;
	public static final byte COLLAPSE_ALL = 4;
	
	/*
	 * ************************* Canvas Fields ***************************
	 */
	
	private static int titleFont = BOLD_ORANGE;
	private static int coinFont = BOLD_BLACK;
	static int infoStringNormal = BOLD_ORANGE;
	private static int infoStringWin = BOLD_YELLOW;
	private static int infoStringError = BOLD_RED;
	public static int topStringFont;
	static int infoStringFont = infoStringNormal;
	
	public static int backImage;
	
	public static int[] currentImageSet;

	// repaintFlags
	public static boolean repaintCoins = true;
	public static boolean repaintCursor = true;
   	public static boolean repaintInfo = true;
	public static boolean repaintButtons = true;
	public static boolean[] repaintCursorSlot = new boolean[NUM_CARDS];
	public static boolean[] repaintTag = new boolean[NUM_CARDS];
	public static boolean[] repaintCard = new boolean[NUM_CARDS];
	public static boolean[] maskCard = new boolean[NUM_CARDS];
	
	static int whichWin;
    
    // Static display references
	public static int aTop,aSide,aDist,bDist;
    public static int dotWidth;
    private static int wTop, wHeight,reqwHeight,numHeight,drawPoint;
    public static int lightTop,lightLeft,lightBottom,lightRight;
    public static int[][] dotTracker;
    public static int numDots, counter;
    private static int collapseState;
    private static int smOverhang,lgOverhang;
    
    public static boolean bet5errorShown = false;
    
    public static boolean easyCorrect;
    public static boolean showCorrect;
    public static int nextToCorrect;
    
    
    public static int bannerTime;
    
    public static int cursorPos;
    public static boolean cursorState;
   
    public static String[] infoStrings = {"","",""};
    public static String topString= "";
    
    public static boolean winner;

   	public static boolean betChanged;
   	public static int cursorLast;
   	public static int lastState;
   	
   	// For letting the canvas know that there are new info strings
   	public static boolean newInfoStrings;
   	
   	// Times for animation and tracking
   	static long timeStartPlay;
   	public  static long timeStopPlay;
   	static long timeTournyEnd;
   	private static long timeBannerOn;
   	private static long timeLastCard;
   	private static long timeTopString;
   	private static long timeLastCursor;
   	private static long timeLastAnimation;
   	public  static int gcCount = 0;
   	
   	// Current state of animations
   	static int topStringAnimation;
   	static int bannerAnimation;
	static int cardAnimation;
	public static int gameHelper = Game.HELPER_ON;
	
	
	public static Object lock = new Object();
	
	public static boolean timerRunning;
	
	private static int topHintNum;
	private static boolean bannerStarted;
	public  static boolean warningPlayed;
	static long timeRemaining;
	private static boolean gameEnded;
	private static int flashCounter;
	private static final String[] BLANK_INFO_STRINGS = {Strings.BLANK,Strings.BLANK,Strings.BLANK};
	private static long timeLastCountDown;
	private static boolean repaintTopString;

	public static void initialize() {
		wTop = GAME_TITLE_HEIGHT + imgHeight[IMAGE_ID_LIST[0]];
		dotWidth = imgHeight[IMAGE_ID_LIST[DOT_BOTTOM]];
		wHeight = mHeight - wTop;
		reqwHeight = (3*imgHeight[FONT]+1) + (2*dotWidth);
		numHeight = (NUM_BOTTOM - NUM_TOP);
		if (wHeight >= reqwHeight) {
			collapseState = COLLAPSE_NONE;	
			drawPoint=GAME_TITLE_HEIGHT;
		} else if ((wHeight + GAME_TITLE_HEIGHT) >= reqwHeight)  {
			collapseState = COLLAPSE_TITLE;
			wTop -= GAME_TITLE_HEIGHT;
		} else if ((wHeight + numHeight + GAME_TITLE_HEIGHT) >= reqwHeight)  {
			collapseState = COLLAPSE_NUMBERS;
			drawPoint = -1*numHeight;
			wTop -= numHeight + GAME_TITLE_HEIGHT;
		} else if ((wHeight + GAME_TITLE_HEIGHT + NUM_BOTTOM) >= reqwHeight) { 
			collapseState = COLLAPSE_TOPSTRING;
			drawPoint = -1* NUM_BOTTOM;
			wTop -= NUM_BOTTOM + GAME_TITLE_HEIGHT;
		} else {
			collapseState = COLLAPSE_ALL;
			drawPoint = -1*NUM_BOTTOM;
			wTop -= NUM_BOTTOM + GAME_TITLE_HEIGHT;
			dotWidth = 0;
		}
		wHeight = mHeight - wTop;
    	try {
    		aDist = dotWidth*2;
	    	bDist = aDist;
	    	aTop = (sWidth / aDist);
	    	aSide = (mHeight / aDist);
	    	
	    	numDots = 2*aTop + 2*aSide;
	    	
	    	lightTop = wTop;
	    	lightLeft = 0;
	    	lightBottom = mHeight - dotWidth;
	    	lightRight = sWidth - dotWidth;
	    	
    	} catch (Exception e) {
    		// Will only trigger if collapseAll, in that case its irrellivant anyways
    	}
		
    	smOverhang = (imgHeight[IMAGE_ID_LIST[SELECTSM]&MASK_IMAGE]-imgHeight[IMAGE_ID_LIST[TAG_HOLD]&MASK_IMAGE])>>1;
    	lgOverhang = (imgHeight[IMAGE_ID_LIST[SELECTLG]&MASK_IMAGE]-imgHeight[IMAGE_ID_LIST[BUTTON_BET]&MASK_IMAGE])>>1;
	}
	
	public static void gameMode() {
		
		fontImageCache = new Image[FONT_IDS.length];
		
		currentImageSet = new int[GPCanvas.GAME_IMAGE_SET.length];
		System.arraycopy(GPCanvas.GAME_IMAGE_SET,1,currentImageSet,1,GPCanvas.GAME_IMAGE_SET.length-1);
		backImage = Game.tGameDef[Game.DEF_BACKGROUND];
		currentImageSet[backImage] = REQ_NEEDED;
		imageHandler(currentImageSet);
		
		bannerTime = 0;
    	gameEnded = false;
    	warningPlayed = false;
    	cardAnimation = OFF;
    	topStringAnimation = OFF;
    	bannerAnimation = OFF;
    	bet5errorShown = false;
	    
	    cursorPos = (byte)Game.nSelect;	
	    lastState = Game.gState;
		
		if (Game.gState == Game.STATE_CORRECT) {
		    showCorrect = true;
		}
		
		try {
			// Reset our lights
			dotTracker = new int[NUM_LIGHTS][4];
	    	dotTracker[LIGHT_1][X] = sWidth>>1;
	    	dotTracker[LIGHT_1][Y] = lightTop;
	    	dotTracker[LIGHT_1][STATE] = L_STATE_TOP;
	    	dotTracker[LIGHT_1][STEPS] = (sWidth>>1)/aDist;
	    	dotTracker[LIGHT_2][X] = lightRight;
	    	dotTracker[LIGHT_2][Y] = wTop+(wHeight>>1);
	    	dotTracker[LIGHT_2][STATE] = L_STATE_RIGHT;
	    	dotTracker[LIGHT_2][STEPS] = (wHeight>>1)/aDist;
	    	dotTracker[LIGHT_3][X] = sWidth>>1;
	    	dotTracker[LIGHT_3][Y] = lightBottom;
	    	dotTracker[LIGHT_3][STATE] = L_STATE_BOTTOM;
	    	dotTracker[LIGHT_3][STEPS] = (sWidth>>1)/aDist;
	    	dotTracker[LIGHT_4][X] = lightLeft;
	    	dotTracker[LIGHT_4][Y] = wTop+(wHeight>>1);
	    	dotTracker[LIGHT_4][STATE] = L_STATE_LEFT;
	    	dotTracker[LIGHT_4][STEPS] = (wHeight>>1)/aDist;
		} catch (Exception e) {
			
		}
		
		fontImageCache = new Image[FONT_IDS.length];
		
		if (Game.curGameType == Game.TOURNAMENT) timeRemaining = TournInfo.nTime;
	}
	
	public static int nextToCorrect() {
		do {
			nextToCorrect++;
			if (nextToCorrect < CURSOR_DRAW) {
				if (((Game.nHold >>> (Game.DRAW_SHIFT + nextToCorrect)) & 1) != (Game.nHold >>> nextToCorrect & 1)) {
					return nextToCorrect;
				}
			}
		} while (nextToCorrect < CURSOR_DRAW);
		return CURSOR_DRAW;
	}	
	
	public static void keyAction(int keyCode) {
		
		if (Game.gState != Game.STATE_ENDTOURNY) {
			switch (VPHandler.screen.getGameAction(keyCode)) {
				case Canvas.FIRE:
					if (keyCode != Canvas.KEY_NUM5) {
					    switch (Game.gState) {
					        case Game.STATE_PREGAME:
					        	GameScreen.timeStartPlay = System.currentTimeMillis();
					        	if (VPHandler.selectedGame == Game.TOURNAMENT) GameScreen.timeTournyEnd = GameScreen.timeStartPlay + TournInfo.nTime*1000;
							case Game.STATE_BET:
							    Game.gState = Game.STATE_DEAL;
								break;
								
							case Game.STATE_PICK_CARDS:
								if (cursorPos != CURSOR_DRAW) {
								    Game.nHold = Game.nHold^(1 << (cursorPos+Game.DRAW_SHIFT));
									cursorPos++;
									if (cursorPos == CURSOR_DRAW) cursorPos = 0;
									repaintCursor = true;
								} else {
								    Game.gState = Game.STATE_CORRECT;
								}
								break;
								
							case Game.STATE_CORRECT:
								if (cursorPos != CURSOR_DRAW) {
								    Game.nHold ^= (1 << (cursorPos+Game.DRAW_SHIFT));
									Game.nHold &= ~(1 << (cursorPos+Game.HOLDERROR_SHIFT));
									Game.nHold &= ~(1 << (cursorPos+Game.DRAWERROR_SHIFT));
									if (easyCorrect) {
									    cursorPos = nextToCorrect();
									} else {
									    cursorPos++;
									}
									repaintCursor = true;
								} else {
								    Game.gState = Game.STATE_DRAW;
								}
								break;
						default:
						}
					}
					break;
					
				case Canvas.UP:
					if (keyCode != KEY_NUM2) {
						switch (Game.gState) {
						    case Game.STATE_PREGAME:
							case Game.STATE_BET:
								if (Game.nBet < 5) {
									Game.nBet++;
									betChanged = true;
								}
								repaintButtons = true;
								break;
							case Game.STATE_PICK_CARDS:
							case Game.STATE_CORRECT:
								easyCorrect = false;
								if (cursorPos == CURSOR_DRAW) {
									cursorPos = 0;
								}
								repaintCursor = true;
								break;
							default:
						}
					}
					break;
					
				case Canvas.DOWN:
					if (keyCode != KEY_NUM8) {
						switch (Game.gState) {
						    case Game.STATE_PREGAME:
							case Game.STATE_BET:
								if (Game.nBet > 1) {
									Game.nBet--;
									betChanged = true;
								}
								repaintButtons = true;
								break;
							case Game.STATE_PICK_CARDS:
							case Game.STATE_CORRECT:
								easyCorrect = false;
								if (cursorPos != CURSOR_DRAW) {
									cursorPos = CURSOR_DRAW;
								}
								repaintCursor = true;
								break;
							default:
						}
					}
					break;
					
				case Canvas.LEFT:
					if (keyCode != KEY_NUM4) {
						if ((Game.gState == Game.STATE_PICK_CARDS) ||
							(Game.gState == Game.STATE_CORRECT)) {
							easyCorrect = false;
							cursorPos--;
							if (cursorPos == CURSOR_BET) cursorPos = 4;
							repaintCursor = true;
						}
					}
					break;
					
				case Canvas.RIGHT:
					if (keyCode != KEY_NUM6) {
						if (((Game.gState == Game.STATE_PICK_CARDS) ||
							(Game.gState == Game.STATE_CORRECT)) &&
							cursorPos != CURSOR_DRAW) {
							easyCorrect = false;
							cursorPos++;
							if (cursorPos == CURSOR_DRAW) cursorPos = 0;
							repaintCursor = true;
						}
					}
					break;
				}
				
				if ((Game.gState == Game.STATE_CORRECT) ||
					(Game.gState == Game.STATE_PICK_CARDS)) {
				    
				    boolean numPressed = false;
				    
					switch (keyCode) {
					case Canvas.KEY_NUM1:
						if (cursorPos != CURSOR_DRAW) repaintCursorSlot[cursorPos] = true;
						cursorPos = 0;
						repaintCursorSlot[0] = true;
						numPressed = true;
						break;
						
					case Canvas.KEY_NUM2:
						if (cursorPos != CURSOR_DRAW) repaintCursorSlot[cursorPos] = true;
						cursorPos = 1;
						repaintCursorSlot[1] = true;
						numPressed = true;
						break;
						
					case Canvas.KEY_NUM3:
						if (cursorPos != CURSOR_DRAW) repaintCursorSlot[cursorPos] = true;
						cursorPos = 2;
						repaintCursorSlot[2] = true;
						numPressed = true;
						break;
						
					case Canvas.KEY_NUM4:
						if (cursorPos != CURSOR_DRAW) repaintCursorSlot[cursorPos] = true;
						cursorPos = 3;
						repaintCursorSlot[3] = true;
						numPressed = true;
						break;
						
					case Canvas.KEY_NUM5:
						if (cursorPos != CURSOR_DRAW) repaintCursorSlot[cursorPos] = true;
						cursorPos = 4;
						repaintCursorSlot[4] = true;
						numPressed = true;
						break;
						
					case Canvas.KEY_NUM9:
						if (cursorPos != CURSOR_DRAW) repaintCursorSlot[cursorPos] = true;
						cursorPos = CURSOR_DRAW;
						repaintCursor = true;
						timeLastCursor = System.currentTimeMillis();
						Game.gState++;
					}
					
					if (numPressed) {
						repaintCursor = true;
						Game.nHold ^= (1 << (cursorPos+Game.DRAW_SHIFT));
						if (Game.gState == Game.STATE_CORRECT) {
							Game.nHold &= ~(1 << (cursorPos+Game.HOLDERROR_SHIFT));
							Game.nHold &= ~(1 << (cursorPos+Game.DRAWERROR_SHIFT));
						}
					}
					
				} else if (Game.gState == Game.STATE_BET ||
				           Game.gState == Game.STATE_PREGAME){
					if (keyCode == Canvas.KEY_NUM8) {
						repaintCursor = true;
						keyAction(VPHandler.screen.getKeyCode(Canvas.FIRE));
					}
				} 
				
			    if (keyCode == KEY_STAR) {
			    	VPHandler.screen.hideNotify();
			    	VPHandler.event(commands[CMD_STRAT], true);
			    	return;
			    } else if (keyCode == KEY_POUND) {
			    	VPHandler.screen.hideNotify();
			    	VPHandler.event(commands[CMD_PAYOFF], true);
			    	return;
			    }	
		}
    }
	
	public static void animate(Graphics g) {
		
		// what do we need to repaint...
		long curTime = System.currentTimeMillis();
		
		// Cursor?
		if ((timeLastCursor + TIME_CURSOR_FLASH) < curTime) {
			switch (Game.gState) {
    			case Game.STATE_PREGAME:
    			case Game.STATE_BET: 
    			case Game.STATE_PICK_CARDS:
    			case Game.STATE_CORRECT:
    			case Game.STATE_BANNER:
    			case Game.STATE_FINISH:
					flashCounter++;
					flashCounter%=2;
					repaintCursor = true;
			}
		}
		
		if (repaintInfo) repaintTopString = true;
		
		// light Animation?
		if ((timeLastAnimation + TIME_ANIMATION) < curTime) {
			timeLastAnimation = curTime;
			repaintAnimation = true;
			for (int c = 0; c < NUM_LIGHTS; c++) {
				int[] temp = dotTracker[c];
				dotTracker[c][X] = temp[X];
				dotTracker[c][Y] = temp[Y];
				dotTracker[c][STATE] = temp[STATE];
				dotTracker[c][STEPS]++;
				switch (temp[STATE]) {
				case L_STATE_TOP:
					dotTracker[c][X] += aDist;
					if (dotTracker[c][X] >= sWidth-dotWidth) {
						dotTracker[c][STEPS] = 0;
						dotTracker[c][STATE] = L_STATE_RIGHT;
						dotTracker[c][X] = lightRight;
						dotTracker[c][Y] += dotWidth;
					}
					break;
				case L_STATE_RIGHT:
					dotTracker[c][Y] += bDist;
					if (dotTracker[c][Y] >= mHeight-dotWidth) {
						dotTracker[c][STATE] = L_STATE_BOTTOM;
						dotTracker[c][STEPS] = 0;
						dotTracker[c][Y] = lightBottom;
						dotTracker[c][X] = lightRight;
					}
					break;
				case L_STATE_BOTTOM:
					dotTracker[c][X] -= aDist;
					if (dotTracker[c][X] <= dotWidth) {
						dotTracker[c][STATE] = L_STATE_LEFT;
						dotTracker[c][STEPS] = 0;
						dotTracker[c][X] = lightLeft;
					}
					break;
				case L_STATE_LEFT:
					dotTracker[c][Y] -= bDist;
					if (dotTracker[c][Y] <= lightTop+dotWidth) {
						dotTracker[c][STATE] = L_STATE_TOP;
						dotTracker[c][STEPS] = 0;
						dotTracker[c][Y] = lightTop;
						dotTracker[c][X] += dotWidth;
					}
				}
				timeLastAnimation = curTime;
			}
		}
		
		// Cards?
		if (((timeLastCard + TIME_CARD_TURNOVER) < curTime) && (cardAnimation != OFF)) {
			timeLastCard = curTime;
			// If the Animation is just starting-
			if (cardAnimation == START) {	        
				cardAnimation = ON;
			} else {
				int drawCard = NUM_CARDS;
				for (drawCard = 0;drawCard < NUM_CARDS;drawCard++) {
					if (maskCard[drawCard]) break;
				}
				if (drawCard < NUM_CARDS) {
					maskCard[drawCard] = false;
					repaintCard[drawCard] = true;
				} else {
					cardAnimation = OFF;
					Game.gState++;
				}
				timeLastCard = curTime;
				
			} 
		}
	
		// Top String?
		if (topStringAnimation == ON && timeTopString + TIME_TOPSTRING_FLASH < curTime) {
			repaintTopString = true;
			topString = Strings.BLANK;
			topStringAnimation = OFF;
		}
		
		// Countdown Timer?
		if (VPHandler.selectedGame == Game.TOURNAMENT && timeLastCountDown + 1000 < curTime) {
			repaintInfo = true;
		}
		
		// Win Banner?
    	if (bannerAnimation == START) {
			bannerAnimation = ON;
			bannerStarted = true;
			int i = Game.nLastWin / Game.nBet;
			if (i >= Game.JACKPOT_THRESHOLD) {
				//#ifdef smmidi
				Media.soundPlay(Media.SND_WIN5, 10,true);
				//#else
				Media.soundPlay(Media.SND_JACKPOT, 10,true);
				//#endif
				Media.vibratePlay(4000);
			} else if (i >= 9) {
				Media.soundPlay(Media.SND_WIN5, 10,true);
				Media.vibratePlay(1500);
			} else if (i >= 4) {
				//#ifdef smmidi
				Media.soundPlay(Media.SND_WIN5, 10,true);
				//#else
				Media.soundPlay(Media.SND_WIN3, 10,true); 
				//#endif
				Media.vibratePlay(1000);
			} else {
				Media.soundPlay(Media.SND_WIN, 10,true);
				Media.vibratePlay(500);
			}
		} else if (bannerAnimation == ON) {
			g.setColor(flashColors[flashCounter]);
			g.setClip(0,0,sWidth,sHeight);
			g.fillRect(dotWidth,wTop+dotWidth,sWidth-2*dotWidth,wHeight-2*dotWidth);
			int winHeight = imgHeight[IMAGE_ID_LIST[WIN_WINNER]]/10;
			drawImage(whichWin,sCenter,(wTop+(wHeight/2)-(winHeight/2)),HCENTER | VCENTER);
			curTime = System.currentTimeMillis();
			if (bannerStarted) {
				timeBannerOn = curTime;
				bannerStarted = false;
			}
			if (bannerTime + timeBannerOn < curTime) { // Shut off the banner and reset the game
				bannerAnimation = OFF;
				Game.gState = Game.STATE_BET;
                cursorPos = CURSOR_BET;
            	repaintCursor = true;
            	repaintButtons = true;
            	repaintInfo = true;
            	repaintCoins = true;
			}
		}
		

		if (images[backImage] != null) {
			
			if (repaintInfo && bannerAnimation == OFF) {
				g.setColor(0x000000);
				g.fillRect(0,wTop,sWidth,wHeight);
				repaintInfo = false;
				
				// Draw Information Strings
				int drawY = wTop+dotWidth+GAME_INFOSTRING_OFFSET;
				int fontSpacing = (mHeight-dotWidth-drawY)/3;
				
				
				if (topStringAnimation == ON) { 
					drawString(topString,infoStringWin,sCenter,drawY+fontSpacing,BOTTOM | HCENTER);
				} else {
					if (VPHandler.selectedGame == Game.TOURNAMENT) { 
						timeLastCountDown = curTime;
						int time = (int)Math.max(timeRemaining,0);
	    				int hours = time / 3600;
	    				time %= 3600;
	    				int minutes = time / 60;
	    				time %= 60;
	    				String min;
	    				String sec;
	    				if (minutes < 10) {
	    					min =  Strings.TMPLT_1_0 + minutes;
	    				} else if (minutes < 100) {
	    					min = String.valueOf(minutes);
	    				} else {
	    					min = Strings.TMPLT_2_0;
	    				}
	    				if (time< 10) {
	    					sec =  Strings.TMPLT_1_0 + time;
	    				} else if (time < 100) {
	    					sec =  String.valueOf(time);
	    				} else {
	    					sec =  Strings.TMPLT_2_0;
	    				}	
	    				
	    				if (Game.gState != Game.STATE_ENDTOURNY) {
		    				if (Game.gState == Game.STATE_PREGAME) {
		    					infoStrings[0] = Strings.STR_TOURNYSTART_1;
								infoStrings[1] = Strings.STR_TOURNYSTART_2;
							} else if (Game.gState != Game.STATE_ENDTOURNY) {
			    				if (!infoStrings[0].startsWith(Strings.STR_WINNER_PAID)) infoStrings[0] = Strings.BLANK;
			    				infoStrings[1] = Strings.BLANK;
		    				}
							if (hours > 0) {
		    					infoStrings[2] = "Time: " + hours+ Strings.COLON + min + Strings.COLON + sec;
		    				} else {
		    					infoStrings[2] = "Time: " + min + Strings.COLON + sec;
		    				}
	    				}
					}	
					
					if (VPHandler.selectedGame == Game.TOURNAMENT && Game.gState != Game.STATE_ENDTOURNY && Game.gState != Game.STATE_PREGAME) { 
						if (Game.nLastWin > 0 && Game.gState == Game.STATE_BET) drawString(infoStrings[0],infoStringWin,sCenter,drawY,TOP | HCENTER);
						drawY += 2*fontSpacing;
						if (timeRemaining > 30) {
							drawString(infoStrings[2],infoStringNormal,sCenter,drawY,TOP | HCENTER);
						} else {
							drawString(infoStrings[2],infoStringError,sCenter,drawY,TOP | HCENTER);
						}
					} else {
						for (int c =0;c < 3; c++) {
							drawString(infoStrings[c],infoStringFont,sCenter,drawY,TOP | HCENTER);
							drawY += fontSpacing;
						}
					}
				}
			}
			
			// Draw Top String
			if (repaintTopString) {
				repaintTopString = false;
				if (collapseState <= COLLAPSE_NUMBERS) {
					int drawTopStringY = 0;
					if (drawPoint > 0) {
						drawTopStringY = drawPoint;
					}
					g.setClip(0,drawTopStringY,sWidth,NUM_TOP);
					drawImage(backImage,sCenter,drawTopStringY,TOP | HCENTER);
					if (Game.gState <= Game.STATE_BET && gameHelper == Game.HELPER_ON && Game.nLastWin == 0) {
						drawString(Strings.STR_HINT1[topHintNum],topStringFont,sWidth/2,drawTopStringY + NUM_TOP + GAME_TOPSTRING_OFFSET,BOTTOM  | HCENTER);
						topHintNum++;
						if (topHintNum >= Strings.STR_HINT1.length) topHintNum = 0;
					} else {
						drawString(topString,topStringFont,sWidth/2,drawTopStringY + NUM_TOP + GAME_TOPSTRING_OFFSET,BOTTOM  | HCENTER);
					}
				} else if ((topString != Strings.BLANK) && topStringAnimation == OFF) {
					topStringAnimation = ON;
					timeTopString = curTime;
				}
				g.setClip(0,0,sWidth,mHeight);
			}
			
			if (repaintCursor) {
				repaintCursor = false;
				timeLastCursor = curTime;
				if (cursorPos != cursorLast) {
					// If our cursor has moved, removed the old
	    			if (cursorPos != cursorLast) {
	    				if (cursorLast == CURSOR_DRAW) {
	    					g.setClip(sCenter+REF_DRAW_X-lgOverhang,drawPoint+REF_BETDRAW_Y-lgOverhang,imgWidth[IMAGE_ID_LIST[SELECTLG]]/2,imgHeight[IMAGE_ID_LIST[SELECTLG]]);
	    					drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
	    					repaintButtons = true;
	    				} else if (cursorLast == CURSOR_BET) {
	    					g.setClip(sCenter+REF_BET_X-lgOverhang,drawPoint+REF_BETDRAW_Y-lgOverhang,imgWidth[IMAGE_ID_LIST[SELECTLG]]/2,imgHeight[IMAGE_ID_LIST[SELECTLG]]);
	    					drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
	    					repaintButtons = true;
	    				} else {
	    					for (int slotIndex = 0; slotIndex < NUM_CARDS; slotIndex++) {	
	    						if (repaintCursorSlot[slotIndex] || slotIndex == cursorLast) {
	    							repaintCursorSlot[slotIndex] = false;
	    							g.setClip(sCenter+REF_SLOT_X[slotIndex]-smOverhang,drawPoint+REF_ACTION_Y-smOverhang,imgWidth[IMAGE_ID_LIST[SELECTSM]]/2,imgHeight[IMAGE_ID_LIST[SELECTSM]]);
	    							drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
	    							repaintTag[slotIndex] = true;
	    						}
	    					}
	    				}
	    				cursorLast = cursorPos;
	    			}  
				} 
				
				switch (Game.gState) {
					case Game.STATE_PREGAME:
					case Game.STATE_BET: // If we're in BET
						g.setClip(0,0,sWidth,mHeight);
						g.setClip(sCenter+REF_BET_X-lgOverhang,drawPoint+REF_BETDRAW_Y-lgOverhang,imgWidth[IMAGE_ID_LIST[SELECTLG]]>>1,imgHeight[IMAGE_ID_LIST[SELECTLG]]);
						drawImage(SELECTLG,sCenter+REF_BET_X-lgOverhang-(imgWidth[IMAGE_ID_LIST[SELECTLG]]>>1)*flashCounter,drawPoint+REF_BETDRAW_Y-lgOverhang,TOP | LEFT);
						g.setClip(0,0,sWidth,mHeight);
						break;
					
					case Game.STATE_PICK_CARDS:
					case Game.STATE_CORRECT: 
		    			if (cursorPos == CURSOR_DRAW) {
		    				g.setClip(sCenter+REF_DRAW_X-lgOverhang,drawPoint+REF_BETDRAW_Y-lgOverhang,imgWidth[IMAGE_ID_LIST[SELECTLG]]/2,imgHeight[IMAGE_ID_LIST[SELECTLG]]);
		    				drawImage(SELECTLG,sCenter+REF_DRAW_X-lgOverhang-(imgWidth[IMAGE_ID_LIST[SELECTLG]]>>1)*flashCounter,drawPoint+REF_BETDRAW_Y-lgOverhang,TOP | LEFT);
		    			} else {
		    				g.setClip(sCenter+REF_SLOT_X[cursorLast]-smOverhang,drawPoint+REF_ACTION_Y-smOverhang,imgWidth[IMAGE_ID_LIST[SELECTSM]]/2,imgHeight[IMAGE_ID_LIST[SELECTSM]]);
		    				drawImage(SELECTSM,sCenter+REF_SLOT_X[cursorPos]-smOverhang-(imgWidth[IMAGE_ID_LIST[SELECTSM]]>>1)*flashCounter,drawPoint+REF_ACTION_Y-smOverhang,TOP | LEFT);	
		    				repaintTag[cursorPos] = true;
		    			}
		    			g.setClip(0,0,sWidth,mHeight);
		    			break;
						
					default:	
						repaintButtons = true;
					
	    		}
			}
		}
		
		if (repaintButtons) {
			repaintButtons = false;
			switch (Game.gState) {
    			case Game.STATE_PREGAME:
    			case Game.STATE_BET: // If we're in BET
	    			drawImage(BUTTON_BET,sCenter+REF_BET_X,drawPoint+REF_BETDRAW_Y,TOP | LEFT);
    				break;
    					
    			case Game.STATE_PICK_CARDS:
    			case Game.STATE_CORRECT:
	    			drawImage(BUTTON_DRAW,sCenter+REF_DRAW_X,drawPoint+REF_BETDRAW_Y,TOP | LEFT);
    				break;
    		    			
			}
			// draw Bet Digit
    		g.setClip(sCenter+REF_BETNUM_X,drawPoint+REF_BETNUM_Y,imgWidth[IMAGE_ID_LIST[BETDIGITS]]/5,imgHeight[IMAGE_ID_LIST[BETDIGITS]]);
    		drawImage(BETDIGITS,sCenter+REF_BETNUM_X-(Game.nBet-1)*imgWidth[IMAGE_ID_LIST[BETDIGITS]]/5,drawPoint+REF_BETNUM_Y,TOP | LEFT);
    		g.setClip(0,0,sWidth,mHeight);
		}
		
		for (int slotNum = 0; slotNum < NUM_CARDS; slotNum++) {
			// Draw hold/draw/error tags
			if (repaintTag[slotNum]) {	
				repaintTag[slotNum] = false;
				if (showCorrect) {
					if (((Game.nHold >>> (Game.HOLDERROR_SHIFT + slotNum)) & 1) == 1) {
						drawImage(TAG_BADHOLD,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_ACTION_Y,TOP | LEFT);
						continue;
					} else if (((Game.nHold >>> (Game.DRAWERROR_SHIFT + slotNum)) & 1) == 1) {
						drawImage(TAG_BADDRAW,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_ACTION_Y,TOP | LEFT);
						continue;
					} 
				}
				if (((Game.nHold >>> (Game.DRAW_SHIFT + slotNum)) & 1) == 1) {
					drawImage(TAG_HOLD,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_ACTION_Y,TOP | LEFT);
				} else {
					drawImage(TAG_DRAW,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_ACTION_Y,TOP | LEFT);
				}
				
			}
			
			if (Game.gState != Game.STATE_PREGAME) {
			
				// Draw Actual Cards
				if (maskCard[slotNum]) {
					g.setClip(sCenter+REF_SLOT_X[slotNum],drawPoint+REF_SLOT_Y,imgWidth[IMAGE_ID_LIST[CARD]&MASK_IMAGE]>>1,imgHeight[IMAGE_ID_LIST[CARD]&MASK_IMAGE]);
					drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
				} else if (repaintCard[slotNum]) {
					g.setClip(sCenter+REF_SLOT_X[slotNum],drawPoint+REF_SLOT_Y,imgWidth[IMAGE_ID_LIST[CARD]&MASK_IMAGE]>>1,imgHeight[IMAGE_ID_LIST[CARD]&MASK_IMAGE]);
					repaintCard[slotNum] = false;
					if ((Game.wildCard != NONE) &&((Game.deck.cards[slotNum].spot) == Game.wildCard)) { // Draw Wild Card
						drawImage(CARD,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_SLOT_Y,TOP | LEFT);
						drawImage(WILD,sCenter+REF_SLOT_X[slotNum]+REF_WILD_X,drawPoint+REF_SLOT_Y+REF_WILD_Y,TOP | LEFT);
						g.setClip(sCenter+REF_SLOT_X[slotNum]+REF_CARDNUM_WILD_X,drawPoint+REF_SLOT_Y+REF_CARDNUM_Y,imgWidth[IMAGE_ID_LIST[CARDNUM]]/13,imgHeight[IMAGE_ID_LIST[CARDNUM]]);
						int cardOffset = (Game.deck.cards[slotNum].spot - 1) * imgWidth[IMAGE_ID_LIST[CARDNUM]]/13;
						if ((Game.SUIT_HEARTS == Game.deck.cards[slotNum].suit) ||
								(Game.SUIT_DIAMONDS == Game.deck.cards[slotNum].suit)) {
							drawImage(REDCARDS,sCenter+REF_SLOT_X[slotNum]+REF_CARDNUM_WILD_X-cardOffset,drawPoint+REF_SLOT_Y+REF_CARDNUM_Y,TOP | LEFT);
						} else {
							drawImage(BLACKCARDS,sCenter+REF_SLOT_X[slotNum]+REF_CARDNUM_WILD_X-cardOffset,drawPoint+REF_SLOT_Y+REF_CARDNUM_Y,TOP | LEFT);
						}
						g.setClip(sCenter+REF_SLOT_X[slotNum] + REF_SMALL_SUIT_X,drawPoint+REF_SLOT_Y + REF_SMALL_SUIT_Y,imgWidth[IMAGE_ID_LIST[SUITSSM]]>>2,imgHeight[IMAGE_ID_LIST[SUITSSM]]);
						drawImage(SUITSSM,sCenter+REF_SLOT_X[slotNum] + REF_SMALL_SUIT_X - (Game.deck.cards[slotNum].suit * (imgWidth[IMAGE_ID_LIST[SUITSSM]]>>2)),drawPoint+REF_SLOT_Y + REF_SMALL_SUIT_Y, TOP | LEFT);		
					} else if (Game.deck.cards[slotNum].spot != Game.JOKER) { // Draw Normal Card
						drawImage(CARD,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_SLOT_Y,TOP | LEFT);
						g.setClip(sCenter+REF_SLOT_X[slotNum]+REF_CARDNUM_NOWILD_X,drawPoint+REF_SLOT_Y+REF_CARDNUM_Y,imgWidth[IMAGE_ID_LIST[CARDNUM]]/13,imgHeight[IMAGE_ID_LIST[CARDNUM]]);
						int cardOffset = (Game.deck.cards[slotNum].spot - 1) * imgWidth[IMAGE_ID_LIST[CARDNUM]]/13;
						if ((Game.SUIT_HEARTS == Game.deck.cards[slotNum].suit) ||
								(Game.SUIT_DIAMONDS == Game.deck.cards[slotNum].suit)) {
							drawImage(REDCARDS,sCenter+REF_SLOT_X[slotNum]+REF_CARDNUM_NOWILD_X-cardOffset,drawPoint+REF_SLOT_Y+REF_CARDNUM_Y,TOP | LEFT);
						} else {
							drawImage(BLACKCARDS,sCenter+REF_SLOT_X[slotNum]+REF_CARDNUM_NOWILD_X-cardOffset,drawPoint+REF_SLOT_Y+REF_CARDNUM_Y,TOP | LEFT);
						}
						g.setClip(sCenter+REF_SLOT_X[slotNum]+REF_CARDSUIT_X,drawPoint+REF_SLOT_Y+REF_CARDSUIT_Y,imgWidth[IMAGE_ID_LIST[SUITS]]/4,imgHeight[IMAGE_ID_LIST[SUITS]]);
						drawImage(SUITS,sCenter+REF_SLOT_X[slotNum]+REF_CARDSUIT_X - (Game.deck.cards[slotNum].suit * imgWidth[IMAGE_ID_LIST[SUITS]]/4),drawPoint+REF_SLOT_Y+REF_CARDSUIT_Y,TOP | LEFT);						
					} else if (((Game.hand.nWildHold >>> slotNum) & 1) != 0) { // Draw Joker
						drawImage(JOKER,sCenter+REF_SLOT_X[slotNum],drawPoint+REF_SLOT_Y,TOP | LEFT);
					} 	
				}
			} else {
				g.setClip(0,drawPoint+REF_SLOT_Y,sWidth,imgHeight[IMAGE_ID_LIST[CARD]&MASK_IMAGE]);
				drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
			}
		}
			
		// Draw Animation
		if (collapseState != COLLAPSE_ALL && repaintAnimation) {
			g.setColor(0x000000);
			cachedGraphics.setClip(0,wTop,sWidth,wHeight);
			for (int c=0; c<NUM_LIGHTS;c++) {
				switch (dotTracker[c][STATE]) {
				case L_STATE_TOP:
					if (dotTracker[c][STEPS]<7)drawImage(DOT_LEFT,lightLeft,lightTop-dotTracker[c][STEPS]*aDist,TOP | LEFT);
					drawImage(DOT_TOP,dotTracker[c][X],dotTracker[c][Y],TOP | RIGHT);
					break;
				case L_STATE_BOTTOM:
					if (dotTracker[c][STEPS]<7)drawImage(DOT_RIGHT,lightRight,lightBottom+dotTracker[c][STEPS]*aDist+aDist,BOTTOM | LEFT);
					drawImage(DOT_BOTTOM,dotTracker[c][X],dotTracker[c][Y],TOP | LEFT);
					break;
				case L_STATE_RIGHT:
					if (dotTracker[c][STEPS]<7)drawImage(DOT_TOP,lightRight+dotTracker[c][STEPS]*aDist+dotWidth,lightTop,TOP | RIGHT);
					drawImage(DOT_RIGHT,dotTracker[c][X],dotTracker[c][Y],BOTTOM | LEFT);
					break;
				case L_STATE_LEFT:
					if (dotTracker[c][STEPS]<8)drawImage(DOT_BOTTOM,lightLeft-dotTracker[c][STEPS]*aDist,lightBottom,TOP | LEFT);
					drawImage(DOT_LEFT,dotTracker[c][X],dotTracker[c][Y],TOP | LEFT);
				}
			}
		}
		
		
    	if (repaintCoins) {
    		
    		repaintCoins = false;
    		g.setColor(0x000000);
    		g.setClip(0,0,sWidth,mHeight);
    		drawImage(COINS,sCenter-(imgWidth[IMAGE_ID_LIST[COINS]]>>1),drawPoint+REF_BETDRAW_Y,LEFT | TOP);
    		drawString(String.valueOf(Math.min(Game.nCoins,9999)),coinFont,sCenter,drawPoint+REF_BETDRAW_Y+1,HCENTER | TOP);
    		
    	}
    	
	}
    
    public static void draw(Graphics g,boolean repaintAll) {
		
    	if (repaintAll) {
			// do not draw information if banner is on
			repaintCursor = true;
			repaintButtons = true;
			repaintCoins = true;
			for (int slotNum = 0; slotNum < NUM_CARDS; slotNum++) {
				repaintTag[slotNum] = true;
				repaintCard[slotNum] = true;
			}
			commands = null;
			
			// Paint Title, backGround and topString based on the collapse level
			switch (collapseState) {
			case COLLAPSE_NONE:
				drawImage(MBACKTOP,sCenter,GAME_TITLE_HEIGHT,BOTTOM | HCENTER);
				g.setClip(0,0,sWidth,GAME_TITLE_TEXT_OFFSET);
				drawImage(MBACKTOP,sCenter,GAME_TITLE_OFFSET_Y,TOP | HCENTER);
				g.setClip(0,0,sWidth,mHeight);
				drawString(Game.gStrings[Game.STR_NAME][0],titleFont,sCenter,(GAME_TITLE_HEIGHT - GAME_TITLE_TEXT_OFFSET),BOTTOM | HCENTER);
				drawImage(backImage,sCenter,drawPoint, TOP | HCENTER);
				break;
			case COLLAPSE_TITLE:
				drawImage(backImage,sCenter,0,TOP | HCENTER);
				break;
			case COLLAPSE_NUMBERS:
				drawImage(backImage,sCenter,0,TOP | HCENTER);				
				g.setClip(0,NUM_TOP,sWidth,sHeight-NUM_TOP);
				drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
				g.setClip(0,0,sWidth,mHeight);
				break;
			case COLLAPSE_TOPSTRING:
			case COLLAPSE_ALL:
				drawImage(backImage,sCenter,drawPoint,TOP | HCENTER);
			}
    	}
    }

	private boolean showingBetError;

    public void run() {
    	if (VPHandler.isBusy) {
    		return; 
    	} else {
	    	long curTime = System.currentTimeMillis();
	    	
	    	if (VPHandler.selectedGame == Game.TOURNAMENT && Game.gState != Game.STATE_PREGAME) {
	    		timeRemaining = timeTournyEnd - curTime;
				timeRemaining/=1000;
				
		    	if (timeRemaining == 30 && !warningPlayed) {
		    		warningPlayed = true;
		    		Media.soundPlay(Media.SND_WARNING,10,false);
		    	}	
		    	
		    	if (timeRemaining <= 0 && !gameEnded &&
		    		(Game.gState <= Game.STATE_BET || Game.gState == Game.STATE_PICK_CARDS || Game.gState == Game.STATE_CORRECT)) {
		    		gameEnded = true;
		    		Game.gState = Game.STATE_ENDTOURNY;
		    		lastState = Game.STATE_ENDTOURNY;
		    		Media.soundPlay(Media.SND_END,10,false);
		    		hideNotify();
		    		VPHandler.event(VPHandler.ACTION_TOURNAMENT_OVER, true);
		    		infoStrings[0] = Strings.STR_TOURNYEND[0];
		    		infoStrings[1] = Strings.STR_TOURNYEND[1];
		    		infoStrings[2] = Strings.BLANK;
		    		infoStringFont = infoStringError;
		    		repaintAll = true;
		    	}
	    	}
	    	
	    	if (lastState != Game.gState) {
	    		System.gc();
	            switch (lastState) {	
	                
	                case Game.STATE_PREGAME:
	                case Game.STATE_BET:			// (STATE_PREGAME|STATE_BET)->(STATE_DEAL)
	                    //System.out.println("(STATE_PREGAME|STATE_BET)->(STATE_DEAL)");
	                    showCorrect = false;
	                
		                Game.nCoins -= Game.nBet;
		            	Game.nCoinsWon -= Game.nBet;
		            	Game.nCoinsBet += Game.nBet;
		            	Game.nHands++;
			    		
		            	Game.nPayoff = -1;
		            	Game.nHold = 0;
			    		
			    		if (!bet5errorShown && (Game.nBet != Game.MAX_BET)) {
			    			infoStrings[0] = Strings.STR_ERRBET[0];
			    			infoStrings[1] = Strings.STR_ERRBET[1];
			    			infoStrings[2] = Strings.STR_ERRBET[2];
			    			infoStringFont = infoStringError;
			    			bet5errorShown = true;
			    			showingBetError = true;
			    		} else {
			    			infoStrings[0] = Strings.BLANK;
			    			infoStrings[1] = Strings.BLANK;
			    			infoStrings[2] = Strings.BLANK;
			    			infoStringFont = infoStringNormal;
			    		} 
			    			
			    		Game.deckInitialize(Game.wildCard,Game.numJokers,Game.numWilds);
			    		Game.deckShuffle(10);
			    		
			    		topString = Strings.BLANK;
			    		
			    		Game.orderHand();
			    		//System.out.println("(STATE_DEAL) " + Game.hand);
			    		Game.evaluateHand(Game.hand);	// evaluate for PAYOUTs
			    		Game.getStrategy(Game.hand,0,false);
			   
	                    cursorPos=0;
			    		for (int slotNum = 0; slotNum < NUM_CARDS; slotNum++) maskCard[slotNum] = true;
			    		cardAnimation = START;
			    		Media.soundPlay(Media.SND_DEAL5,2,true); // If its 1 or more, play the appropriate sound	
			    		
			    		showCorrect = false;
	                	repaintCoins = true;
	                	repaintCursor = true;
	                	repaintInfo = true;
	                	for (int c = 0; c < NUM_CARDS ; c++) repaintTag[c] = true;
		                break;
	                
	                case Game.STATE_DEAL: 		// (STATE_DEAL)->(STATE_PICK_CARDS)
	                    //System.out.println("(STATE_DEAL)->(STATE_PICK_CARDS)");
	                    if (Game.nPayoff != -1) {
	            			topString=Game.gStrings[Game.STR_PAYOFFS][Game.nPayoff];
	            			Media.soundPlay(Media.SND_PAT, 4,false);	// sound for a paying hand
	            			Media.vibratePlay(400);
	            		} 
	                    if (!showingBetError) {
	                    	showingBetError = false;
	                    	infoStrings[0] = Strings.STR_GAMESELECT[0];
			    			infoStrings[1] = Strings.STR_GAMESELECT[1];
			    			infoStrings[2] = Strings.STR_GAMESELECT[2];
	                    }
	                    repaintInfo = true;
	                    repaintButtons = true;
	                    break;
	                    
	                case Game.STATE_PICK_CARDS:		// (STATE_PICK_CARDS) -> (STATE_CORRECT)
	                    //System.out.println("(STATE_PICK_CARDS) -> (STATE_CORRECT)");
		                int errors;
						errors = (Game.nHold & Game.HOLD_ALL) & ~(Game.nHold >>> Game.DRAW_SHIFT);
						if (errors != 0) {							// we have draw errors (user said draw when it should hold)
						    Game.nHold |= errors << Game.DRAWERROR_SHIFT;
						}
						errors = ~(Game.nHold & Game.HOLD_ALL) & (Game.nHold >>> Game.DRAW_SHIFT);
						if (errors != 0) {								// we have hold errors (user said hold when it should draw)
						    Game.nHold |= errors << Game.HOLDERROR_SHIFT;
						}
	                    if ((Game.nHold & Game.ERROR_ALL) != 0) { // if there any errors			
	    					// recheck to see if another pair or straight is selected and valid
	    					if (Game.RecheckStrategy(Game.hand) == false) {
	    						Game.nErrors += 1;	// add errors to stats too
	    						if (gameHelper != Game.HELPER_OFF) {
	    						    infoStringFont = infoStringError;
		    						infoStrings[0] = Strings.BLANK;
		    						infoStrings[1] = Strings.BLANK;
		    						infoStrings[2] = Strings.BLANK;	
		    						if (Game.nPayoff >= 0 &&  Game.tStrategy[Game.nStrategyLine].type < Game.STRATEGY_START) {	// and we should take the payoff
		    							if (((Game.nHold & Game.HOLD_ALL) & (Game.nHold >>> Game.DRAW_SHIFT)) == (Game.nHold & Game.HOLD_ALL)) {
		    								infoStrings[0] = Strings.STR_ERR5_1;	// do not hold extra cards
		    								infoStrings[1] = Strings.STR_ERR5_2;
		    							} else {
		    								infoStrings[0] = Strings.STR_ERR4_1;	// You missed ....
		    								infoStrings[1] = Game.gStrings[Game.STR_PAYOFFS][Game.nPayoff];
		    							}
		    						} else {	
		    							Hand	held = new Hand();					// new hand of hand cards only
		    							int	nOldLine = Game.nStrategyLine;
		    							int	nOldHold = Game.nHold;	// save old hold values
		    							int	hold = nOldHold >>> Game.DRAW_SHIFT;
		    							int		j;
		
		    							// copy cards hand into new hand
		    							held.nWilds = Game.hand.nWilds;
		    							held.nWildHold = Game.hand.nWildHold;
		    							held.nLast = -1;
		    							held.nPair = -1;
		    							for (int i = 0; i < NUM_CARDS; i++) {
		    								if ((hold & (1 << Game.hand.info[i].card)) != 0) {
		    									j = held.nLast += 1;
		    									held.info[j].spot = Game.hand.info[i].spot;
		    									held.info[j].suit = Game.hand.info[i].suit;
		    									held.info[j].card = Game.hand.info[i].card;
		    									held.info[j].nSpotHold = Game.hand.info[i].nSpotHold & hold;
		    									held.info[j].nSuitHold = Game.hand.info[i].nSuitHold & hold;
		    									held.info[j].nRoyalHold = Game.hand.info[i].nRoyalHold & hold;
		    									held.info[j].nSpots = Game.CountBits(held.info[j].nSpotHold);
		    									held.info[j].nSuits = Game.CountBits(held.info[j].nSuitHold);
		    									held.info[j].nRoyal = Game.CountBits(held.info[j].nRoyalHold);
		    									held.info[j].nRoyalSpots = 0;		// no need to worry about Royals
		    									if (held.info[j].nSpots == 2) {
		    										held.nPair = (short)j;
		    									}
		    								}
		    							}
		    							Game.getStrategy(held, Game.nStrategyLine, false);
		    							if (Game.nStrategyLine == nOldLine) {	// in case they did not hold the wild card but got everything else right
		    								if ((held.nWildHold & hold) != held.nWildHold) {
		    									infoStrings[0] = Strings.STR_ERR6_1;	// always hold wild cards
		    								} else {
		    									infoStrings[0] = Strings.STR_ERR7_1;	// hold these for best chance
		    									infoStrings[1] = Strings.STR_ERR7_2;
		    								}
		    							} else {
		    							    Game.PickHint(nOldLine); 
		    							}
		    							Game.nHold = nOldHold;			// restore these values
		    							Game.nStrategyLine = nOldLine;
		    						}
		    						easyCorrect = true;
		    				    	nextToCorrect = -1;
		    						showCorrect = true;
		    					    cursorPos = nextToCorrect();
		    						Media.soundPlay(Media.SND_ERROR, 5,false);
		    						Media.vibratePlay(500);
		    						repaintCursor = true;
		    						repaintInfo = true;
		    						for (int c = 0; c < NUM_CARDS ; c++) repaintTag[c] = true;
		    						break;
	    						} 
	    					}
	    				} 
	                    Game.gState = Game.STATE_DRAW;
	                case Game.STATE_CORRECT: 		// (STATE_PICK_CARDS|STATE_CORRECT)->(STATE_DRAW)
	                    //System.out.println("(STATE_CORRECT)->(STATE_DRAW)");
	    				Game.deal();
		                Game.orderHand();
		                //System.out.println("(STATE_DRAW) " + Game.hand);
			    		Game.evaluateHand(Game.hand);	// evaluate for PAYOUTs
			    		
			    		infoStrings = BLANK_INFO_STRINGS;
			    		int ticksNeeded = 0;
			    		for (int slotNum = 0; slotNum < NUM_CARDS; slotNum++) {
	    					maskCard[slotNum] = ((Game.nHold >>> (Game.DRAW_SHIFT + slotNum)) & 1) != 1;
	    					if (((Game.nHold >>> (Game.DRAW_SHIFT + slotNum)) & 1) == 1) ticksNeeded++; // Count the number of cards needed
	    				} 
			    		if (ticksNeeded >= 0) Media.soundPlay((4-ticksNeeded),2,true); // If its 1 or more, play the appropriate sound	
			    		cardAnimation = START;
			    		
			    		repaintCursor = true;
						repaintButtons = true;
						repaintInfo = true;
						for (int c = 0; c < NUM_CARDS ; c++) repaintTag[c] = true;
		                break;
	    				
	                case Game.STATE_DRAW:
					    if (Game.nPayoff >= 0) {
						    topString = Game.gStrings[Game.STR_PAYOFFS][Game.nPayoff];
							int i = Game.tPayoff[Game.nPayoff].coins;
							if (Game.nBet != Game.MAX_BET &&  Game.nPayoffType == Game.PAYOUT_ROYAL &&  i > 250) {
								i = 250;					// royal is 250 to 1 if not at max BET
							}
							Game.nHandsWon += 1;
							Game.nWins[Game.nPayoff] += 1;
							Game.nLastWin = i * Game.nBet;
							Game.nCoins += Game.nLastWin;
							Game.nCoinsWon += Game.nLastWin;
							int nTime = 1750;					// time to flash the winner unless changed below
							if (i >= 2)	{					// count good wins in a row
							    Game.nBannerWins += 1;			// NOTE this is not saved in the prefs
							} else {
							    Game.nBannerWins = 0;
							}
							if (i >= Game.JACKPOT_THRESHOLD) {
							    bannerTime = 4000;				// flash jackpot longer
								whichWin = WIN_JACKPOT;
							} else if (i >= 3 &&  ((Game.nHold & Game.DRAWHOLD_ALL) == Game.DRAWHOLD_ALL ||
							        			   (~Game.nHold & Game.DRAWHOLD_ALL) == Game.DRAWHOLD_ALL)){
							    bannerTime = nTime;
							    whichWin = WIN_LUCKYHAND;
							} else if (Game.nBannerWins >= 3) {
							    bannerTime = nTime;
							    whichWin = WIN_READYVEGAS;
							    Game.nBannerWins = 0;			// reset counter
							} else if (i >= 15)	{			// big win
							    bannerTime = nTime;
							    //#ifdef smlow
							    whichWin = WIN_BIGWIN;
							    //#else
							    whichWin = (curTime & 1) == 0 ? WIN_BIGWIN : WIN_EXQUISITE;
							    //#endif
							} else if (i >= 9) {
							    bannerTime = nTime;
							    //#ifdef smlow
							    whichWin =  WIN_EXCELLENT;
							    //#else
							    whichWin = (curTime & 1) == 0 ? WIN_EXCELLENT : WIN_TOPNOTCH;
							    //#endif
							} else if (i >= 4) {
							    bannerTime = nTime;
							    //#ifdef smlow
							    whichWin = WIN_GREATHAND;
							    //#else
							    whichWin = (curTime & 1) == 0 ? WIN_GREAT : WIN_GREATHAND;
							    //#endif
							} else {							// small winner
							    bannerTime = nTime;
							    whichWin = WIN_WINNER;
							}
							bannerAnimation = START;
							infoStringFont = infoStringWin;
							infoStrings[0] = Strings.STR_WINNER_PAID + Game.nLastWin;
							infoStrings[1] = Strings.BLANK;
							if (VPHandler.selectedGame != Game.TOURNAMENT) {
								infoStrings[2] = Strings.STR_GAMESTART[2];
							} else {
								infoStrings[2] = Strings.BLANK;
							}
							repaintCursor = true;
							repaintInfo = true;
							break;
							
						} else {
							Game.nLastWin = 0;
						    infoStringFont = infoStringNormal;
							infoStrings[0] = Strings.BLANK;
							infoStrings[1] = Strings.BLANK;
							infoStrings[2] = Strings.STR_GAMESTART[2];	
						}  
		                Game.gState = Game.STATE_BET;
	                    cursorPos = CURSOR_BET;
	                	repaintCursor = true;
	                	repaintButtons = true;
	                	repaintInfo = true;
	                	repaintCoins = true;
	                    break;
	            }
	            lastState = Game.gState;
	            gcCount++;
	            if (gcCount%GC_COUNT == 0) System.gc();
	        }
	        VPHandler.screen.repaint();
    	}
    }
}
