package genplay.vp.core;

import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.*;

import genplay.vp.logi.*;
import genplay.vp.game.Game;
import genplay.util.*;

public class InfoScreen extends GPCanvas {
   
    // Constants
    public static final int NONE = -1; // A standard accross the entire
                                       // application
    
    public static final int CMD_ABBREV = 2;

    // Various Colors used throughout the canvas
    private static final int SELECT_COLOR = 0x554400; // Color of Select background
    
	// Fonts used by this Canvas
    private static final int titleFont=BOLD_ORANGE;
	private static final int infoFont=BOLD_YELLOW;
	private static final int glossaryFont = BOLD_WHITE;
	private static int highlightFont = SMALL_RED;
	private static int mainFont = SMALL_WHITE;
    
    // Display References
    private static int mTop, mSpacing;

    // Data Registers
    public static String windowInfo;
    private static String[] strings;
    private static int[] values;
    private static int[] modified;
    private static boolean hasData;
    private static int numLines;

    // Window Scrolling
    public static int windowIndex;
    public static int selectIndex;
    private static int showAtOnce;
    private static int highlightOffset;

    // a quick and dirty way to send commands to the main handler
    public static Command quickCommand;
	
    /* V-- Constructors --V */
    public static void initialize() {
    	mTop = imgHeight[IMAGE_ID_LIST[MBACKTOP]] + INFO_OFFSET + imgHeight[IMAGE_ID_LIST[ARROWS]];
        showAtOnce = (mHeight-mTop-imgHeight[IMAGE_ID_LIST[ARROWS]])/(imgHeight[FONT]+1);
        mSpacing = (mHeight - mTop-imgHeight[IMAGE_ID_LIST[ARROWS]])/showAtOnce;
        highlightOffset = ((imgHeight[FONT]-mSpacing)>>1)-1;
    }
    
    public static void mode(int purpose) {
       
    	imageHandler(GPCanvas.INFO_IMAGE_SET);
    	
        hasData=false;
        values=null;
        strings=null;
        
        switch (purpose) {
            case MODE_SHOW_PAYOFF:
            	selectIndex = NONE;
            case MODE_MOD_PAYOFF:
                if (purpose!=MODE_SHOW_PAYOFF) selectIndex = 0;
                windowIndex = 0;
                numLines = Game.gStrings[Game.STR_PAYOFFS].length;
                strings = new String[numLines];
                values = new int[numLines];
                for (int c = 0; c < numLines; c++) {
                    strings[c] = Game.gStrings[Game.STR_PAYOFFS][c];
                }
                for (int c = 0; c < numLines; c++) {
                    values[c] = Game.tPayoff[c].coins;
                }
                
                int tmp = Game.tPayBack[1] / 10;
                String tail;
                if (tmp < 10) {
        			tail =   Strings.TMPLT_1_0 + tmp;
        		} else if (tmp < 100) {
        			tail =  String.valueOf(tmp);
        		} else {
        			tail = Strings.TMPLT_2_0;
        		}
                windowInfo = Strings.STR_PAYBACK + Game.tPayBack[Game.PAY_WHOLE] + Strings.DECIMAL + tail + Strings.PERCENT;
                
                hasData = true;
                modified = new int[numLines];
                break;
                
            case MODE_STATISTICS:
                windowInfo = "";
                windowIndex = 0;
                selectIndex = NONE;
                int headLength = Strings.STAT_HEADINGS.length-1;
                numLines = headLength + Game.gStrings[Game.STR_PAYOFFS].length;
                strings = new String[numLines];
                values = new int[numLines];
                
                for (int c = 1; c < Strings.STAT_HEADINGS.length; c++) {
                    strings[c-1] = Strings.STAT_HEADINGS[c];
                }
                for (int c = 0; c < Game.gStrings[Game.STR_PAYOFFS].length; c++) {
                    strings[c+headLength] = Game.gStrings[Game.STR_PAYOFFS][c];
                }
                
    			int value;
    			
    			if (Game.nCoinsBet > 10000000) {	// prevent overflow
    				value = (int)(Game.nCoinsWon) * 100 / (int)(Game.nCoinsBet / 100);
    			} else if (Game.nCoinsBet > 1000000) {	// prevent overflow
    				value = (int)(Game.nCoinsWon) * 1000 / (int)(Game.nCoinsBet / 10);
    			} else if (Game.nCoinsBet > 0) {
    				value = (int)(Game.nCoinsWon) * 10000 / (int)Game.nCoinsBet;
    			} else {
    				value = 0;
    			}
    			
    			value += 10000;							// 100%
                int input = value%100;
				if (input < 10) {
					tail =  Strings.TMPLT_1_0 + input;
				} else if (input < 100) {
				    tail =  String.valueOf(input);
				} else {
				    tail =  Strings.TMPLT_2_0;
				}
				windowInfo = Strings.STAT_HEADINGS[0].substring(1) + value/100+Strings.DECIMAL+ tail + Strings.PERCENT;
    			
				modified = new int[numLines];
				values[0]=Game.nCoins;
    			values[1]=Math.abs(Game.nCoinsWon);
    			if (Game.nCoinsWon != 0) modified[1] = (byte)(Game.nCoinsWon / Math.abs(Game.nCoinsWon));
    			values[2]=Game.nHands;
    			values[3]=Game.nHandsWon;
    			values[4]=Game.nErrors;
    			values[5]=Game.nSeconds;
    			
    			if (Game.nSeconds == 0) {
    			    values[6] = 0;
    			} else {
    			    values[6] = Game.nHands * 3600 / Game.nSeconds;
    			}
    			
    			for (int c = 0; c < Game.nWins.length; c++) {
    			    values[c+7] = Game.nWins[c];
    			}
                
                hasData = true;
                break;
                
            case MODE_STRATEGY:
                numLines = Game.gStrings[Game.STR_STRATEGIES].length;
                strings = new String[numLines];
                for (int c = 0; c < numLines; c++) {
                    strings[c] = Game.gStrings[Game.STR_STRATEGIES][c];
                }
		        
		        hasData = true;
		        windowInfo = Strings.BLANK;
                if (Game.nStrategyLine != 0) {
		            selectIndex = Game.nStrategyLine;
		            if ((selectIndex - showAtOnce / 2 + showAtOnce) <= strings.length) {
		                windowIndex = selectIndex - showAtOnce / 2;
		                if (windowIndex < 0) {
		                    windowIndex = 0;
		                }
		            } else {
		                windowIndex = strings.length - 1 - showAtOnce;
		            }
		        }
                hasData = true;
                modified = new int[numLines];
                break;
                
            case MODE_GLOSSARY:
                numLines = Strings.GAME_GLOSSARY.length;
                strings = new String[numLines];
                for (int c = 0; c < numLines; c++) {
                    strings[c] = Strings.GAME_GLOSSARY[c];
                }
                windowInfo = Strings.STR_ABBREV;
                windowIndex = 0;
                hasData = true;
                modified = new int[numLines];
                break;
        }
        fontImageCache = new Image[FONT_IDS.length];
    }
    
    /* V-- Canvas Related Operation --V */
    
    protected static void keyAction(int keyCode) {
    	int drawTill;
    	switch (canvasMode) {
	        case MODE_GLOSSARY:
	        case MODE_STRATEGY:
	            drawTill = showAtOnce;
	    		break;
	    	default:
	            drawTill = showAtOnce-1;
	    }
    
    	switch (VPHandler.screen.getGameAction(keyCode)) {
            case Canvas.UP:
                if (canvasMode == MODE_MOD_PAYOFF) {
                    if (selectIndex > 0) {
                        selectIndex--;
                    }
                    if (selectIndex < windowIndex) {
                        windowIndex--;
                    }
                } else {
                    if (windowIndex > 0) {
                        windowIndex--;
                    }
                }
                break;
            case Canvas.DOWN:
                if (canvasMode == MODE_MOD_PAYOFF) {
                    if (selectIndex + 1 < numLines) {
                        selectIndex++;
                    }
                    if (selectIndex + 1 > windowIndex + drawTill) {
                        windowIndex++;
                    }
                } else {
                    if ((numLines > drawTill) && (windowIndex+drawTill < numLines)) {
                        windowIndex++;
                    }
                }
                break;
            case Canvas.RIGHT:
                if (canvasMode == MODE_MOD_PAYOFF) {
                    if (values[selectIndex] < 0x4000000) values[selectIndex] += Game.tChanges[selectIndex][Game.CH_AMOUNT];
                    
                    Game.tPayoff[selectIndex].coins = (short)values[selectIndex];
                    modified[selectIndex]++;
                    
                    
                    
                    // Modify InfoString
                    Game.tPayBack[Game.PAY_FRAC] += Game.tChanges[selectIndex][Game.CH_FRAC];
                    if (Game.tPayBack[Game.PAY_FRAC] / 1000 > 0) {
                        Game.tPayBack[Game.PAY_WHOLE]++;
                        Game.tPayBack[Game.PAY_FRAC] %= 1000;
                    }
                    Game.tPayBack[Game.PAY_WHOLE] += Game.tChanges[selectIndex][Game.CH_WHOLE];
                    
                    int tmp= Game.tPayBack[Game.PAY_FRAC]/10;
                    String tail;
                    if (tmp < 10) {
            			tail =   Strings.TMPLT_1_0 + tmp;
            		} else if (tmp < 100) {
            			tail =  String.valueOf(tmp);
            		} else {
            			tail = Strings.TMPLT_2_0;
            		}
                    windowInfo = Strings.STR_PAYBACK + Game.tPayBack[Game.PAY_WHOLE] + Strings.DECIMAL + tail + Strings.PERCENT; 
                    if (getStringWidth(F_BOLD,windowInfo) > sWidth) windowInfo = "Bored yet?";
                }
                break;
            case Canvas.LEFT:
                if (canvasMode == MODE_MOD_PAYOFF 
						&& (values[selectIndex] - Game.tChanges[selectIndex][Game.CH_AMOUNT]) >= 0) {
                    // Change our Payoff
                    values[selectIndex] -= Game.tChanges[selectIndex][Game.CH_AMOUNT];
                    Game.tPayoff[selectIndex].coins = (short)values[selectIndex];
                    modified[selectIndex]--;
                    // Modify InfoString
                    Game.tPayBack[Game.PAY_FRAC] -= Game.tChanges[selectIndex][Game.CH_FRAC];
                    if (Game.tPayBack[Game.PAY_FRAC] < 0) {
                        Game.tPayBack[Game.PAY_WHOLE]--;
                        Game.tPayBack[Game.PAY_FRAC] += 1000;
                    }
                    Game.tPayBack[Game.PAY_WHOLE] -= Game.tChanges[selectIndex][Game.CH_WHOLE];
                    
                    int tmp = Game.tPayBack[Game.PAY_FRAC]/10;
                    String tail;
                    if (tmp < 10) {
            			tail =   Strings.TMPLT_1_0 + tmp;
            		} else if (tmp < 100) {
            			tail =  String.valueOf(tmp);
            		} else {
            			tail = Strings.TMPLT_2_0;
            		}
                    windowInfo = Strings.STR_PAYBACK + Game.tPayBack[Game.PAY_WHOLE] + Strings.DECIMAL + tail + Strings.PERCENT;
                }
    	}
    
    
    	VPHandler.screen.repaint();

    }

    public short[] getData() {

        short[] toReturn = new short[values.length];
        for (int c = 0; c < values.length; c++) {
            toReturn[c] = (short) values[c];
        }
        return toReturn;
    }
    
    public static void draw(Graphics g,boolean repaintAll) {
    	
    	g.setClip(0,0,sWidth,mHeight);
    	
        if (repaintAll) {
        	
        	// Draw Background
        	repaintAll = false;
        	commands = null; // flag to repaint the commands
	    	g.setColor(0x000000);
	        g.fillRect(0, 0,sWidth,sHeight);
	    	drawImage(MBACKTOP,sCenter,0,HCENTER | TOP);
	    	drawImage(MBACKGRN,sCenter,(mTop+mHeight)>>1,HCENTER | VCENTER);
	    	if (canvasMode != MODE_GLOSSARY) {
	            drawString(Game.gStrings[Game.STR_NAME][0],titleFont,sCenter,(imgHeight[IMAGE_ID_LIST[MBACKTOP]]>>1)+(imgHeight[IMAGE_ID_LIST[FONT]]>>1),HCENTER | BOTTOM);
	        } else {
	            drawString(Strings.STR_ABBREV,titleFont,sCenter,(imgHeight[IMAGE_ID_LIST[MBACKTOP]]>>1)+(imgHeight[FONT]>>1),HCENTER | BOTTOM);
	        }
	    	
        }
    	
        if (hasData) {
            // create local references to these variables in case ABBREV needs
            // to take them over
        	int drawTill;
            int dataTop;
            
            g.setClip(0,0,sWidth,mHeight);
            g.setColor(0x000000);
            g.fillRect(0,imgHeight[IMAGE_ID_LIST[MBACKTOP]],sWidth,mHeight-imgHeight[IMAGE_ID_LIST[MBACKTOP]]);
            drawImage(MBACKGRN,sCenter,(mTop+mHeight)>>1,HCENTER | VCENTER);
            
            switch (canvasMode) {
	            case MODE_GLOSSARY:
	            case MODE_STRATEGY:
	            	// Use All available to us
	            	dataTop = mTop;
		            drawTill = showAtOnce;
	        		break;
	        	default:
	        		// Use top line For Info
	                drawString(windowInfo,infoFont, sCenter, mTop-imgHeight[IMAGE_ID_LIST[ARROWS]],HCENTER | TOP);
	        		dataTop = mTop+mSpacing;
	                drawTill = showAtOnce-1;
            }
           
            for (int lineNum = 0; lineNum < drawTill; lineNum++) {

                int drawIndex = lineNum + windowIndex;
                int drawY = dataTop + (lineNum * mSpacing)+imgHeight[IMAGE_ID_LIST[ARROWS]]+INFO_TEXT_Y_OFFSET;

                boolean drawVal = true;

                if (canvasMode == MODE_GLOSSARY) {

                    drawString(strings[drawIndex],glossaryFont, 2,drawY, LEFT | TOP);

                } else {

                    if (drawIndex == selectIndex) {
                        g.setColor(SELECT_COLOR);
                        g.fillRect(0,drawY+highlightOffset, sWidth, mSpacing);
                        g.setColor(0xffffff);
                    }

                    if (strings[drawIndex].startsWith(Strings.PERCENT)) {
                        if (canvasMode == MODE_STRATEGY) {
                            drawString(strings[drawIndex].substring(1),highlightFont, 2, drawY, LEFT | TOP);
                        } else {
                            drawVal = false;
                            drawString(strings[drawIndex].substring(1),mainFont,2, drawY, LEFT | TOP);
                            int time = values[drawIndex];
                            int hours = time / 3600;
                            time %= 3600;
                            int minutes = time / 60;
                            time %= 60;
							String min;
  							String sec;
  							if (minutes < 10) {
  								min =  Strings.TMPLT_1_0 + minutes;
  							} else if (minutes < 100) {
  							    min = String.valueOf(minutes);
  							} else {
  							    min = Strings.TMPLT_2_0;
  							}
  							if (time< 10) {
  								sec =  Strings.TMPLT_1_0 + time;
  							} else if (time < 100) {
  							    sec =  String.valueOf(time);
  							} else {
  							    sec =  Strings.TMPLT_2_0;
  							}	
  							drawString(hours+ Strings.COLON + min + Strings.COLON + sec,mainFont,sWidth-2,drawY,RIGHT | TOP);
                        }
                    } else {
                        drawString(strings[drawIndex],mainFont, 2, drawY, LEFT | TOP);
                    }

                }
                if ((values != null) && drawVal) {
                	if (modified[drawIndex] > 0) {
                        drawString(String.valueOf(values[drawIndex]),SMALL_GREEN, sWidth-2, drawY, RIGHT | TOP);
                    } else if (modified[drawIndex] < 0) {
                        drawString(String.valueOf(values[drawIndex]),SMALL_RED, sWidth-2, drawY, RIGHT | TOP);
                    } else {
                    	drawString(String.valueOf(values[drawIndex]),mainFont, sWidth-2, drawY, RIGHT | TOP);
                    }
                }
            }
	        
	        if (windowIndex != 0) {
	            drawImage(UP, sCenter, dataTop, BOTTOM | HCENTER);
	        }
	        
	        if ((numLines > drawTill) && ((windowIndex + drawTill) < numLines)) {
	        	System.out.println("WI:"+windowIndex+ " DT:"+drawTill+" NL:"+numLines);
	            drawImage(DOWN, sCenter, mHeight, BOTTOM | HCENTER);
	        }
        } 	
    }
}