package genplay.vp.core;

import javax.microedition.lcdui.*;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import genplay.vp.logi.*;
import genplay.util.*;
import genplay.vp.game.*;

public class MenuScreen extends GPCanvas {
    
	/* ------------------------- Fields ------------------------ */
	/* -- Static -- */
	public static final int CMD_MAINMENU = 2;
	
	// Constants	
    public static final byte NONE = -1;
   
	public static boolean repaintAnimation = false;
    private static final int DIALOG_BOX_OVERHANG = 20; 	// How much larger is the dialog box than its text
    public static final byte QUICKCOMMAND = 0;
	
	// Fonts used by this canvas
	private static int dialogFont = BOLD_YELLOW;
	private static int textFont = SMALL_ORANGE;
	private static int unselectedMenuFont = BOLD_YELLOW;
	private static int selectedMenuFont = BOLD_WHITE;
	private static int titleFont = BOLD_ORANGE;
    
    // File Parse Registers
    public static final int INBUFFSIZE = 2048;
    public static final int MAX_LINES = 150;
    public static final String[] TEXT_FILES = {
			"/credits.txt",
			"/about.txt",
			"/help.txt",
			"/Game_0.txt",
			"/Game_1.txt",
			"/Game_2.txt",
			"/Game_3.txt",
			"/Game_4.txt",
			"/Game_5.txt",
			"/Game_6.txt",
			"/Game_7.txt",
			"/Game_8.txt",
			"/Game_9.txt",
			"/Game_10.txt",
			"/Game_11.txt",
			"/Game_12.txt"
	};
    public static short[][] parseIndex;
    
    /*
	 * The Following Variables are dynamic between modes.
	 * 
	 * They are static for faster access
	 */
    
    // Logo animation registers
    public static int logoTop,logoLeft,logoHeight,logoWidth,logoPlainWidth,logoHighlightWidth;
  
    // Register to prevent multiple setmodes
    static int lastState;
    
    // Text file parsing
    private static InputStreamReader reader;
    private static int fileIndex;
    private static int indexReadTo;
    
    // Display References
    private static int mTop, mSpacing;
        
    // Window Scrolling
    static int windowIndex;
    private static int showAtOnce;
    public static boolean[] lineChanged;
    
    // Dialog Box Control
    static boolean showDialog = false;
    private static String dialog1;
    private static String dialog2;
    
    // Items and selection
    public static int selectedIndex;
    public static String formTitle;
    public static String[] localData;
    public static int[] items;
    
    // Repaint control
    public static boolean repaintAllItems = false;
	public static int menuFlashFrame;

	static int menuFlashType;
	static final boolean[][][] ANIMATIONS = {
			// Animation 0
			{{false,true},
			 {true,false}},
			// Animation 1
			{{false,true,true},
			 {true,false,true},
			 {true,true,false}},
			// Animation 2
			{{false,true,true,true},
			 {true,false,true,true},
			 {true,true,false,true},
			 {true,true,true,false}},
			// Animation 3
			{{false,true,true,true,true},
		     {true,false,true,true,true},
			 {true,true,false,true,true},
			 {true,true,true,false,true},
			 {true,true,true,true,false}},
			// Animation 4
			{{false,false,true,true},
			 {true,false,false,true},
			 {true,true,false,false},
			 {false,true,true,false}},
			// Animation 5
			{{false,true,true},
			 {true,false,true},
			 {true,true,false},
		 	 {true,false,true}},
			// Animation 6
			{{false,true,true,true},
		 	 {true,false,true,true},
			 {true,true,false,true},
			 {true,true,true,false},
			 {true,true,false,true},
			 {true,false,true,true}},
			// Animation 7
			{{false,true,true,true,true},
			 {true,false,true,true,true},
			 {true,true,false,true,true},
			 {true,true,true,false,true},
			 {true,true,true,true,false},
			 {true,true,true,false,true},
			 {true,true,false,true,true},
			 {true,false,true,true,true}}
	};

	private static int numLights;

	private static int lightHeight;

	private static int lightWidth;

	/*
	 * Initialization
	 */
	public static void parseTextFiles() {
		
		try {
			
			parseIndex = new short[TEXT_FILES.length][];
			byte[] inBuffer = new byte[INBUFFSIZE]; 
			
			for (int fileNum=0; fileNum < TEXT_FILES.length;fileNum++) { // For each text file
				//System.out.println("Parsing: " + textFileList[fileNum]);
				
				short[] indexes = new short[MAX_LINES+2];
				
				// Prep the stream
				int lengthRead;
				int nextParseIndex=1;
				int iBuffIndex=0;
				int indexWidth=0;
				int iBuffMark=0;
				int markWidth=0;
				
				// load the buffer and set the Index Properly 
				InputStream iStream = new Object().getClass().getResourceAsStream(TEXT_FILES[fileNum]);
				lengthRead = iStream.read(inBuffer,0,INBUFFSIZE);
				iStream.close();
				indexes[0] = 0;
				
				for (iBuffIndex = iBuffMark;iBuffIndex < lengthRead;iBuffIndex++) {			
					char charIndex = (char)inBuffer[iBuffIndex];	
					if (charIndex == ' ') {
						iBuffMark = iBuffIndex+1;
            			markWidth = indexWidth;
            			indexWidth += 3;
					} else if (charIndex == '/') {
						indexes[nextParseIndex] = (short)(iBuffIndex+1);
						indexWidth=0;
						nextParseIndex++;
					} else if (charIndex == '\n') {
						indexes[nextParseIndex] = (short)(iBuffIndex+1);
						indexes[nextParseIndex+1] = (short)(iBuffIndex+1);
						indexWidth=0;
						nextParseIndex+=2;
					} else {
						int drawIndex = (int)charIndex - startsWith;
			            if (0 > drawIndex || drawIndex >= fontCharWidths[F_SMALL].length) {
			                indexWidth += 3; 
			            } else {
			            	indexWidth += fontCharWidths[F_SMALL][drawIndex]+1;
			            }
					}	
					if (indexWidth >= MENU_TEXT_WIDTH) {
						indexes[nextParseIndex] = (short)iBuffMark;
            			indexWidth-=(markWidth+3);
            			nextParseIndex++;
					} 
				}
				
				parseIndex[fileNum] = new short[nextParseIndex+1];
				parseIndex[fileNum][nextParseIndex] = (short)iBuffIndex;
				System.arraycopy(indexes,0,parseIndex[fileNum],0,nextParseIndex);
			}
			System.gc();
		} catch (Exception e) {
			System.out.println("PARSE ERROR: " + e);
		}
	}
	
	static void mode(int newMode,String name,String[] data, int textFileIndex) {
    	formTitle = name;	
		switch (canvasMode) {
			case MODE_STARTUP:	
				imageHandler(GPCanvas.SPLASH_IMAGE_SET);
				logoHeight = images[ANIMATELOGO].getHeight()>>1;
				logoWidth = images[ANIMATELOGO].getWidth();
				logoTop = (sHeight>>1)-(logoHeight>>1);
				logoLeft = sCenter-(logoWidth>>1);
				logoPlainWidth = 0;
				logoHighlightWidth = 0;	
				new ThreadHandler(ThreadHandler.THREAD_ANIMATION_HANDLER).start();
		    	ThreadHandler.threads[ThreadHandler.THREAD_ANIMATION_HANDLER].setPriority(Thread.MAX_PRIORITY);
		    	break;
		    	
		    case MODE_STATIC_MENU:
		    case MODE_DYNAMIC_MENU:
		    	try {
		    		imageHandler(GPCanvas.MENU_IMAGE_SET);
					// Load indexes for this State 
			    	selectedIndex = refIndexes[newState][SELECTED_INDEX];
			    	windowIndex = refIndexes[newState][WINDOW_INDEX];
			    	mTop = imgHeight[IMAGE_ID_LIST[MBACKTOP]] + MENU_TEXT_OFFSET;
			    	mSpacing = imgHeight[FONT]+5;
			    	lightHeight = imgHeight[IMAGE_ID_LIST[M_LIGHT]&MASK_IMAGE];
			    	lightWidth = imgWidth[IMAGE_ID_LIST[M_LIGHT]&MASK_IMAGE]>>2;
			    	numLights = (mHeight - imgHeight[IMAGE_ID_LIST[MBACKTOP]] - MENU_LIGHT_OFFSET)/lightHeight;
					showAtOnce = (mHeight-imgHeight[IMAGE_ID_LIST[ARROWS]]-mTop)/mSpacing;
					mSpacing = (mHeight-imgHeight[IMAGE_ID_LIST[ARROWS]] - mTop) / showAtOnce;
					lineChanged = new boolean[showAtOnce];	
			    	// If we have data, grab it
			    	if (data != null) {
			    		localData = new String[data.length];
			    		System.arraycopy(data,0,localData,0,data.length);
			    	}
		    	} catch (Exception e) {
		    		System.out.println("mode: Menu: " + e);
		    	}
		    	break;
		    case MODE_DIALOG:
		    	repaintAll = true;
		    	showDialog = true;
		    	dialog1 = data[0];
		    	dialog2 = data[1];
		    	break;
		    case MODE_TEXT:	
		    	try {
			    	// Text Always restarts from top
		    		imageHandler(GPCanvas.MENU_IMAGE_SET);
			    	selectedIndex = 0;
			    	windowIndex = 0;
		    		fileIndex = textFileIndex;
		    		mTop = imgHeight[IMAGE_ID_LIST[MBACKTOP]] + MENU_TEXT_OFFSET;
		    		showAtOnce = (mHeight-imgHeight[IMAGE_ID_LIST[ARROWS]]-mTop)/imgHeight[FONT];
					mSpacing = (mHeight-imgHeight[IMAGE_ID_LIST[ARROWS]] - mTop) / showAtOnce;
		    		if (fileIndex != NONE) {
			    		int sizeRead;
			    		localData = new String[parseIndex[fileIndex].length-1];
			    		char[] inBuffer = new char[sWidth>>1]; 
			    		InputStream iStream = new Object().getClass().getResourceAsStream(TEXT_FILES[textFileIndex]);
			    		reader = new InputStreamReader(iStream);
			    		reader.skip(parseIndex[textFileIndex][0]);
						for (indexReadTo=0; indexReadTo < localData.length;indexReadTo++) {
							try {
								int cur = parseIndex[textFileIndex][indexReadTo+1];
								int last = parseIndex[textFileIndex][indexReadTo];
								if (cur != last) {
									sizeRead = reader.read(inBuffer,0,cur-last-1);
									reader.skip(1);
									localData[indexReadTo]=String.valueOf(inBuffer,0,sizeRead);
								} else {
									localData[indexReadTo]=Strings.BLANK;
								}
							} catch (Exception e) {
								showDialog = true;
								dialog1 = "Parse Exception:";
								dialog2 = e.toString();
								System.out.println("MenuInit: TextLoad:    " + e);
							}
						}
		    		} else if (data != null) {
			    		localData = new String[data.length];
			    		indexReadTo = data.length;
			    		System.arraycopy(data,0,localData,0,data.length);
			    	}
		    	} catch (Exception e) {
		    		System.out.println("mode: Text: " + e);
		    	}
		}
	}
    
    static void keyAction(int keyCode) {
    	int oldWindowIndex = windowIndex;
        try {		        
        	switch (canvasMode) {
        		case MODE_STATIC_MENU:
        		case MODE_DYNAMIC_MENU:
			    	switch (VPHandler.screen.getGameAction(keyCode)) {
						case Canvas.FIRE:
						    if (!showDialog) {
								// Pass the command under selectedIndex to the command MIDlet
						    	if ((short)(items[selectedIndex] & VPHandler.MASK_ACTION) == VPHandler.ACTION_TOGGLE_AV ||
									(short)(items[selectedIndex] & VPHandler.MASK_ACTION) == VPHandler.ACTION_TOGGLE_HELPER) {
									VPHandler.event((short)(items[selectedIndex] & VPHandler.MASK_ACTION), false);
								} else {
									VPHandler.event((short)(items[selectedIndex] & VPHandler.MASK_ACTION), true);
								}
						    }
							break;
							
						case Canvas.UP:
							if (!showDialog) {
								// Scroll selectedIndex to previous Command, if none exists, do nothing
								if (selectedIndex > 0) {
									int indexAbove = selectedIndex - 1;
									boolean foundAbove = true;
									while (!(items[indexAbove] != NONE)) {
										if (indexAbove > 0) {
											indexAbove--;
										} else {
											foundAbove = false;
											break;
										}
									}
									if (foundAbove) {
										if (indexAbove >= windowIndex) {
											lineChanged[selectedIndex-windowIndex] = true;
											lineChanged[indexAbove-windowIndex] = true;
											selectedIndex = indexAbove;
										} else {
											selectedIndex = indexAbove;
											windowIndex=Math.max(windowIndex-showAtOnce,0);
										} 
									} 
								}	
							}
							break;
							
						case Canvas.DOWN:
							if (!showDialog) {
								if (selectedIndex < (items.length - 1)) {
									int indexBelow = selectedIndex + 1;
									boolean foundBelow = true;
									try {
										while (!(items[indexBelow] != NONE)) {
											if (indexBelow < windowIndex+showAtOnce) {
												indexBelow++;
											} else {
												foundBelow = false;
												break;
											}
										}
										if (foundBelow) {
											if (indexBelow < windowIndex + showAtOnce) {
												lineChanged[selectedIndex-windowIndex] = true;
												lineChanged[indexBelow-windowIndex] = true;	
												selectedIndex = indexBelow;
											} else {
												selectedIndex = indexBelow;
												windowIndex=selectedIndex;
											}
										} else if ((items.length > showAtOnce) && ((windowIndex + showAtOnce) != items.length)) {
											if (windowIndex != selectedIndex) {
												windowIndex=selectedIndex;
											} else {
												windowIndex=items.length-showAtOnce;
											}
										}
									} catch (Exception e) {
										
									}
								}
							}
							break;
							
						case Canvas.LEFT:
						case Canvas.RIGHT:
							// If its an item (for options), pass the selected command to command MIDlet 
							if ((short)(items[selectedIndex] & VPHandler.MASK_ACTION) == VPHandler.ACTION_TOGGLE_AV ||
									(short)(items[selectedIndex] & VPHandler.MASK_ACTION) == VPHandler.ACTION_TOGGLE_HELPER) {
									VPHandler.event((short)(items[selectedIndex] & VPHandler.MASK_ACTION), false);
								}
					}
				    break;
			    	
        		case MODE_TEXT:
				    switch (VPHandler.screen.getGameAction(keyCode)) {
				    	case Canvas.FIRE:
				    		if (commands.length > CMD_QUICK) {
				    			if (commands.length > CMD_QUICK) if (commands[CMD_QUICK]!=NONE) VPHandler.event(commands[CMD_QUICK], true);
				    		}
				    		break;
		            	case Canvas.UP:
		            		if (windowIndex > 0) windowIndex--;
		        			break;
		        		case Canvas.DOWN:
		        			if ((localData.length > showAtOnce) && ((windowIndex+showAtOnce) < localData.length)) windowIndex++;
		        			break;
	            	}
				    break;
        	}
            
            if (windowIndex != oldWindowIndex) repaintAllItems=true;
            	
        	if (canvasMode == MODE_DYNAMIC_MENU) {
        		
        		commands = null;
        		if (!Game.gamePlayed[selectedIndex]) {
        			newCommands[CMD_FORE] = ((VPHandler.PLAY<<8) | VPHandler.ACTION_SELECT_GAME);
        		} else {
        			newCommands[CMD_FORE] = ((VPHandler.RESUME<<8) | VPHandler.ACTION_SELECT_GAME);
        		}        
        		
        	} 
            	
            VPHandler.screen.repaint();
            
        } catch (Exception e) {
        	System.out.println("MenuScreen: keyPressed: " + e);
        }
        
    }
    
    public static void animate(Graphics g) {
    	switch (canvasMode) {
	    	case MODE_LOGO_ANIMATE:
	    	case MODE_LOGO_PERSISTANT:
	    		repaintAll=false;
	    		g.setColor(BLACK);
	    		g.fillRect(0,0,sWidth,sHeight);
	    		g.setClip(logoLeft,logoTop,logoPlainWidth,logoHeight);
	    		drawImage(ANIMATELOGO,logoLeft,logoTop,TOP | LEFT);
	    		g.setClip(logoLeft+logoPlainWidth,logoTop,logoHighlightWidth,logoHeight);
	    		drawImage(ANIMATELOGO,logoLeft,logoTop-logoHeight,TOP | LEFT);
	    		break;
	    		
	    	case MODE_SPLASH:
	    		repaintAll=false;
	    		g.setClip(0,0,sWidth,sHeight);
	        	g.setColor(BLACK);
	        	g.fillRect(0,0,sWidth,sHeight);
	            int logoHeight = images[LOGO].getHeight();
	            int copyrightHeight = images[COPYRIGHT].getHeight();
	        	drawImage(SPLASHBACK,sCenter,logoHeight,HCENTER|TOP);
	        	g.fillRect(0,0,sWidth,logoHeight);
	        	drawImage(LOGO,sCenter,0,HCENTER|TOP);
	        	g.fillRect(0,sHeight-copyrightHeight,0,copyrightHeight);
	        	drawImage(COPYRIGHT,sCenter,sHeight,HCENTER|BOTTOM);
	        	drawImage(SUBTITLE,sWidth,sHeight-copyrightHeight-SPLASH_TITLE_OFFSET_Y,BOTTOM|RIGHT);
	    		break;

	    	default:
	    		try {
		    		if (repaintAllItems = true && animateMode == canvasMode) {
		    			repaintAllItems = false;
		    			drawImage(M_BOTTOM,sCenter,mHeight,BOTTOM | HCENTER);
		    			g.setClip(sCenter - imgHeight[IMAGE_ID_LIST[ARROWS]],imgHeight[IMAGE_ID_LIST[MBACKTOP]],imgHeight[IMAGE_ID_LIST[ARROWS]]<<1,MENU_TEXT_OFFSET);
		    			drawImage(M_BORDER,sCenter,imgHeight[IMAGE_ID_LIST[MBACKTOP]],HCENTER | TOP);	
		    		}
		    		switch (canvasMode) {
		    			case MODE_TEXT:
		    				drawImage(M_BOTTOM,sCenter,mHeight,BOTTOM | HCENTER);
		    	    		if (windowIndex != 0) {
		    	    			drawImage(UP,sCenter,mTop,BOTTOM | HCENTER);
		    	    		} 
		    	    		if ((indexReadTo > showAtOnce) && ((windowIndex + showAtOnce) != indexReadTo)) {
		    	    			drawImage(DOWN,sCenter,mHeight,BOTTOM | HCENTER);
		    	    		}
		    				break;
					    case MODE_STATIC_MENU:
					    case MODE_DYNAMIC_MENU:
					    	if (items.length > showAtOnce) {
			    	    		if (windowIndex != 0) {
			    	    			drawImage(UP,sCenter,mTop,BOTTOM | HCENTER);
			    	    		} 
			    	    		if ((windowIndex + showAtOnce) < items.length) {
			    	    			drawImage(DOWN,sCenter,mHeight,BOTTOM | HCENTER);
			    	    		}
				    		}
					    break;
		    		}
//#ifndef NO_MENU_ANIMATION
		    		int flashMode = menuFlashType%ANIMATIONS.length;
			        boolean invertLights = menuFlashType/ANIMATIONS.length==2;
			        for (int lightNum= 0; lightNum <= numLights; lightNum++) {
			        	int lightPosition = lightNum%ANIMATIONS[flashMode][menuFlashFrame].length;
			        	int drawLight =0;
			        	boolean lightOn = ANIMATIONS[flashMode][menuFlashFrame][lightPosition];
			        	if (invertLights) lightOn = !lightOn;
			        	if (lightOn) drawLight+=2;
			        	int drawLeft = drawLight + lightNum%2;
			        	int drawRight = drawLight + (lightNum+1)%2;
			        	g.setClip(0,0,sCenter-(MENU_LIGHT_WIDTH>>1),mHeight);
			        	g.drawImage(images[M_LIGHT],sCenter-(MENU_LIGHT_WIDTH>>1)-lightWidth-(drawLeft*lightWidth),imgHeight[IMAGE_ID_LIST[MBACKTOP]]+MENU_LIGHT_OFFSET+lightNum*lightHeight,TOP|LEFT);
			        	g.setClip(sCenter+(MENU_LIGHT_WIDTH>>1),0,lightWidth,mHeight);
			        	g.drawImage(images[M_LIGHT],sCenter+(MENU_LIGHT_WIDTH>>1)-(drawRight*lightWidth),imgHeight[IMAGE_ID_LIST[MBACKTOP]]+MENU_LIGHT_OFFSET+lightNum*lightHeight,TOP|LEFT);
			        }
//#endif
	    		} catch (Exception e) {
	    			// rare, random exception... not critical.
	    		}
			    break;
    	}
    }
    
    public static void draw(Graphics g,boolean repaintAll) {
    	g.setClip(0,0,sWidth,mHeight);
        switch (canvasMode) {
	    	case MODE_LOGO_ANIMATE:
	    	case MODE_LOGO_PERSISTANT:	
	    	case MODE_SPLASH:
	    		// handled in Animate
	    		break;
        		
        	default:
		        if (repaintAll) {
		        	repaintAll = false;
		        	repaintAllItems = true;
		        	commands = null;
		        	g.setColor(BLACK);
		        	g.fillRect(0,0,sWidth,sHeight);
		        	g.setColor(YELLOW);	
		        	drawImage(MBACKGRN,sCenter,(mTop+mHeight)>>1,HCENTER | VCENTER);
		        	drawImage(M_BORDER,sCenter,imgHeight[IMAGE_ID_LIST[MBACKTOP]],HCENTER | TOP);
	            	drawImage(MBACKTOP,sCenter,0,HCENTER | TOP);
	        		drawString(formTitle,titleFont,sCenter,(imgHeight[IMAGE_ID_LIST[MBACKTOP]]>>1)+(imgHeight[FONT]>>1)+2,HCENTER | BOTTOM);
		        }
		        
		    	try {
			    	int drawTill;
		    		if (repaintAllItems) {
		    			System.gc();
		            	g.setClip(sCenter-(MENU_TEXT_WIDTH>>1),mTop,MENU_TEXT_WIDTH+1,mHeight-mTop);
		            	g.setColor(BLACK);
		            	g.fillRect(sCenter-(MENU_TEXT_WIDTH>>1),mTop,MENU_TEXT_WIDTH+1,mHeight-mTop);
		            	drawImage(MBACKGRN,sCenter,(mTop+mHeight)>>1,HCENTER | VCENTER);
		            	g.setClip(0,0,sWidth,mHeight);
		            }
		    	    switch (canvasMode) {
		    	        case MODE_TEXT:    
		    	            if (repaintAllItems) {
			    	        	drawTill = Math.min(showAtOnce,indexReadTo);
			    	    		for (int c = 0; c < drawTill; c++) {
			    	    			int drawIndex = c+windowIndex;
			    	    			int drawY = mTop + (c * mSpacing);
			    	    			if (fileIndex != NONE) {
				    	    			drawString(localData[drawIndex],textFont,sCenter,drawY,HCENTER | TOP);
			    	    			} else {
			    	    				int splitIndex = localData[drawIndex].indexOf(Strings.SPLIT_DELINIATOR);
			    	    				drawString(localData[drawIndex].substring(0,splitIndex),textFont,sCenter-(MENU_TEXT_WIDTH>>1),drawY,LEFT | TOP);
			    	    				drawString(localData[drawIndex].substring(splitIndex+1),textFont,sCenter+(MENU_TEXT_WIDTH>>1),drawY,RIGHT | TOP);
			    	    			}
			    	    		}
			    	    		g.setClip(0,0,sWidth,sHeight);
		    	            }
		    	            break;
		    	            
		    	        case MODE_STATIC_MENU:
		    	        case MODE_DYNAMIC_MENU:
		    	        	if (items != null) {
			    	    		drawTill = Math.min(showAtOnce,items.length);
			    	    		for (int lineNum = 0; lineNum < drawTill; lineNum++) {
			    	    			if (lineChanged[lineNum] || repaintAllItems) {
			    	    				lineChanged[lineNum] = false;
				    	    			int drawIndex = lineNum+windowIndex;
				    	    			if (drawIndex >= items.length) break;
				    	    			int drawY = mTop + (lineNum * mSpacing);
				    	    			boolean wideBox;
				    	    			g.setColor(BLACK);
				    	    			int boxHeight;
		    	    					if ((items[drawIndex]&VPHandler.MASK_ACTION)==VPHandler.ACTION_TOGGLE_HELPER ||
		    	    						(items[drawIndex]&VPHandler.MASK_ACTION)==VPHandler.ACTION_TOGGLE_AV) {
		    	    						boxHeight = mSpacing*2;
		    	    						wideBox = true;
		    	    					} else {
		    	    						boxHeight = mSpacing;
		    	    						wideBox = false;
		    	    					}			
		    	    					g.setClip(sCenter-(MENU_TEXT_WIDTH>>1)-1,drawY,MENU_TEXT_WIDTH+2,boxHeight);
		    	    					g.fillRect(sCenter-(MENU_TEXT_WIDTH>>1)-1,drawY,MENU_TEXT_WIDTH+2,boxHeight);
		    	    					drawImage(MBACKGRN,sCenter,(mTop+mHeight)>>1,HCENTER | VCENTER);
				    	    			if (items[drawIndex] != NONE) {
				    	    				int itemFont = NONE;		
				    	    				if (drawIndex == selectedIndex) {	
					    	    				itemFont = selectedMenuFont;
					    	    				g.setColor(TAN);
					    	    				g.drawLine(sCenter-(MENU_TEXT_WIDTH>>1)+1,drawY,sCenter+(MENU_TEXT_WIDTH>>1)-1,drawY);
					    	    				g.drawLine(sCenter+(MENU_TEXT_WIDTH>>1),drawY+1,sCenter+(MENU_TEXT_WIDTH>>1),drawY+boxHeight-2);
					    	    				g.drawLine(sCenter-(MENU_TEXT_WIDTH>>1)+1,drawY+boxHeight-1,sCenter+(MENU_TEXT_WIDTH>>1)-1,drawY+boxHeight-1);
					    	    				g.drawLine(sCenter-(MENU_TEXT_WIDTH>>1),drawY+1,sCenter-(MENU_TEXT_WIDTH>>1),drawY+boxHeight-2);
					    	    				g.drawRect(sCenter-(MENU_TEXT_WIDTH>>1)+1,drawY+1,MENU_TEXT_WIDTH-2,boxHeight-3);
				    	    				} else {
				    	    					itemFont = unselectedMenuFont;
				    	    				}
				    	    				if ((items[drawIndex]>>>8) < Strings.COMMANDS.length) {
				    	    					if (wideBox) {
				    	    						drawString(Strings.COMMANDS[items[drawIndex]>>>8],unselectedMenuFont,sCenter,drawY+(mSpacing>>1)+(imgHeight[FONT]>>1),BOTTOM|HCENTER);
				    	    						drawString(localData[drawIndex+1],selectedMenuFont,sCenter,drawY+mSpacing,HCENTER | TOP);
				    	    						lineChanged[lineNum+1]=false;
				    	    						lineNum++;
				    	    					} else {
				    	    						drawString(Strings.COMMANDS[items[drawIndex]>>>8],itemFont,sCenter,drawY+(mSpacing>>1)+(imgHeight[FONT]>>1),BOTTOM|HCENTER);
				    	    					}
				    	    				} else {
				    	    					drawString(Game.GAME_LIST[(items[drawIndex]>>>8)-Strings.COMMANDS.length],itemFont,sCenter,drawY+(mSpacing>>1)+(imgHeight[FONT]>>1),BOTTOM|HCENTER);
				    	    				}
				    	    			} else {
				    	    				String toDraw = localData[drawIndex];
				    	    				g.setClip(sCenter-(MENU_TEXT_WIDTH>>1)-1,drawY,MENU_TEXT_WIDTH+2,mSpacing);
				    	    				//drawImage(MBACKTOP,sCenter,imgHeight[IMAGE_ID_LIST[MBACKTOP],HCENTER | TOP);
				    	    				drawString(toDraw,textFont,sCenter,drawY,HCENTER | TOP);
				    	    				//g.setClip(0,0,sWidth,mHeight);
				    	    			}
			    	    			} 
			    	    		}
		    	    		}
		    	    		g.setClip(0,0,sWidth,sHeight);
		    	    		break;
		    	    		
		    	    	case MODE_DIALOG:
		    	    		int len1 = 0,len2 = 0;
			    			len1 = getStringWidth(1,dialog1);
			    			if (dialog2 != null) len2 = getStringWidth(1,dialog2);
			    			int textWidth = Math.max(len1,len2) + DIALOG_BOX_OVERHANG;
			    			int textHeight = (2 * imgHeight[FONT]) + DIALOG_BOX_OVERHANG;
			    			int boxCenterY = mTop + (mHeight>>3) + (textHeight>>1);
			    			g.setClip(0,0,sWidth,sHeight);
			    			g.setColor(ORANGE);
			    			
			    			
			    			
			    			g.drawRect(sCenter-(textWidth>>1),(boxCenterY-(textHeight>>1)),textWidth,textHeight);
			    			drawString(dialog1,dialogFont,sCenter,boxCenterY-2,HCENTER | BOTTOM);
			    			if (dialog2 != null) drawString(dialog2,dialogFont,sCenter,boxCenterY+2,HCENTER | TOP);
			    			g.setClip(0,0,sWidth,sHeight);
		    	    		drawImage(M_BOTTOM,sCenter,mHeight,BOTTOM | HCENTER);
		    	    }
		        } catch (Exception e) {
		        	System.out.println("MenuCanvas: paintMain: " + e);
		        }
        }
    }
    
    public static long thread(long startTime) {
    	if (VPHandler.isRunning) {
			switch (canvasMode) {
				case MODE_LOGO_ANIMATE:
					// Calculate the logo showpercentage
					int runTime = (int)(System.currentTimeMillis() - startTime);
					int overRun = MenuScreen.logoWidth/GENPLAY_GLOW_DENOMINATOR;
					int totalWidth = MenuScreen.logoWidth + overRun;
					int runWidth = (runTime*totalWidth)/GENPLAY_ANIMATE_INTERVAL;
					MenuScreen.logoPlainWidth = Math.max(0,runWidth-overRun);
					MenuScreen.logoHighlightWidth = Math.min(overRun,runWidth);
					repaintAnimation = true;
					VPHandler.screen.repaint();
					if (runWidth < totalWidth) {
						 return TICK;
					} else {
						canvasMode = MODE_LOGO_PERSISTANT;
						VPHandler.event(VPHandler.ACTION_GAME_INITIALIZE, false);
						return GENPLAY_PERSISTANT_INTERVAL;
					}
					
				case MODE_LOGO_PERSISTANT:
					// Set Timer for Splashscreen
					canvasMode = MODE_SPLASH;
					repaintAll = true;
				    VPHandler.screen.repaint();
					Media.soundPlay(Media.theme,1,false);
					return SPLASH_PERSISTANT_INTERVAL;
				
				case MODE_TEXT:
			    case MODE_STATIC_MENU:
			    case MODE_DYNAMIC_MENU:
			    case MODE_DIALOG:
			    	// Set the menuScreen flash state
			        boolean swapDirection = menuFlashType/ANIMATIONS.length==1;
			        if (swapDirection) {
			        	MenuScreen.menuFlashFrame--;
			        	if (MenuScreen.menuFlashFrame < 0) MenuScreen.menuFlashFrame = ANIMATIONS[menuFlashType%ANIMATIONS.length].length-1;
			        } else {
						MenuScreen.menuFlashFrame++;
						MenuScreen.menuFlashFrame%=ANIMATIONS[menuFlashType%ANIMATIONS.length].length;
			        }
					repaintAnimation = true;
					VPHandler.screen.repaint();
					return MENU_LIGHT_INTERVAL;
					
				case MODE_SPLASH:
					if (!VPHandler.loaded) return TICK;
					VPHandler.event(VPHandler.STATE_MAIN, false);
					VPHandler.screen.showNotify();
					VPHandler.screen.repaint();
					break;
					
				case MODE_MOD_PAYOFF:
			    case MODE_SHOW_PAYOFF:
			    case MODE_STRATEGY:
			    case MODE_STATISTICS:
			    case MODE_GLOSSARY:
			    	try { Thread.sleep(250);} catch (Exception e) { }
			    	VPHandler.screen.repaint();
			    break;
			    	
			}
				
			synchronized (VPHandler.screen) {
				try {VPHandler.screen.wait();} catch (Exception e) {} // Wait for the VPHandler to tell us we're needed again
			}	
			menuFlashType = Game.random.nextInt(ANIMATIONS.length*3);
			return 0;
    	} else {
    		synchronized (VPHandler.screen) {
    			long timeAtPause = System.currentTimeMillis() - startTime;
				try { VPHandler.screen.wait(); } catch (Exception e) {} // Wait for the VPHandler to tell us we're needed again
				startTime = System.currentTimeMillis() - timeAtPause;
    		}	
    		menuFlashType = Game.random.nextInt(ANIMATIONS.length*3);
			return 0;
    	}
	}
}
