package genplay.vp.core;
/*
 * File: VPHandler.java
 * App : Poker Coach: Tournament Edition
 * 
 * This Class handles MIDlet functionality, as well as housing the the
 * Game's main state machine in thread();
 */

import javax.microedition.midlet.*;
import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.lcdui.*;

import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.util.*;

import genplay.vp.game.*;
import genplay.vp.logi.*;
import genplay.util.*;
import genplay.util.Media;

public class VPHandler extends MIDlet implements CommandListener {
	
	public static final int NONE = -1;
	
	public static final long SPLASH_TIME = 1000;
	public static final int QUEUE_SIZE = 10;
	
	public static final int MASK_ACTION = 0x000000ff;
	public static final int MASK_LABEL = 0x0000ff00;
	
	// Command lable indices (in genplay.vp.logi.Strings.COMMANDS
	// Text File names:
	public static final int CREDITS = 0;
	public static final int ABOUT = 1;
	public static final int HELP = 2;

	public static boolean shutDown=false;
	public static final int GAME = 3;
	public static final int OK = 3;
	public static final int BACK = 4;
	public static final int YES = 5;
	public static final int NO = 6;
	public static final int SELECTGAME = 7;
	public static final int OPTIONS = 8;
	public static final int QUIT = 9;
	public static final int PLAY = 10;
	public static final int PAYOFFS = 11;
	public static final int STRATEGY = 12;
	public static final int STATS = 13;
	public static final int GLOSSARY = 14;
	public static final int BET = 15;
	public static final int DRAW = 16;
	public static final int RESUME = 17;
	public static final int RESETGAME = 18;
	public static final int RESET = 19;
	public static final int MAINMENU = 20;
	public static final int MENU = 21;
	public static final int PAUSE = 22;
	public static final int DONE = 23;
	public static final int RETRY = 24;
	public static final int END = 25;
	public static final int LOAD = 26;
	public static final int GAMEINFO = 27;
	public static final int TOURNAMENT = 28;
	public static final int CANCEL = 29;
	public static final int AVSETTING = 30;
	public static final int HELPERSETTING = 31;
	public static final int GOTOCURLEADERS = 32;
	public static final int GOTOPREVLEADERS = 33;
	public static final int RESTART = 34;
	
	// States are explicit Game Screens
	public static final int STATE_INTRO = 0;
	public static final int STATE_MAIN = 1;
	public static final int STATE_MAIN_QUIT = 2;
	public static final int STATE_MAIN_CONNECT_ATTEMPT = 3;
	public static final int STATE_MAIN_CONNECT_FAILED = 4;
	public static final int STATE_MAIN_CONNECT_UNAVAILABLE = 5;
	public static final int STATE_CREDITS = 6;
	public static final int STATE_ABOUT = 7;
	public static final int STATE_OPTIONS_MAIN = 8;
	public static final int STATE_OPTIONS_TOURNY = 9;
	public static final int STATE_OPTIONS_GAME = 10;
	public static final int STATE_TOURNY = 11;
	public static final int STATE_TOURNY_END= 12;
	public static final int STATE_TOURNY_HIGHSCORE = 13;
	public static final int STATE_TOURNY_CONNECT_ATTEMPT = 14;
	public static final int STATE_TOURNY_CONNECT_FAILED = 15;
	public static final int STATE_HIGHSCORES_CURRENT = 17;
	public static final int STATE_HIGHSCORES_PREV = 18;
	public static final int STATE_GAME_GOTO_TOURNY = 19;
	public static final int STATE_GOTO_GAME_REGULAR = 20;
	public static final int STATE_INFO_PREGAME = 21;
	public static final int STATE_INFO_PREGAME_LOADING = 22;
	public static final int STATE_GAMESELECT = 23;
	public static final int STATE_GAME_LOADING = 24;
	public static final int STATE_INGAME_PAYOFF = 25;
	public static final int STATE_INGAME_STRATEGY = 26;
	public static final int STATE_GAME_MENU = 27;
	public static final int STATE_GAME_INFO = 29;
	public static final int STATE_GAME_STRATEGY = 28;
	public static final int STATE_GAME_STATS = 30;
	public static final int STATE_GAME_HELP = 31;
	public static final int STATE_OPTIONS_PAYOFFS = 32;
	public static final int STATE_GOTO_ENDSTATS = 33;
	public static final int STATE_OPTIONS_RESET = 34;
	public static final int STATE_GAME_STRATEGY_GLOSSARY = 35;
	public static final int STATE_TOURNY_STATS = 36;
	public static final int STATE_ENDGAME_STATS = 37;
	public static final int STATE_OPTIONS_RESET_DIALOG = 38;
	public static final int STATE_TOURNY_LOAD = 39;
	public static final int STATE_GOTO_HIGHSCORE = 40;
	public static final int STATE_GAME_MENU_LOADING = 41;
	public static final int STATE_GAME_REGULAR = 42;
	public static final int STATE_GAME_TOURNY = 43;
	public static final int NUM_STATES = 44;	
	// Actions are events not associated with any particular screen
	public static final int ACTION_TOGGLE_AV = 45;
	public static final int ACTION_TOGGLE_HELPER = 46;
	public static final int ACTION_GAME_INITIALIZE = 47;
	public static final int ACTION_QUIT = 48;
	public static final int ACTION_CHECK_EXPIRATION = 49;
	public static final int ACTION_LOAD_GAME = 50;
	public static final int ACTION_RESET = 51;
	public static final int ACTION_BACK = 52;
	public static final int ACTION_PROCEED = 53;
	public static final int ACTION_GET_CURRENT_TOURNAMENT = 54;
	public static final int ACTION_GET_PREV_TOURNAMENT = 55;
	public static final int ACTION_RELOAD_CURRENT_TOURNAMENT = 56;
	public static final int ACTION_SELECT_GAME = 57;
	public static final int ACTION_TOURNAMENT_OVER = 58;
	public static final int ACTION_CONFIRM_RESET = 59;
	public static final int ACTION_SAVE_SETTINGS = 60;
	public static int gameState;
	
	// Main Screen Commands
	public static final int CMD_FORE = 0;
    public static final int CMD_BACK = 1;
    public static final int CMD_QUICK = 2;
	public static final int[] NOCOMMANDS = {NONE,NONE,NONE,NONE};

	// Commands for various states
	public static final int[] MAIN_CMDS = {
			((OK<<8) | ACTION_PROCEED),
			NONE};
	public static final int[] MAIN_MENU_ITEMS = {
			((SELECTGAME<<8) | STATE_GAMESELECT),
			((TOURNAMENT<<8) | STATE_TOURNY),
			((OPTIONS<<8) | STATE_OPTIONS_MAIN),
			((ABOUT<<8) | STATE_ABOUT),
			((CREDITS<<8) | STATE_CREDITS),
			((QUIT<<8) | STATE_MAIN_QUIT)};
	public static final int[] MAIN_QUIT_CMDS = {
			((YES<<8) | ACTION_QUIT),
            ((NO<<8) | STATE_MAIN)};
	public static final int[] MAIN_CONNECT_FAILED_CMDS = {
			((RETRY<<8) | STATE_TOURNY),
			((CANCEL<<8) | STATE_MAIN)};
	public static final int[] CREDITS_CMDS = {
			NONE,
			((BACK<<8) | STATE_MAIN)
			,NONE};
	public static final int[] ABOUT_CMDS = {
			NONE,
			((BACK<<8) | STATE_MAIN)
			,NONE};
	public static final int[] MAIN_OPTION_ITEMS = {
			((AVSETTING<<8) | ACTION_TOGGLE_AV), 
			NONE,
			((HELPERSETTING<<8) | ACTION_TOGGLE_HELPER), 
			NONE};
	public static final int[] TOURNY_OPTION_ITEMS = {
			((AVSETTING<<8) | ACTION_TOGGLE_AV), 
			NONE,
			((RESETGAME<<8) | ACTION_CONFIRM_RESET)};
	public static final int[] TOURNY_OPTION_CMDS = {
			NONE,
			((BACK<<8) | ACTION_SAVE_SETTINGS)};
	public static final int[] GAME_OPTION_ITEMS = {
			((AVSETTING<<8) | ACTION_TOGGLE_AV), 
			NONE,
			((HELPERSETTING<<8) | ACTION_TOGGLE_HELPER), 
			NONE,
			((PAYOFFS<<8) | STATE_OPTIONS_PAYOFFS),
			((RESET<<8 | STATE_OPTIONS_RESET_DIALOG))};
	public static final int[] GAME_OPTION_CMDS = {
			NONE,
			((BACK<<8) | ACTION_SAVE_SETTINGS)};
	public static final int[] GAMESELECT_CMDS = {
			((PLAY<<8) | ACTION_PROCEED),
			((BACK<<8) | STATE_MAIN)};
	public static final int[] MAIN_OPTION_CMDS = {
			NONE,
			((BACK<<8) | ACTION_SAVE_SETTINGS)};
	public static final int[] INFO_CMDS = {
			((PLAY<<8) | STATE_GOTO_GAME_REGULAR),
			((BACK<<8) | STATE_GAMESELECT),
			STATE_GOTO_GAME_REGULAR};
	public static final int[] REGULAR_GAME_CMDS = {
			((PAUSE<<8) | STATE_GAME_MENU),
			((BACK<<8) | STATE_GAMESELECT),
			STATE_INGAME_STRATEGY,
			STATE_INGAME_PAYOFF};
	public static final int[] BACK_CMDS = {
			NONE,
			((BACK<<8) | ACTION_BACK)};
	public static final int[] INGAME_STRATEGY_CMDS = {
			((GLOSSARY<<8) | STATE_GAME_STRATEGY_GLOSSARY),
			((BACK<<8) | ACTION_BACK)};
	public static final int[] GLOSSARY_CMDS = {
			NONE,
			((BACK<<8) | STATE_INGAME_STRATEGY)};
	public static final int[] GAME_MENU_CMDS = {
			((RESUME<<8) | STATE_GOTO_GAME_REGULAR),
			((BACK<<8) | STATE_GAMESELECT)};
	public static final int[] GAME_MENU_ITEMS = {
			((GAMEINFO<<8) | STATE_GAME_INFO),
			((OPTIONS<<8) | STATE_OPTIONS_GAME),
			((STATS<<8) | STATE_GAME_STATS),
			((STRATEGY<<8) | STATE_GAME_STRATEGY),
			((HELP<<8) | STATE_GAME_HELP)};
	public static final int[] GAME_INFO_CMDS = {
			NONE,
			((BACK<<8) | STATE_GAME_MENU)};
	public static final int[] OPTIONS_PAYOFF_CMDS = {
			NONE,
			((BACK<<8) | STATE_OPTIONS_GAME)};
	public static final int[] ENDSTATS_CMDS = {
			((RETRY<<8) | STATE_GAME_GOTO_TOURNY),
			((MENU<<8) | STATE_TOURNY)};
	public static final int[] HELP_CMDS = {
			NONE,
			((BACK<<8) | STATE_GAME_MENU)};
	public static final int[] MAIN_RESET_CMDS = {
			((YES<<8) | ACTION_CONFIRM_RESET),
			((NO<<8) | STATE_OPTIONS_GAME)};
	public static final int[] TOURNY_CMDS = {
			NONE,
			((BACK<<8) | STATE_MAIN)};

	public static Runtime runtime;
	public static Display display;
	
	public static GPCanvas screen;
	public static Form highScore;
	
	public static VPHandler me;
	
	boolean firstRun;
	public static boolean loaded;
	public static boolean isBusy;

	// Public Accessors - Use to set which games to load
	public static int selectedGame = Game.NONE;	
	public static int backTarget;
	
	public static boolean isRunning = false;
	
	static int queueStateIn = NONE;
	static int queueStateOut = NONE;
	static int[] stateQueue = new int[QUEUE_SIZE];

	private static int lastState;
	
	public VPHandler() {
		
		runtime = Runtime.getRuntime();
		display = Display.getDisplay(this);
		loaded = false;
		me = this;
		Media.initialize();
		Game.loadGameSettings();
	    screen = new GPCanvas(GPCanvas.SOFTKEY_HEIGHT,true);
	    firstRun = true;
	    
	}
	
	protected void startApp() {
		
		isRunning = true;
		
		if (lastState == STATE_GOTO_HIGHSCORE) {
			display.setCurrent(highScore);
		} else {
			display.setCurrent(screen);
		}
		
		if (firstRun) {
			firstRun = false;
			new ThreadHandler(ThreadHandler.THREAD_GAME_HANDLER).start();
			event(STATE_INTRO, false);
		} else {
			screen.repaintAll = true;
			screen.repaint();
		}
	
	}
	
	protected void pauseApp() {
		
		isRunning = false;
		
	    Media.stop();
	    Game.saveGame();
	    
	    GPCanvas.repaintAll = true;
	    
	}
	
	protected void destroyApp(boolean arg0) {
		
		shutDown = true;
		
		// Let our threads know to shut down
		synchronized (stateQueue) { stateQueue.notifyAll(); }
		synchronized (screen) { screen.notifyAll(); }
	    
		Media.stop();
		Game.saveGame();
	    
	}

	/*
	 * Push the new event onto the queue and notify the handler thread
	 */
	public static void event(int newAction, boolean newScreen) {

		synchronized (stateQueue) {	
			
			if (newAction != NONE) {
				queueStateIn++;
				queueStateIn %= QUEUE_SIZE;
				stateQueue[queueStateIn] = newAction&MASK_ACTION;
				stateQueue.notify();		
				isBusy = true;
			}
			
			if (newScreen) {
				screen.hideNotify();
				GPCanvas.commands = VPHandler.NOCOMMANDS;
			}

		}
		
	}
	
	public static void thread() {
		int gotoState;
		synchronized (stateQueue) {
			
			// If we're up to date, display the correct screen and wait
			if (queueStateIn == queueStateOut) {
				
				if (lastState < NUM_STATES) GPCanvas.curState = GPCanvas.newState;
				
				isBusy = false;
				//#ifdef DEBUG
				System.out.println("VPHandler: Thread: Done, sleeping");
				//#endif
				screen.showNotify();
				try {stateQueue.wait();} catch (Exception e) { } 	// wait for more input
				//#ifdef DEBUG
				System.out.println("VPHandler: Thread: AWAKE!");
				//#endif
				
			}
			
			queueStateOut++;
			if (queueStateOut == QUEUE_SIZE) queueStateOut=0;
			gotoState = stateQueue[queueStateOut];
			
		}

		if (gotoState < NUM_STATES) GPCanvas.newState = gotoState;
		
		//#ifdef DEBUG
		System.out.println("VPHandler: Thread: DOING->" + gotoState);
		//#endif
		switch (gotoState) {
			case ACTION_PROCEED:
				event(MenuScreen.items[MenuScreen.selectedIndex], false);
				break;
				
			case ACTION_BACK:
				event(backTarget, false);
				break;
				
			case ACTION_GAME_INITIALIZE:
				//#ifdef DEBUG
				try {
					System.out.println("START LOAD");
				//#endif
					// initialize Graphics
					GPCanvas.initialize();
					GameScreen.initialize();
					InfoScreen.initialize();
			    	MenuScreen.parseTextFiles();
			    //#ifdef DEBUG
			    	System.out.println("END LOAD");
				} catch (Exception e) {
					System.out.println("INIT: " + e);
					e.printStackTrace();
				}	
				//#endif
				loaded = true;
				break; 
				
			case ACTION_QUIT:
		    	me.destroyApp(true);
				me.notifyDestroyed(); 
		    	break;
		    
		    case ACTION_GET_PREV_TOURNAMENT:
		    	if (!TournInfo.prevIsLoaded) { 
		    		new TournInfo((int)TournInfo.LOAD_PREV,STATE_HIGHSCORES_PREV,STATE_MAIN_CONNECT_FAILED,me).start();
		    	} else {
		    		event(STATE_HIGHSCORES_PREV, false);
		    	}
		    	break;
		    	
		    case ACTION_RELOAD_CURRENT_TOURNAMENT:
		    	if (!TournInfo.tournamentIsUpToDate) {
		    		new TournInfo((int)TournInfo.LOAD_CURRENT,STATE_HIGHSCORES_CURRENT,STATE_MAIN_CONNECT_FAILED,me).start();
		    	} else {
		    		event(STATE_HIGHSCORES_CURRENT, false);
		    	}
		    	break;
		    	
		    case ACTION_GET_CURRENT_TOURNAMENT:
		    	if (!TournInfo.tournamentIsLoaded) {
		    		new TournInfo((int)TournInfo.LOAD_CURRENT,STATE_TOURNY,STATE_MAIN_CONNECT_FAILED,me).start();
		    	} else {
		    		event(STATE_TOURNY, false);
		    	}
		    	break;
		    	
			case ACTION_TOGGLE_AV:
				Game.avSetting--;
				if (Game.avSetting <= Game.LOOP) Game.avSetting = Game.AV_ALL;
				//#ifdef NO_AV_VIB
					//#define NO_AV_ALL
				if (Game.avSetting == Game.AV_VIBRATE) Game.avSetting--;
				//#endif
				//#ifdef NO_AV_ALL
				if (Game.avSetting == Game.AV_ALL) Game.avSetting--;
				//#endif
				if (Game.avSetting == Game.AV_OFF || Game.avSetting == Game.AV_VIBRATE) Media.stop();
				MenuScreen.localData[MenuScreen.selectedIndex+1] = Strings.AV_OPTIONS[Game.avSetting];
				MenuScreen.lineChanged[MenuScreen.selectedIndex]=true;
				break;

			case ACTION_TOGGLE_HELPER:
				Game.hSetting--;
				if (Game.hSetting <= Game.LOOP) Game.hSetting = Game.HELPER_ON;
				MenuScreen.localData[MenuScreen.selectedIndex+1]=Strings.HELP_OPTIONS[Game.hSetting];
				MenuScreen.lineChanged[MenuScreen.selectedIndex]=true;
				break;
				
			case ACTION_SELECT_GAME:
				selectedGame = MenuScreen.selectedIndex;
				if (Game.gamePlayed[selectedGame]) {
					event(STATE_GOTO_GAME_REGULAR, false);
				} else {
					event(STATE_INFO_PREGAME, false);
				}
				break;
				
			case ACTION_LOAD_GAME:
				Media.stop();
				Game.saveGame(); 
				Game.setGame(selectedGame);
				break;
				
			case ACTION_CONFIRM_RESET:
				event(ACTION_RESET, false);
				if (Game.TOURNAMENT == selectedGame) {
					GPCanvas.refIndexes[STATE_TOURNY][GPCanvas.WINDOW_INDEX] = 0;
					GPCanvas.refIndexes[STATE_TOURNY][GPCanvas.SELECTED_INDEX] = 0;
					event(STATE_TOURNY, false);
				} else {
					event(STATE_GAMESELECT, false);
				}
				break;
				
			case ACTION_RESET:
				Game.createNew();
				break;
				
			case ACTION_SAVE_SETTINGS:
				Game.saveGame();
				event(backTarget, true);
				break;
				
			case STATE_INTRO:
				screen.setMode(GPCanvas.MODE_STARTUP,NOCOMMANDS,null,null, NONE);
				break;
				
			case STATE_MAIN:
				screen.setMode(MenuScreen.MODE_STATIC_MENU,MAIN_CMDS,Strings.COMMANDS[MAINMENU],null, NONE);
				MenuScreen.items = new int[MAIN_MENU_ITEMS.length];
				System.arraycopy(MAIN_MENU_ITEMS,0,MenuScreen.items,0,MAIN_MENU_ITEMS.length);
				break;
				
			case STATE_MAIN_QUIT:
				String[] quitDialog = {Strings.STR_QUITQ,Strings.STR_SUREQ};
				screen.setMode(screen.MODE_DIALOG,MAIN_QUIT_CMDS,Strings.COMMANDS[QUIT],quitDialog,NONE);		
		    	break;
		    	
			case STATE_MAIN_CONNECT_ATTEMPT:
			case STATE_TOURNY_CONNECT_ATTEMPT:
				String[] connectDialog = {Strings.STR_CONNECTING,Strings.STR_PLEASEWAIT};
				screen.setMode(screen.MODE_DIALOG,NOCOMMANDS,Strings.COMMANDS[TOURNAMENT],connectDialog,NONE);
		    	break;
		    	
			case STATE_MAIN_CONNECT_FAILED:
				String[] failedDialog = {Strings.STR_FAILED,Strings.STR_FAILED2};
				MAIN_CONNECT_FAILED_CMDS[CMD_FORE] = ((RETRY<<8) | STATE_TOURNY);
				screen.setMode(screen.MODE_DIALOG,MAIN_CONNECT_FAILED_CMDS,Strings.COMMANDS[TOURNAMENT],failedDialog,NONE);
				break;
				
			case STATE_MAIN_CONNECT_UNAVAILABLE:
				String[] unavailableDialog = {Strings.STR_NOTOURNY1,Strings.STR_NOTOURNY2};
		    	MAIN_CONNECT_FAILED_CMDS[CMD_FORE] = NONE;
				screen.setMode(screen.MODE_DIALOG,MAIN_CONNECT_FAILED_CMDS,Strings.COMMANDS[TOURNAMENT],unavailableDialog,NONE);
				break;
				
			case STATE_CREDITS:
				screen.setMode(MenuScreen.MODE_TEXT,CREDITS_CMDS,Strings.COMMANDS[CREDITS],null, CREDITS);
				break;
				
			case STATE_ABOUT:
				screen.setMode(MenuScreen.MODE_TEXT,ABOUT_CMDS,Strings.COMMANDS[ABOUT],null, ABOUT);
				break;
				
			case STATE_OPTIONS_MAIN:
				String[] MAIN_OPTION_STRINGS = {
					Strings.BLANK,
					Strings.AV_OPTIONS[Game.avSetting],
				    Strings.BLANK,
					Strings.HELP_OPTIONS[Game.hSetting]};
				screen.setMode(MenuScreen.MODE_STATIC_MENU,MAIN_OPTION_CMDS,Strings.COMMANDS[OPTIONS],MAIN_OPTION_STRINGS, NONE);
				MenuScreen.items = new int[MAIN_OPTION_ITEMS.length];
				backTarget = STATE_MAIN;
				System.arraycopy(MAIN_OPTION_ITEMS,0,MenuScreen.items,0,MAIN_OPTION_ITEMS.length);
				break;
			
			case STATE_OPTIONS_TOURNY:
				String[] TOURNY_OPTION_STRINGS = {
					Strings.BLANK,
					Strings.AV_OPTIONS[Game.avSetting],
					Strings.BLANK};
				screen.setMode(MenuScreen.MODE_STATIC_MENU,TOURNY_OPTION_CMDS,Strings.COMMANDS[OPTIONS],TOURNY_OPTION_STRINGS, NONE);
				MenuScreen.items = new int[TOURNY_OPTION_ITEMS.length];
				backTarget = STATE_TOURNY;
				System.arraycopy(TOURNY_OPTION_ITEMS,0,MenuScreen.items,0,TOURNY_OPTION_ITEMS.length);
				break;
				
			case STATE_OPTIONS_GAME:
				String[] GAME_OPTION_STRINGS = {
					Strings.BLANK,
					Strings.AV_OPTIONS[Game.avSetting],
				    Strings.BLANK,
					Strings.HELP_OPTIONS[Game.hSetting],
					Strings.BLANK,
					Strings.BLANK};
				screen.setMode(MenuScreen.MODE_STATIC_MENU,GAME_OPTION_CMDS,Strings.COMMANDS[OPTIONS],GAME_OPTION_STRINGS, NONE);
				MenuScreen.items = new int[GAME_OPTION_ITEMS.length];
				backTarget = STATE_GAME_MENU;
				System.arraycopy(GAME_OPTION_ITEMS,0,MenuScreen.items,0,GAME_OPTION_ITEMS.length);
				break;
				
			case STATE_GAMESELECT:
				int[] startCommands = new int[GAMESELECT_CMDS.length];
				System.arraycopy(GAMESELECT_CMDS,0,startCommands,0,GAMESELECT_CMDS.length);
				if (!Game.gamePlayed[MenuScreen.refIndexes[STATE_GAMESELECT][MenuScreen.SELECTED_INDEX]]) {
        			startCommands[CMD_FORE] = ((VPHandler.PLAY<<8) | ACTION_SELECT_GAME);
        		} else {
        			startCommands[CMD_FORE] = ((VPHandler.RESUME<<8) | ACTION_SELECT_GAME);
        		}
				screen.setMode(MenuScreen.MODE_DYNAMIC_MENU,startCommands,Strings.COMMANDS[SELECTGAME],null, NONE);
				MenuScreen.items = new int[Game.NUM_GAMES];
				for (int gameIndex = 0; gameIndex < Game.GAME_LIST.length;gameIndex++) {
					MenuScreen.items[gameIndex] = (((Strings.COMMANDS.length+gameIndex)<<8) | ACTION_SELECT_GAME);
				}
				break;

			case STATE_INFO_PREGAME:
				VPHandler.selectedGame = MenuScreen.selectedIndex;
				screen.setMode(MenuScreen.MODE_TEXT,INFO_CMDS,Game.GAME_LIST[VPHandler.selectedGame],null, GAME+VPHandler.selectedGame);
				break;
				
				//#ifdef SLOW
			case STATE_GAME_LOADING:
				String gameName;
				if (selectedGame != Game.TOURNAMENT) {
					gameName = Game.GAME_LIST[selectedGame];
				} else {
					gameName = Game.GAME_LIST[TournInfo.nGame];
				}
				String[] loadingDialog = {Strings.STR_LOADINGGAME,gameName};
			    screen.setMode(screen.MODE_DIALOG,NOCOMMANDS,null,loadingDialog,NONE);
			    screen.showNotify();
			    screen.serviceRepaints();
				break;
				//#endif
				
			case STATE_GOTO_GAME_REGULAR:
				event(STATE_GAME_LOADING, false);
				if (Game.curGameType != selectedGame) event(ACTION_LOAD_GAME, false);	
				event(STATE_GAME_REGULAR, false);
				break;
				
			case STATE_GAME_GOTO_TOURNY:
				if (GPCanvas.curState != STATE_GOTO_ENDSTATS) event(STATE_GAME_LOADING, false);
				if (Game.gState == Game.STATE_ENDTOURNY) event(ACTION_RESET, false);
				event(STATE_GAME_TOURNY, false);
				break;
				
			case STATE_GAME_REGULAR:
				backTarget = STATE_GAME_REGULAR;
				screen.setMode(GPCanvas.MODE_GAME,REGULAR_GAME_CMDS,null,null,NONE);
				break;
				
			case STATE_GAME_TOURNY:
				if (Game.gState == Game.STATE_ENDTOURNY) {
					event(ACTION_RESET, false);
					event(STATE_GAME_TOURNY, false);
				} else {
					backTarget = STATE_GAME_TOURNY;
					int[] TOURNY_GAME_CMDS = {
						NONE,
						((BACK<<8) | STATE_TOURNY),
						STATE_INGAME_STRATEGY,
						STATE_INGAME_PAYOFF};
					screen.setMode(GPCanvas.MODE_GAME,TOURNY_GAME_CMDS,null,null,NONE);
				}
				break;
				
			case STATE_INGAME_PAYOFF:
				screen.setMode(screen.MODE_SHOW_PAYOFF,BACK_CMDS,null,null,NONE);
				break;
				
			case STATE_GAME_STRATEGY:
				backTarget = STATE_GAME_MENU;
			case STATE_INGAME_STRATEGY:
				screen.setMode(screen.MODE_STRATEGY,INGAME_STRATEGY_CMDS,null,null,NONE);
				break;
				
			case STATE_GAME_STRATEGY_GLOSSARY:
				screen.setMode(screen.MODE_GLOSSARY,GLOSSARY_CMDS,null,null,NONE);
				break;
				
			case STATE_GAME_MENU:
				screen.setMode(MenuScreen.MODE_STATIC_MENU,GAME_MENU_CMDS,Game.GAME_LIST[selectedGame],null, NONE);
				MenuScreen.items = new int[GAME_MENU_ITEMS.length];
				System.arraycopy(GAME_MENU_ITEMS,0,MenuScreen.items,0,GAME_MENU_ITEMS.length);
				backTarget = STATE_GAME_MENU;
				break;
				
			case STATE_GAME_INFO:
				screen.setMode(MenuScreen.MODE_TEXT,GAME_INFO_CMDS,Game.GAME_LIST[selectedGame],null, GAME + selectedGame);
				break;
				
			case STATE_OPTIONS_PAYOFFS:
				screen.setMode(screen.MODE_MOD_PAYOFF,OPTIONS_PAYOFF_CMDS,null,null,NONE);
				break;
				
			case STATE_TOURNY_STATS:
				backTarget = STATE_TOURNY;
				screen.setMode(screen.MODE_STATISTICS,BACK_CMDS,null,null,NONE);
				break;
				
			case ACTION_TOURNAMENT_OVER:
				int[] doneCommand = {NONE,((DONE<<8) | STATE_TOURNY_END)};
				screen.setMode(GPCanvas.MODE_CMD_MODIFY,doneCommand,null,null,NONE);
				break;
				
			case STATE_ENDGAME_STATS:
				screen.setMode(screen.MODE_STATISTICS,ENDSTATS_CMDS,null,null,NONE);
				break;
				
			case STATE_GAME_STATS:
				backTarget = STATE_GAME_MENU;
				screen.setMode(screen.MODE_STATISTICS,BACK_CMDS,null,null,NONE);
				break;
				
			case STATE_GAME_HELP:
				screen.setMode(MenuScreen.MODE_TEXT,HELP_CMDS,Strings.STR_HOWTOPLAY,null, HELP);
				break;
				
			case STATE_OPTIONS_RESET_DIALOG:
				String[] resetDialog = {Strings.STR_RESETQ,Strings.STR_SUREQ};
			    screen.setMode(screen.MODE_DIALOG,MAIN_RESET_CMDS,null,resetDialog,NONE);
				break;
				
			case STATE_TOURNY:
				if (!TournInfo.tournamentIsLoaded) {
					event(STATE_MAIN_CONNECT_ATTEMPT, false);
					event(ACTION_GET_CURRENT_TOURNAMENT, false);
				} else if (!TournInfo.activeTournament) {
					event(STATE_MAIN_CONNECT_UNAVAILABLE, false);
				} else if (Game.curGameType != Game.TOURNAMENT){
					selectedGame = Game.TOURNAMENT;
					event(ACTION_LOAD_GAME, false);
					event(STATE_TOURNY, false);
				} else {
					String[] TOURNY_DATA = {Strings.BLANK,
											Strings.BLANK,
											Strings.BLANK,
								            Strings.BLANK,
											Strings.BLANK,
											new String(Strings.STR_TNUM + TournInfo.nNumber),	
											new String(Game.GAME_LIST[TournInfo.nGame]),
											new String(Strings.STR_NTIME + (TournInfo.nTime/60) + Strings.SPACE + Strings.STR_MIN),
											new String(Strings.STR_NENDS + Strings.MONTH[TournInfo.nEndMonth-1] + Strings.SPACE + TournInfo.nEndDay+ Strings.COMMA + TournInfo.nEndYear),
											new String(Strings.STR_NHANDS + TournInfo.nMinHands)};
					screen.setMode(screen.MODE_STATIC_MENU,TOURNY_CMDS,Strings.COMMANDS[TOURNAMENT],TOURNY_DATA, NONE);
					int topLabel;
					if (Game.activeGame == NONE || !(selectedGame == Game.TOURNAMENT)) {
						topLabel = PLAY;
					} else {
						if (Game.gState == Game.STATE_PREGAME || 
							Game.gState == Game.STATE_ENDTOURNY) {
							topLabel = PLAY;
						} else {
							topLabel = RESUME;
						} 
					}
					int[] TOURNY_ITEMS = {((topLabel<<8) | STATE_GAME_GOTO_TOURNY),
							((OPTIONS<<8) | STATE_OPTIONS_TOURNY),
							((STATS<<8) | STATE_TOURNY_STATS),
							((GOTOCURLEADERS<<8) | STATE_HIGHSCORES_CURRENT),
							((GOTOPREVLEADERS<<8) | STATE_HIGHSCORES_PREV),
							NONE,NONE,NONE,NONE,NONE};
					MenuScreen.items = new int[TOURNY_ITEMS.length];
					System.arraycopy(TOURNY_ITEMS,0,MenuScreen.items,0,TOURNY_ITEMS.length);
				}
				break;
				
			case STATE_HIGHSCORES_CURRENT:
				if (!TournInfo.tournamentIsUpToDate) {
					event(STATE_TOURNY_CONNECT_ATTEMPT, false);
					event(ACTION_RELOAD_CURRENT_TOURNAMENT, false);
				} else {
					String[] highScoreList = new String[TournInfo.numScores];
					for (int n = 0; n < TournInfo.numScores; n++) {
						if (TournInfo.scoresName[n] != null) highScoreList[n] = new String((n+1)+"."+TournInfo.scoresName[n]+Strings.SPLIT_DELINIATOR + TournInfo.scores[n]);
					}
					int[] CUR_HIGHSCORE_CMDS = {
						NONE,
						((BACK<<8) | STATE_TOURNY),NONE};
					screen.setMode(screen.MODE_TEXT,CUR_HIGHSCORE_CMDS,Strings.STR_TNUM + (TournInfo.nNumber),highScoreList,NONE);						
				}
				break;
				
			case STATE_HIGHSCORES_PREV:
				if (!TournInfo.prevIsLoaded) {
					event(STATE_TOURNY_CONNECT_ATTEMPT, false);
					event(ACTION_GET_PREV_TOURNAMENT, false);
				} else {
					String[] prevScoreList = new String[TournInfo.numLastScores];
					for (int n = 0; n < TournInfo.numLastScores; n++) {
						if (TournInfo.lastScoresName[n] != null) prevScoreList[n] = new String((n+1)+"."+TournInfo.lastScoresName[n]+Strings.SPLIT_DELINIATOR + TournInfo.lastScores[n]);
					}
					int[] PREV_HIGHSCORE_CMDS = {
						NONE,
						((BACK<<8) | STATE_TOURNY)};
					screen.setMode(screen.MODE_TEXT,PREV_HIGHSCORE_CMDS,Strings.STR_TNUM + (TournInfo.nNumber-1),prevScoreList,NONE);
				} 
				break;
		    	
			case STATE_TOURNY_CONNECT_FAILED:	
				String[] tFailedDialog = {Strings.STR_FAILED,Strings.STR_FAILED2};
				int[] TOURNY_CONNECT_FAILED_CMDS = {
					((RETRY<<8) | STATE_HIGHSCORES_PREV),
					((CANCEL<<8) | STATE_TOURNY)};
				screen.setMode(screen.MODE_DIALOG,TOURNY_CONNECT_FAILED_CMDS,null,tFailedDialog,NONE);
		    	break;
			
			case STATE_TOURNY_END:
				Game.nSeconds = TournInfo.nTime;
				Calendar curDate = Calendar.getInstance(TimeZone.getTimeZone(Strings.STR_TIMEZONE));
				boolean exp = (curDate.get(Calendar.YEAR) >= TournInfo.nEndYear
						   && curDate.get(Calendar.MONTH) >= TournInfo.nEndMonth
						   && curDate.get(Calendar.DAY_OF_MONTH) > TournInfo.nEndDay);
				if (Game.nCoins > TournInfo.scores[9]
				    && Game.nHands >= TournInfo.nMinHands
					&& !exp) {
						event(STATE_GOTO_HIGHSCORE, false);
				} else {
					event(STATE_GOTO_ENDSTATS, false);
				}
				break;
				
			case STATE_GOTO_ENDSTATS:
				int[] END_CMDS = {
					((PLAY<<8) | STATE_GAME_GOTO_TOURNY),
					((BACK<<8) | STATE_TOURNY)};
				screen.setMode(screen.MODE_STATISTICS,END_CMDS,null,null,NONE);
				break;
				
			case STATE_GOTO_HIGHSCORE:
				highScore = new Form(Strings.STR_NEWHIGHSCORE);
				highScore.append(Strings.STR_ENTERNAME);
				highScore.append(new TextField(Strings.STR_NAME,Strings.BLANK,TournInfo.SCORENAME_LENGTH,TextField.ANY));
				highScore.addCommand(new Command(Strings.COMMANDS[DONE],Command.BACK,1));
				highScore.setCommandListener(me);
				display.setCurrent(highScore);
				break;
		}
		
		lastState = gotoState;
		
	}
	
	public static final int RECORD_HIGHSCORE_SIZE = 16;
	public static final int RECORD_HIGHSCORE_INDEX = 1;
	
	public void commandAction(Command arg0, Displayable d) {
		
		// Encode data into new 16 byte record
		byte[] highScoreData = new byte[RECORD_HIGHSCORE_SIZE];
		Form highScore = (Form)d;
		TextField nameField = (TextField)highScore.get(1); // index of textfield
		int nCoins = Game.nCoins;
		int nNumber = TournInfo.nNumber;
		char[] sName = nameField.getString().toCharArray();
		int byteIndex = 0;
		for (int c=0 ; c<4 ; byteIndex++,c++) highScoreData[byteIndex] = (byte)((nCoins>>>8*(3-c))&0x0ff);
		for (int c=0 ; c<4 ; byteIndex++,c++) highScoreData[byteIndex] = (byte)((nNumber>>>8*(3-c))&0x0ff);
		for (int c=0 ; c<sName.length ; byteIndex++,c++) highScoreData[byteIndex] = (byte)sName[c];
			
		// Append that data to RECORD_HIGHSCORE record 1;
		int startAtIndex = 0;
		byte[] curHighScoreRecord = new byte[0];
		try {
			RecordStore saveData = RecordStore.openRecordStore(Strings.RECORD_HIGHSCORE,true);
			startAtIndex = 0;	
			try {
				curHighScoreRecord = saveData.getRecord(RECORD_HIGHSCORE_INDEX);
				startAtIndex = curHighScoreRecord.length;
				byte[] newHighScoreRecord = new byte[startAtIndex + RECORD_HIGHSCORE_SIZE];
				System.arraycopy(curHighScoreRecord,0,newHighScoreRecord,0,startAtIndex);
				System.arraycopy(highScoreData,0,newHighScoreRecord,startAtIndex,RECORD_HIGHSCORE_SIZE);
				saveData.setRecord(RECORD_HIGHSCORE_INDEX,newHighScoreRecord,0,newHighScoreRecord.length);
				//#ifdef DEBUG
				System.out.println("highscore: encoding: " + ((curHighScoreRecord.length+2)/16) + " - added");
				for (int c = 0; c < 16 ; c++ ) System.out.println(highScoreData[c]);
				//#endif
			} catch (Exception needCreate) {
				//#ifdef DEBUG
				System.out.println("highscore: encoding: " + ((curHighScoreRecord.length+2)/16) + " created because " + needCreate);
				for (int c = 0; c < 16 ; c++ ) System.out.println(highScoreData[c]);
				//#endif

				saveData.addRecord(highScoreData,0,highScoreData.length);
			}
			saveData.closeRecordStore();
			
		} catch (Exception bad) {
			//#ifdef DEBUG
			System.out.println("highscore: encoding: " + bad);
			//#endif
		}
		TournInfo.tournamentIsUpToDate = false;
		event(STATE_GOTO_ENDSTATS, false);
		display.setCurrent(screen);
	}
}
