package genplay.vp.logi;

import genplay.util.Media;
import genplay.vp.game.Game;
import genplay.vp.core.*;

import java.io.*;
import java.util.Calendar;
import java.util.Random;
import java.util.TimeZone;

import javax.microedition.io.*;
import javax.microedition.rms.*;

public class TournInfo extends Thread {
	
	/*
	 * -------------------- Optional Tournament Information ------------------------------------------
	 * 
	 * Information related to online play
	 * 
	 */
	
	private static Random random = new Random();
		
	// Constants
	public static final int LOAD_CURRENT = 0;
	public static final int LOAD_PREV = 1;
	public static final int SAVE_SCORES = 2;
	public static final int LOAD_REFRESH = 3;
	
	//public static final String  TOURNAMENT_SERVER1="http://www.genplayMIDlets.com/vps";
	//public static final String  TOURNAMENT_SERVER2="http://www2.genplayMIDlets.com/vps";
	public static final String  TOURNAMENT_SERVER1="http://genplaygames.com/vps.dev";
	public static final String  TOURNAMENT_SERVER2="http://genplaygames.com/vps.dev";
	public static final String	TOURNAMENT_LOAD="/actions/GetTournamentText.php?";
	public static final String	TOURNAMENT_POST="/actions/PostScoreEncrypted.php?";
	public static final String KEY_PNUMBER="tournament_num=";
	public static final String KEY_PSCORE="score=";
	public static final String KEY_PNAME="username=";
	
	public static final String KEY_NNUMBER = "number=";
	public static final String KEY_NENDYEAR = "endYear=";
	public static final String KEY_NENDMONTH = "endMonth=";
	public static final String KEY_NENDDAY = "endDay=";
	public static final String KEY_NMINHANDS = "hands=";
	public static final String KEY_NTIME = "time=";
	public static final String KEY_NGAME = "game=";
	public static final String KEY_SCORE = "score";

	public static final int   SCORES_MAX=		10;		// max number of best results
	public static final int   SCORENAME_LENGTH=	8;		// length of player's name			

	public static final int 	INPUT_BUFFER_SIZE = 1024 * 50;

	// Registers
	public static boolean hasHighScore = false;
	public static boolean tournamentIsLoaded = false;
	public static boolean tournamentIsUpToDate = false;
	public static boolean prevIsLoaded = false;
	
	public static short		nNumber;				// number of tournament
	public static short		nMinHands;				// minimum number of hands required
	public static short		nEndYear;				// ending year 2004...
	public static byte 		nEndMonth;				// 1-12
	public static byte		nEndDay;				// 1-31
	public static byte		nGame;					// what type of MIDlet
	public static int		nTime;					// amount of MIDlet time allowed in seconds
	public static int 		numScores;
	public static String[] 	scoresName = new String[SCORES_MAX];
	public static int[]		scores = new int[SCORES_MAX];		// tournament scores 
	public static int 		numLastScores;
	public static String[] 	lastScoresName = new String[SCORES_MAX];
	public static int[] 	lastScores= new int[SCORES_MAX];
	
	private int type;
	private int succeedCmd;
	private int failedCmd;
	
	public TournInfo(int type,int succeedCmd,int failedCmd,VPHandler listener) {
		this.type = type;
		this.succeedCmd = succeedCmd;
		this.failedCmd = failedCmd;
	}
	
	public void run() {
		//#ifdef DEBUG
		try {
			//#endif
			try {// See if we have new highscore data to send
				
				// Get record data
				RecordStore highScoreRecord = RecordStore.openRecordStore(Strings.RECORD_HIGHSCORE,false);
				byte[] curHighScoreData = highScoreRecord.getRecord(VPHandler.RECORD_HIGHSCORE_INDEX);
				int numScores = curHighScoreData.length/VPHandler.RECORD_HIGHSCORE_SIZE;
				
				//#ifdef DEBUG
				System.out.println("Current size: " + curHighScoreData.length);
				//#endif
				
				// for each record- grab it, encrypt it, try to send it, delete if successful
				for (int scoreNum = 0; scoreNum < numScores; scoreNum++) {	
					
					// Get data
					byte[] data = new byte[VPHandler.RECORD_HIGHSCORE_SIZE];
					System.arraycopy(curHighScoreData,0,data,0,VPHandler.RECORD_HIGHSCORE_SIZE);
					int dataIndex = 0,stringIndex = 0;
					int nScore = 0;
					int nNumber = 0;
					char[] sName = new char[8];
					for (int c=0 ; c<4 ; c++,dataIndex++) nScore |= (data[dataIndex]&0x0ff)<<(8*(3-c));
					for (int c=0 ; c<4 ; c++,dataIndex++) nNumber |= (data[dataIndex]&0x0ff)<<(8*(3-c));
					for (stringIndex=0 ; (stringIndex<8 && data[dataIndex] != 0); stringIndex++,dataIndex++) {
						sName[stringIndex] = (char)data[dataIndex];
					}	
					
					//#ifdef DEBUG
					System.out.println(nScore + " " + nNumber + " " + String.valueOf(sName));
					//#endif
					
					// Build submission key
					String inVal = new String(KEY_PNUMBER + nNumber + "&" + KEY_PSCORE + nScore + "&" + KEY_PNAME + String.valueOf(sName,0,stringIndex));
					//#ifdef DEBUG
					System.out.println(inVal);
					//#endif
					
					// Encrypt submission key
					char[] source = inVal.toCharArray();
					char[] dest = new char[source.length*2+8];
					int seed = random.nextInt();
					int nextFactor;
					int sourceIndex,destIndex;
					byte hexVal,encodedByte;;
					nextFactor = seed * 0xB90D12B9 + 0x38CE051F;			// constants taken from Knuth 
					for (destIndex = 0; destIndex < 8; destIndex++) {		// include key in string
						hexVal = (byte)((nextFactor >>> ((7-destIndex) * 4)) & (byte)0x000f);
						dest[destIndex] = (char)(hexVal < 10 ? hexVal+'0' : hexVal-10+'A');
					}
					for (sourceIndex = 0; sourceIndex < source.length; sourceIndex++) { // process string
						encodedByte = (byte)(source[sourceIndex] ^ nextFactor);
						nextFactor = nextFactor * 0xB90D12B9 + 0x38CE051F;		// constants taken from Knuth	
						hexVal = (byte)((encodedByte>>>4) & (byte)0x000f);
						dest[destIndex++] = (char)(hexVal < 10 ? hexVal+'0' : hexVal-10+'A');
						hexVal = (byte)((encodedByte) & (byte)0x000f);
						dest[destIndex++] = (char)(hexVal < 10 ? hexVal+'0' : hexVal-10+'A');
					}
					String outData = String.valueOf(dest);
					
					// post the data to primary first, secondary if fails
					boolean sendSucceeded = false;
					if (getViaHttpConnection(TOURNAMENT_SERVER1+TOURNAMENT_POST+outData,SAVE_SCORES)) {
						sendSucceeded = true;
					} else if (getViaHttpConnection(TOURNAMENT_SERVER2+TOURNAMENT_POST+outData,SAVE_SCORES)) {
						sendSucceeded = true;
					}
					 
					// if we suceeded, create a new highScoreData ommitting the first record (that we just used)
					if (sendSucceeded) {	
						byte[] tmpHighScoreData = new byte[curHighScoreData.length - VPHandler.RECORD_HIGHSCORE_SIZE];
						System.arraycopy(curHighScoreData,VPHandler.RECORD_HIGHSCORE_SIZE,tmpHighScoreData,0,tmpHighScoreData.length);
						curHighScoreData = new byte[tmpHighScoreData.length];
						System.arraycopy(tmpHighScoreData,0,curHighScoreData,0,tmpHighScoreData.length);
					}					
					
					//#ifdef DEBUG
					System.out.println("Record Complete");
					//#endif
				}
				
				// Clean up
				highScoreRecord.setRecord(VPHandler.RECORD_HIGHSCORE_INDEX,curHighScoreData,0,curHighScoreData.length);
				highScoreRecord.closeRecordStore();
				System.gc();
				
				//#ifdef DEBUG
				System.out.println("DONE WITH HIGHSCORE SUBMISSION");
			} catch (RecordStoreNotFoundException e) {
				
				System.out.println("highscore: submit: NOTICE: Nothing to Submit");
				//#endif
			} catch (Exception e) {
				//#ifdef DEBUG
				System.out.println("TournInfo: submit: " + e);
				e.printStackTrace();
				//#endif
			}

			String queryURL;
			switch (type) {
				case LOAD_PREV:
					queryURL = TOURNAMENT_LOAD+KEY_PNUMBER+(nNumber-1);
					//#ifdef SPRINT
					//#endif
					break;
				default:
					queryURL = TOURNAMENT_LOAD;
					//#ifdef SPRINT
					//#endif
			}
			if (getViaHttpConnection(TOURNAMENT_SERVER1+queryURL,type)) {
				VPHandler.event(succeedCmd, true);
			} else if (getViaHttpConnection(TOURNAMENT_SERVER2+queryURL,type)) {
				VPHandler.event(succeedCmd, true);
			} else {
				VPHandler.event(failedCmd, true);
			}
	    //#ifdef DEBUG
		} catch (Exception e) {
			//System.out.println("Tournament: Get: "+ e);
		}
		//#endif
	}
	
	public static boolean activeTournament;
	
	private boolean getViaHttpConnection(String url,int type) {
		
		//#ifdef DEBUG
		System.out.println("NET: " + url);
		//#endif

        try {
        	String iBuffer = new String();
        	HttpConnection c = null;
            InputStream is = null;
            DataInputStream reader = null;
        	
            c = (HttpConnection)Connector.open(url);

            // Connect To Server
            int rc = c.getResponseCode();
            if (rc != HttpConnection.HTTP_OK) {
            	//#ifdef DEBUG
            	System.out.println("NET: BAD RESPONSE: " + c.getResponseMessage());
            	//#endif
                return false;
            }

            is = c.openInputStream();
            reader = new DataInputStream(is);
            
            // Physically read Input Data
            boolean done = false;
            while (!done) {
            	try {
            		iBuffer += (char)reader.readByte();
            	} catch (EOFException e) {
            		done = true;
            	}
            }
            
            c.close();
            is.close();
            reader.close();
            System.gc();
            
            //#ifdef DEBUG
            System.out.println("NET: GOT: " + iBuffer);
            //#endif
            
            if (type == SAVE_SCORES) {
                if (iBuffer.indexOf("OK") != 0) {
                	return true;
                } else {
                	return false;
                }
            } else {
		        
                
                short		number;				// number of tournament
            	short		minHands;				// minimum number of hands required
            	short		endYear;				// ending year 2004...
            	byte 		endMonth;				// 1-12
            	byte		endDay;				// 1-31
            	byte		game;					// what type of MIDlet
            	int			time;					// amount of MIDlet time allowed in seconds
            	int			scoreCount=0;
            	String[] 	name = new String[SCORES_MAX];
            	int[]		score = new int[SCORES_MAX];		// tournament scores 
				int index;
				
				done = false;
				
				index = iBuffer.indexOf(KEY_NNUMBER);
				if (index >= 0) {
					number = Short.parseShort(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
				} else return false;
		
				index = iBuffer.indexOf(KEY_NENDYEAR);
				if (index >= 0) {
					endYear = Short.parseShort(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
				} else return false;
		
				index = iBuffer.indexOf(KEY_NENDMONTH);
				if (index >= 0) {
					endMonth = Byte.parseByte(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
				} else return false;
				
				index = iBuffer.indexOf(KEY_NENDDAY);
				if (index >= 0) {
					endDay = Byte.parseByte(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
				} else return false;
				
				index = iBuffer.indexOf(KEY_NMINHANDS);
				if (index >= 0) {
					minHands = Short.parseShort(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
				} else return false;
		
				index = iBuffer.indexOf(KEY_NTIME);
				if (index >= 0) {
					time = Integer.parseInt(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
				} else return false;
		
				index = iBuffer.indexOf(KEY_NGAME);
				if (index >= 0) {
					try {
						game = Byte.parseByte(iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1)));
					} catch (Exception e) {
						game = Byte.parseByte(iBuffer.substring(iBuffer.indexOf("=",index)+1));
						done = true;
					}
				} else return false;
		
				if (!done) {
					String key;
					String value;
		    		for (int sNum = 1; sNum <= SCORES_MAX; sNum++) {
		    			key = new String(KEY_SCORE+sNum+"=");
		    			index = iBuffer.indexOf(key);
		    			if (index >= 0) {
		    				int endOfKey = iBuffer.indexOf(",",index+1);
		    				if (endOfKey != -1) {
		    					value = iBuffer.substring(iBuffer.indexOf("=",index)+1,iBuffer.indexOf(",",index+1));
		    				} else { 
		    					// Note, this _WILL_ break if there is any crap after the last score key
		    					// That is beyond the scope of the API, however, so I believe that its a
		    					// Safe assumption to make
		    					value = iBuffer.substring(iBuffer.indexOf("=",index)+1);
		    				}
		    				score[sNum-1] = Integer.parseInt(value.substring(0,value.indexOf(":")));
		    				name[sNum-1] = value.substring(value.indexOf(":")+1);
		    				scoreCount++;
		    			}
		    		}
				} 
    		
	    		switch (type) {
	    		case LOAD_CURRENT:
	    			if (game < Game.NUM_GAMES &&
	    				minHands >= 0 &&
						endYear > 2004 &&
						0 < endMonth && endMonth <= 12 &&
						endDay < 31 &&
						60 <= time) {	
	    				
		    			nNumber = number;				// number of tournament
		    			nMinHands = minHands;				// minimum number of hands required
		    			nEndYear = endYear;				// ending year 2004...
		    			nEndMonth = (byte)(endMonth-1);				// 1-12
		    			nEndDay = endDay;			// 1-31
		    			nGame = game;					// what type of MIDlet
		    			nTime = time;					// amount of MIDlet time allowed in seconds
		    			numScores = scoreCount;
		    			System.arraycopy(name,0,scoresName,0,name.length);
		    			System.arraycopy(score,0,scores,0,score.length);
				    	Calendar date = Calendar.getInstance(TimeZone.getTimeZone(Strings.STR_TIMEZONE));
						boolean expired = false;
						expired = (date.get(Calendar.YEAR) >= TournInfo.nEndYear
							   && date.get(Calendar.MONTH) >= TournInfo.nEndMonth
							   && date.get(Calendar.DAY_OF_MONTH) > TournInfo.nEndDay);
						activeTournament = !expired;
		    			tournamentIsLoaded = true;
		    			tournamentIsUpToDate = true;
	    			} else {
	    				activeTournament = false;
		    			tournamentIsLoaded = true;
		    			tournamentIsUpToDate = true;
	    			}
	    			break;
	    		case LOAD_PREV:
	    			numLastScores = scoreCount;
	    			System.arraycopy(name,0,lastScoresName,0,name.length);
	    			System.arraycopy(score,0,lastScores,0,score.length);
	    			prevIsLoaded = true;
	    			break;
				case LOAD_REFRESH:
					System.arraycopy(name,0,scoresName,0,name.length);
	    			System.arraycopy(score,0,scores,0,score.length);
	    			tournamentIsUpToDate = true;
	    		}

	    		return true;
            }
        } catch (Exception e) {
        	//#ifdef DEBUG
        	System.out.println("NET: Exception: " + e.getMessage());
        	//#endif
        } 
        return false;
	}
}